@@ -1,29 +1,34 @@
 use std::{env, fmt};
 
-use serde::de::DeserializeOwned;
-use serde::Serialize;
+use serde::{de::DeserializeOwned, Serialize};
 
 use rpki::uri;
 
-#[cfg(feature = "multi-user")]
-use crate::cli::options::KrillUserDetails;
-use crate::cli::report::{ApiResponse, ReportError};
-use crate::commons::api::{
-    AllCertAuthIssues, CaRepoDetails, CertAuthIssues, ChildCaInfo, ParentCaContact, ParentStatuses, PublisherDetails,
-    PublisherList, RepoStatus, Token,
-};
-use crate::commons::bgp::BgpAnalysisAdvice;
-use crate::commons::remote::rfc8183;
-use crate::commons::util::{file, httpclient};
-use crate::constants::KRILL_CLI_API_ENV;
-use crate::daemon::config::Config;
 use crate::{
-    cli::options::{BulkCaCommand, CaCommand, Command, KrillInitDetails, Options, PubServerCommand},
-    commons::error::KrillIoError,
+    cli::{
+        options::{BulkCaCommand, CaCommand, Command, KrillInitDetails, Options, PubServerCommand},
+        report::{ApiResponse, ReportError},
+    },
+    commons::{
+        api::{
+            AllCertAuthIssues, AspaDefinitionUpdates, CaRepoDetails, CertAuthIssues, ChildCaInfo,
+            ChildrenConnectionStats, ParentCaContact, ParentStatuses, PublisherDetails, PublisherList, RepoStatus,
+            Token,
+        },
+        bgp::BgpAnalysisAdvice,
+        error::KrillIoError,
+        remote::rfc8183,
+        util::{file, httpclient},
+    },
+    constants::KRILL_CLI_API_ENV,
+    daemon::config::Config,
 };
 
 #[cfg(feature = "multi-user")]
-use crate::constants::{PW_HASH_LOG_N, PW_HASH_P, PW_HASH_R};
+use crate::{
+    cli::options::KrillUserDetails,
+    constants::{PW_HASH_LOG_N, PW_HASH_P, PW_HASH_R},
+};
 
 fn resolve_uri(server: &uri::Https, path: &str) -> String {
     format!("{}{}", server, path)
@@ -237,6 +242,12 @@ impl KrillClient {
                 Ok(ApiResponse::ParentCaContact(parent))
             }
 
+            CaCommand::Refresh(handle) => {
+                let uri = format!("api/v1/cas/{}/sync/parents", handle);
+                post_empty(&self.server, &self.token, &uri).await?;
+                Ok(ApiResponse::Empty)
+            }
+
             CaCommand::ChildInfo(handle, child) => {
                 let uri = format!("api/v1/cas/{}/children/{}", handle, child);
                 let info: ChildCaInfo = get_json(&self.server, &self.token, &uri).await?;
@@ -258,6 +269,11 @@ impl KrillClient {
                 delete(&self.server, &self.token, &uri).await?;
                 Ok(ApiResponse::Empty)
             }
+            CaCommand::ChildConnections(handle) => {
+                let uri = format!("api/v1/cas/{}/stats/children/connections", handle);
+                let stats: ChildrenConnectionStats = get_json(&self.server, &self.token, &uri).await?;
+                Ok(ApiResponse::ChildrenStats(stats))
+            }
 
             CaCommand::KeyRollInit(handle) => {
                 let uri = format!("api/v1/cas/{}/keys/roll_init", handle);
@@ -316,6 +332,32 @@ impl KrillClient {
                 Ok(ApiResponse::BgpAnalysisSuggestions(suggestions))
             }
 
+            CaCommand::AspasList(handle) => {
+                let uri = format!("api/v1/cas/{}/aspas", handle);
+                let aspas = get_json(&self.server, &self.token, &uri).await?;
+                Ok(ApiResponse::AspaDefinitions(aspas))
+            }
+
+            CaCommand::AspasAddOrReplace(handle, aspa) => {
+                let uri = format!("api/v1/cas/{}/aspas", handle);
+                let updates = AspaDefinitionUpdates::new(vec![aspa], vec![]);
+                post_json(&self.server, &self.token, &uri, updates).await?;
+                Ok(ApiResponse::Empty)
+            }
+
+            CaCommand::AspasRemove(handle, customer) => {
+                let uri = format!("api/v1/cas/{}/aspas", handle);
+                let updates = AspaDefinitionUpdates::new(vec![], vec![customer]);
+                post_json(&self.server, &self.token, &uri, updates).await?;
+                Ok(ApiResponse::Empty)
+            }
+
+            CaCommand::AspasUpdate(handle, customer, update) => {
+                let uri = format!("api/v1/cas/{}/aspas/as/{}", handle, customer);
+                post_json(&self.server, &self.token, &uri, update).await?;
+                Ok(ApiResponse::Empty)
+            }
+
             CaCommand::Show(handle) => {
                 let uri = format!("api/v1/cas/{}", handle);
                 let ca_info = get_json(&self.server, &self.token, &uri).await?;
@@ -483,7 +525,7 @@ impl KrillClient {
     }
 
     #[cfg(feature = "multi-user")]
-    #[allow(clippy::clippy::unnecessary_wraps)]
+    #[allow(clippy::unnecessary_wraps)]
     fn user(&self, details: KrillUserDetails) -> Result<ApiResponse, Error> {
         let (password_hash, salt) = {
             use scrypt::scrypt;
