@@ -1,34 +1,41 @@
 #[cfg(feature = "multi-user")]
 use std::collections::HashMap;
 
-use std::convert::TryFrom;
-use std::path::PathBuf;
-use std::str::{from_utf8_unchecked, FromStr};
-use std::{env, fmt};
+use std::{
+    convert::TryFrom,
+    path::PathBuf,
+    str::{from_utf8_unchecked, FromStr},
+    {env, fmt},
+};
 
 use bytes::Bytes;
 use clap::{App, Arg, ArgMatches, SubCommand};
 
-use rpki::crypto::KeyIdentifier;
-use rpki::uri;
-use rpki::x509::Time;
-
-use crate::commons::crypto::{IdCert, SignSupport};
-use crate::commons::remote::rfc8183;
-use crate::commons::util::file;
-use crate::commons::{
-    api::{
-        AddChildRequest, AuthorizationFmtError, CertAuthInit, ChildHandle, Handle, ParentCaContact, ParentCaReq,
-        ParentHandle, PublicationServerUris, PublisherHandle, ResourceSet, ResourceSetError, RoaDefinition,
-        RoaDefinitionUpdates, RtaName, Token, UpdateChildRequest,
+use rpki::{
+    repository::{
+        aspa::{DuplicateProviderAs, ProviderAs},
+        crypto::KeyIdentifier,
+        x509::Time,
     },
-    error::KrillIoError,
+    uri,
 };
-use crate::constants::*;
-use crate::daemon::ca::{ResourceTaggedAttestation, RtaContentRequest, RtaPrepareRequest};
+
 use crate::{
     cli::report::{ReportError, ReportFormat},
-    commons::api::RepositoryContact,
+    commons::{
+        api::{
+            AddChildRequest, AspaCustomer, AspaDefinition, AspaDefinitionFormatError, AspaProvidersUpdate,
+            AuthorizationFmtError, CertAuthInit, ChildHandle, Handle, ParentCaContact, ParentCaReq, ParentHandle,
+            PublicationServerUris, PublisherHandle, RepositoryContact, ResourceSet, ResourceSetError, RoaDefinition,
+            RoaDefinitionUpdates, RtaName, Token, UpdateChildRequest,
+        },
+        crypto::{IdCert, SignSupport},
+        error::KrillIoError,
+        remote::rfc8183,
+        util::file,
+    },
+    constants::*,
+    daemon::ca::{ResourceTaggedAttestation, RtaContentRequest, RtaPrepareRequest},
 };
 
 struct GeneralArgs {
@@ -469,6 +476,36 @@ impl Options {
         app.subcommand(sub)
     }
 
+    fn make_cas_children_connections_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub = SubCommand::with_name("connections").about("Show connections stats for children of a CA");
+
+        sub = Self::add_general_args(sub);
+        sub = Self::add_my_ca_arg(sub);
+
+        app.subcommand(sub)
+    }
+
+    fn make_cas_children_suspend_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub = SubCommand::with_name("suspend").about("Suspend a child CA: hide certificate(s) issued to child");
+
+        sub = Self::add_general_args(sub);
+        sub = Self::add_my_ca_arg(sub);
+        sub = Self::add_child_arg(sub);
+
+        app.subcommand(sub)
+    }
+
+    fn make_cas_children_unsuspend_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub =
+            SubCommand::with_name("unsuspend").about("Suspend a child CA: republish certificate(s) issued to child");
+
+        sub = Self::add_general_args(sub);
+        sub = Self::add_my_ca_arg(sub);
+        sub = Self::add_child_arg(sub);
+
+        app.subcommand(sub)
+    }
+
     fn make_cas_children_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
         let mut sub = SubCommand::with_name("children").about("Manage children for a CA");
 
@@ -477,6 +514,9 @@ impl Options {
         sub = Self::make_cas_children_info_sc(sub);
         sub = Self::make_cas_children_remove_sc(sub);
         sub = Self::make_cas_children_response_sc(sub);
+        sub = Self::make_cas_children_connections_sc(sub);
+        sub = Self::make_cas_children_suspend_sc(sub);
+        sub = Self::make_cas_children_unsuspend_sc(sub);
 
         app.subcommand(sub)
     }
@@ -697,6 +737,100 @@ impl Options {
         app.subcommand(sub)
     }
 
+    #[cfg(feature = "aspa")]
+    fn make_cas_aspas_add_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub = SubCommand::with_name("add").about("Add or replace an ASPA configuration");
+
+        sub = Self::add_general_args(sub);
+        sub = Self::add_my_ca_arg(sub);
+
+        sub = sub.arg(
+            Arg::with_name("aspa")
+                .long("aspa")
+                .help("ASPA formatted like: 65000 => 65001, 65002(v4), 65003(v6)")
+                .value_name("definition")
+                .required(true),
+        );
+
+        app.subcommand(sub)
+    }
+
+    #[cfg(feature = "aspa")]
+    fn make_cas_aspas_remove_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub = SubCommand::with_name("remove").about("Remove the ASPA for a customer ASN");
+
+        sub = Self::add_general_args(sub);
+        sub = Self::add_my_ca_arg(sub);
+
+        sub = sub.arg(
+            Arg::with_name("customer")
+                .long("customer")
+                .help("Customer ASN for an existing ASPA definition")
+                .value_name("ASN")
+                .required(true),
+        );
+
+        app.subcommand(sub)
+    }
+
+    #[cfg(feature = "aspa")]
+    fn make_cas_aspas_update_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub = SubCommand::with_name("update").about("Update an existing ASPA configuration");
+
+        sub = Self::add_general_args(sub);
+        sub = Self::add_my_ca_arg(sub);
+
+        sub = sub.arg(
+            Arg::with_name("customer")
+                .long("customer")
+                .help("Customer ASN for an existing ASPA definition")
+                .value_name("ASN")
+                .required(true),
+        );
+
+        sub = sub.arg(
+            Arg::with_name("add")
+                .long("add")
+                .help("Provider ASNs to add (multiple allowed)")
+                .value_name("<Provider AS>")
+                .multiple(true)
+                .required(false),
+        );
+
+        sub = sub.arg(
+            Arg::with_name("remove")
+                .long("remove")
+                .help("Provider ASNs to remove (multiple allowed)")
+                .value_name("<Provider AS>")
+                .multiple(true)
+                .required(false),
+        );
+
+        app.subcommand(sub)
+    }
+
+    #[cfg(feature = "aspa")]
+    fn make_cas_aspas_list_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub = SubCommand::with_name("list").about("Show current ASPA configurations");
+
+        sub = Self::add_general_args(sub);
+        sub = Self::add_my_ca_arg(sub);
+
+        app.subcommand(sub)
+    }
+
+    #[cfg(feature = "aspa")]
+    fn make_cas_aspas_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
+        let mut sub = SubCommand::with_name("aspas").about("Manage ASPAs for a CA (experimental)");
+
+        sub = Self::make_cas_aspas_add_sc(sub);
+        sub = Self::make_cas_aspas_remove_sc(sub);
+        sub = Self::make_cas_aspas_update_sc(sub);
+        sub = Self::make_cas_aspas_list_sc(sub);
+
+        app.subcommand(sub)
+    }
+
     fn make_cas_repo_request_sc<'a, 'b>(app: App<'a, 'b>) -> App<'a, 'b> {
         let mut sub = SubCommand::with_name("request").about("Show RFC8183 Publisher Request XML");
 
@@ -1127,6 +1261,11 @@ impl Options {
         app = Self::make_cas_issues_sc(app);
         app = Self::make_pubserver_sc(app);
 
+        #[cfg(feature = "aspa")]
+        {
+            app = Self::make_cas_aspas_sc(app);
+        }
+
         #[cfg(feature = "rta")]
         {
             app = Self::make_cas_rta_sc(app);
@@ -1362,7 +1501,7 @@ impl Options {
         let id_cert = {
             if let Some(path) = matches.value_of("idcert") {
                 let bytes = Self::read_file_arg(path)?;
-                let id_cert = IdCert::decode(bytes).map_err(|_| Error::InvalidChildIdCert)?;
+                let id_cert = IdCert::decode(bytes.as_ref()).map_err(|_| Error::InvalidChildIdCert)?;
                 Some(id_cert)
             } else {
                 None
@@ -1370,7 +1509,7 @@ impl Options {
         };
         let resources = Self::parse_resource_args(matches)?;
 
-        let update = UpdateChildRequest::new(id_cert, resources);
+        let update = UpdateChildRequest::new(id_cert, resources, None);
 
         let command = Command::CertAuth(CaCommand::ChildUpdate(my_ca, child, update));
         Ok(Options::make(general_args, command))
@@ -1409,6 +1548,40 @@ impl Options {
         Ok(Options::make(general_args, command))
     }
 
+    fn parse_matches_cas_children_connections(matches: &ArgMatches) -> Result<Options, Error> {
+        let general_args = GeneralArgs::from_matches(matches)?;
+        let my_ca = Self::parse_my_ca(matches)?;
+
+        let command = Command::CertAuth(CaCommand::ChildConnections(my_ca));
+        Ok(Options::make(general_args, command))
+    }
+
+    fn parse_matches_cas_children_suspend(matches: &ArgMatches) -> Result<Options, Error> {
+        let general_args = GeneralArgs::from_matches(matches)?;
+        let my_ca = Self::parse_my_ca(matches)?;
+
+        let child = matches.value_of("child").unwrap();
+        let child = Handle::from_str(child).map_err(|_| Error::InvalidHandle)?;
+
+        let update = UpdateChildRequest::suspend();
+
+        let command = Command::CertAuth(CaCommand::ChildUpdate(my_ca, child, update));
+        Ok(Options::make(general_args, command))
+    }
+
+    fn parse_matches_cas_children_unsuspend(matches: &ArgMatches) -> Result<Options, Error> {
+        let general_args = GeneralArgs::from_matches(matches)?;
+        let my_ca = Self::parse_my_ca(matches)?;
+
+        let child = matches.value_of("child").unwrap();
+        let child = Handle::from_str(child).map_err(|_| Error::InvalidHandle)?;
+
+        let update = UpdateChildRequest::unsuspend();
+
+        let command = Command::CertAuth(CaCommand::ChildUpdate(my_ca, child, update));
+        Ok(Options::make(general_args, command))
+    }
+
     fn parse_matches_cas_children(matches: &ArgMatches) -> Result<Options, Error> {
         if let Some(m) = matches.subcommand_matches("add") {
             Self::parse_matches_cas_children_add(m)
@@ -1420,6 +1593,12 @@ impl Options {
             Self::parse_matches_cas_children_update(m)
         } else if let Some(m) = matches.subcommand_matches("remove") {
             Self::parse_matches_cas_children_remove(m)
+        } else if let Some(m) = matches.subcommand_matches("connections") {
+            Self::parse_matches_cas_children_connections(m)
+        } else if let Some(m) = matches.subcommand_matches("suspend") {
+            Self::parse_matches_cas_children_suspend(m)
+        } else if let Some(m) = matches.subcommand_matches("unsuspend") {
+            Self::parse_matches_cas_children_unsuspend(m)
         } else {
             Err(Error::UnrecognizedSubCommand)
         }
@@ -1635,6 +1814,92 @@ impl Options {
         }
     }
 
+    fn parse_matches_cas_aspas_add(matches: &ArgMatches) -> Result<Options, Error> {
+        let general_args = GeneralArgs::from_matches(matches)?;
+        let my_ca = Self::parse_my_ca(matches)?;
+
+        let aspa_config_str = matches.value_of("aspa").unwrap(); // required argument
+        let aspa = AspaDefinition::from_str(aspa_config_str)?;
+
+        let command = Command::CertAuth(CaCommand::AspasAddOrReplace(my_ca, aspa));
+
+        Ok(Options::make(general_args, command))
+    }
+
+    fn parse_matches_cas_aspas_remove(matches: &ArgMatches) -> Result<Options, Error> {
+        let general_args = GeneralArgs::from_matches(matches)?;
+        let my_ca = Self::parse_my_ca(matches)?;
+        let customer_str = matches.value_of("customer").unwrap();
+        let customer = AspaCustomer::from_str(customer_str).map_err(|_| Error::invalid_asn(customer_str))?;
+
+        let command = Command::CertAuth(CaCommand::AspasRemove(my_ca, customer));
+
+        Ok(Options::make(general_args, command))
+    }
+
+    fn parse_matches_cas_aspas_update(matches: &ArgMatches) -> Result<Options, Error> {
+        let general_args = GeneralArgs::from_matches(matches)?;
+        let my_ca = Self::parse_my_ca(matches)?;
+
+        let mut added = vec![];
+        let mut removed = vec![];
+
+        let customer_str = matches.value_of("customer").unwrap();
+        let customer = AspaCustomer::from_str(customer_str).map_err(|_| Error::invalid_asn(customer_str))?;
+
+        if let Some(add) = matches.values_of("add") {
+            for provider_str in add {
+                let provider = ProviderAs::from_str(provider_str).map_err(|_| Error::invalid_asn(provider_str))?;
+                added.push(provider);
+            }
+        }
+
+        if let Some(remove) = matches.values_of("remove") {
+            for provider_as_str in remove {
+                let provider_as =
+                    ProviderAs::from_str(provider_as_str).map_err(|_| Error::invalid_asn(provider_as_str))?;
+
+                if added.iter().any(|added| added.provider() == provider_as.provider()) {
+                    return Err(Error::general("Do not add and remove the same AS in a single update."));
+                }
+
+                removed.push(provider_as);
+            }
+        }
+
+        let update = AspaProvidersUpdate::new(added, removed);
+        if update.is_empty() {
+            return Err(Error::general("You MUST specify at least one of --add or --remove"));
+        }
+
+        let command = Command::CertAuth(CaCommand::AspasUpdate(my_ca, customer, update));
+
+        Ok(Options::make(general_args, command))
+    }
+
+    fn parse_matches_cas_aspas_list(matches: &ArgMatches) -> Result<Options, Error> {
+        let general_args = GeneralArgs::from_matches(matches)?;
+        let my_ca = Self::parse_my_ca(matches)?;
+
+        let command = Command::CertAuth(CaCommand::AspasList(my_ca));
+
+        Ok(Options::make(general_args, command))
+    }
+
+    fn parse_matches_cas_aspas(matches: &ArgMatches) -> Result<Options, Error> {
+        if let Some(m) = matches.subcommand_matches("add") {
+            Self::parse_matches_cas_aspas_add(m)
+        } else if let Some(m) = matches.subcommand_matches("remove") {
+            Self::parse_matches_cas_aspas_remove(m)
+        } else if let Some(m) = matches.subcommand_matches("update") {
+            Self::parse_matches_cas_aspas_update(m)
+        } else if let Some(m) = matches.subcommand_matches("list") {
+            Self::parse_matches_cas_aspas_list(m)
+        } else {
+            Err(Error::UnrecognizedSubCommand)
+        }
+    }
+
     fn parse_matches_cas_repo_request(matches: &ArgMatches) -> Result<Options, Error> {
         let general_args = GeneralArgs::from_matches(matches)?;
         let my_ca = Self::parse_my_ca(matches)?;
@@ -2022,6 +2287,8 @@ impl Options {
             Self::parse_matches_cas_keyroll(m)
         } else if let Some(m) = matches.subcommand_matches("roas") {
             Self::parse_matches_cas_routes(m)
+        } else if let Some(m) = matches.subcommand_matches("aspas") {
+            Self::parse_matches_cas_aspas(m)
         } else if let Some(m) = matches.subcommand_matches("repo") {
             Self::parse_matches_cas_repo(m)
         } else if let Some(m) = matches.subcommand_matches("issues") {
@@ -2080,6 +2347,7 @@ pub enum CaCommand {
     MyParentCaContact(Handle, ParentHandle),
     ParentStatuses(Handle),
     RemoveParent(Handle, ParentHandle),
+    Refresh(Handle), // Refresh with all parents
 
     // Children
     ParentResponse(Handle, ChildHandle), // Get an RFC8183 parent response for a child
@@ -2087,6 +2355,7 @@ pub enum CaCommand {
     ChildAdd(Handle, AddChildRequest),
     ChildUpdate(Handle, ChildHandle, UpdateChildRequest),
     ChildDelete(Handle, ChildHandle),
+    ChildConnections(Handle),
 
     // Key Management
     KeyRollInit(Handle),
@@ -2100,6 +2369,12 @@ pub enum CaCommand {
     BgpAnalysisFull(Handle),
     BgpAnalysisSuggest(Handle, Option<ResourceSet>),
 
+    // ASPAs
+    AspasList(Handle),
+    AspasAddOrReplace(Handle, AspaDefinition),
+    AspasUpdate(Handle, AspaCustomer, AspaProvidersUpdate),
+    AspasRemove(Handle, AspaCustomer),
+
     // Show details for this CA
     Show(Handle),
     ShowHistoryCommands(Handle, HistoryOptions),
@@ -2265,6 +2540,9 @@ pub enum Error {
     Rfc8183(rfc8183::Error),
     ResSetErr(ResourceSetError),
     InvalidRouteDelta(AuthorizationFmtError),
+    InvalidAsn(String),
+    DuplicateAspaProvider(DuplicateProviderAs),
+    InvalidAspaConfig(AspaDefinitionFormatError),
     InvalidHandle,
     InvalidSeconds,
     MissingArgWithEnv(String, String),
@@ -2274,6 +2552,12 @@ pub enum Error {
     GeneralArgumentError(String),
 }
 
+impl Error {
+    fn invalid_asn(asn: &str) -> Self {
+        Error::InvalidAsn(asn.to_string())
+    }
+}
+
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -2283,6 +2567,9 @@ impl fmt::Display for Error {
             Error::Rfc8183(e) => write!(f, "Invalid RFC8183 XML: {}", e),
             Error::ResSetErr(e) => write!(f, "Invalid resources requested: {}", e),
             Error::InvalidRouteDelta(e) => e.fmt(f),
+            Error::InvalidAsn(s) => write!(f, "Invalid ASN format. Expected 'AS#', got: {}", s),
+            Error::DuplicateAspaProvider(e) => e.fmt(f),
+            Error::InvalidAspaConfig(e) => e.fmt(f),
             Error::InvalidHandle => write!(
                 f,
                 "The publisher handle may only contain -_A-Za-z0-9, (\\ /) see issue #83"
@@ -2346,3 +2633,15 @@ impl From<AuthorizationFmtError> for Error {
         Error::InvalidRouteDelta(e)
     }
 }
+
+impl From<AspaDefinitionFormatError> for Error {
+    fn from(e: AspaDefinitionFormatError) -> Self {
+        Error::InvalidAspaConfig(e)
+    }
+}
+
+impl From<DuplicateProviderAs> for Error {
+    fn from(e: DuplicateProviderAs) -> Self {
+        Error::DuplicateAspaProvider(e)
+    }
+}
