@@ -1,29 +1,35 @@
 //! Defines all Krill server side errors
 
-use std::fmt::Display;
-use std::{fmt, io};
+use std::{fmt, fmt::Display, io};
 
 use hyper::StatusCode;
 
-use rpki::crypto::KeyIdentifier;
-use rpki::uri;
-use rpki::x509::ValidationError;
-
-use crate::commons::api::rrdp::PublicationDeltaError;
-use crate::commons::api::{
-    ChildHandle, ErrorResponse, Handle, ParentHandle, PublisherHandle, ResourceClassName, ResourceSetError,
-    RoaDefinition,
+use rpki::{
+    repository::{crypto::KeyIdentifier, x509::ValidationError},
+    uri,
 };
-use crate::commons::eventsourcing::{AggregateStoreError, KeyValueError};
-use crate::commons::remote::rfc6492;
-use crate::commons::remote::rfc6492::NotPerformedResponse;
-use crate::commons::remote::rfc8181;
-use crate::commons::remote::rfc8181::ReportErrorCode;
-use crate::commons::util::httpclient;
-use crate::commons::util::softsigner::SignerError;
-use crate::daemon::ca::RouteAuthorization;
-use crate::daemon::http::tls_keys;
 
+use crate::{
+    commons::{
+        api::{
+            rrdp::PublicationDeltaError, AspaCustomer, AspaProvidersUpdateConflict, ChildHandle, ErrorResponse, Handle,
+            ParentHandle, PublisherHandle, ResourceClassName, ResourceSetError, RoaDefinition,
+        },
+        eventsourcing::{AggregateStoreError, KeyValueError},
+        remote::{
+            rfc6492::{self, NotPerformedResponse},
+            rfc8181::{self, ReportErrorCode},
+        },
+        util::{httpclient, softsigner::SignerError},
+    },
+    daemon::{ca::RouteAuthorization, http::tls_keys},
+    upgrades::UpgradeError,
+};
+
+//------------ RoaDeltaError -----------------------------------------------
+
+/// This type contains a detailed error report for a ROA delta
+/// that could not be applied.
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
 pub struct RoaDeltaError {
     duplicates: Vec<RoaDefinition>,
@@ -111,6 +117,8 @@ impl fmt::Display for RoaDeltaError {
     }
 }
 
+//------------ ApiAuthError ------------------------------------------------
+
 // ApiAuthError is *also* implemented as a separate enum,
 // so that we don't have to implement the Clone trait for
 // all of the Error enum.
@@ -154,6 +162,8 @@ impl From<Error> for ApiAuthError {
     }
 }
 
+//------------ Error -------------------------------------------------------
+
 #[derive(Debug)]
 #[allow(clippy::large_enum_variant)]
 pub enum Error {
@@ -167,6 +177,8 @@ pub enum Error {
     HttpsSetup(String),
     HttpClientError(httpclient::Error),
     ConfigError(String),
+    UpgradeError(UpgradeError),
+
     //-----------------------------------------------------------------
     // General API Client Issues
     //-----------------------------------------------------------------
@@ -223,7 +235,6 @@ pub enum Error {
 
     // CA Repo Issues
     CaRepoInUse(Handle),
-    CaRepoAlreadyConfigured(Handle),
     CaRepoIssue(Handle, String),
     CaRepoResponseInvalidXml(Handle, String),
     CaRepoResponseWrongXml(Handle),
@@ -236,6 +247,7 @@ pub enum Error {
     CaParentResponseInvalidXml(Handle, String),
     CaParentResponseWrongXml(Handle),
     CaParentAddNotResponsive(Handle, ParentHandle),
+    CaParentSyncError(Handle, ParentHandle, ResourceClassName, String),
 
     //-----------------------------------------------------------------
     // RFC6492 (requesting resources)
@@ -261,7 +273,16 @@ pub enum Error {
     CaAuthorizationDuplicate(Handle, RouteAuthorization),
     CaAuthorizationInvalidMaxLength(Handle, RouteAuthorization),
     CaAuthorizationNotEntitled(Handle, RouteAuthorization),
-    RoaDeltaError(RoaDeltaError),
+    RoaDeltaError(Handle, RoaDeltaError),
+
+    //-----------------------------------------------------------------
+    // Autonomous System Provider Authorization - ASPA
+    //-----------------------------------------------------------------
+    AspaCustomerAsNotEntitled(Handle, AspaCustomer),
+    AspaCustomerAlreadyPresent(Handle, AspaCustomer),
+    AspaCustomerUnknown(Handle, AspaCustomer),
+    AspaProvidersUpdateEmpty(Handle, AspaCustomer),
+    AspaProvidersUpdateConflict(Handle, AspaProvidersUpdateConflict),
 
     //-----------------------------------------------------------------
     // Key Usage Issues
@@ -297,6 +318,7 @@ pub enum Error {
     // If we really don't know any more..
     //-----------------------------------------------------------------
     Custom(String),
+    Multiple(Vec<Error>),
 }
 
 impl fmt::Display for Error {
@@ -312,6 +334,7 @@ impl fmt::Display for Error {
             Error::HttpsSetup(e) => write!(f, "Cannot set up HTTPS: {}", e),
             Error::HttpClientError(e) => write!(f, "HTTP client error: {}", e),
             Error::ConfigError(e) => write!(f, "Configuration error: {}", e),
+            Error::UpgradeError(e) => write!(f, "Could not upgrade Krill: {}", e),
 
             //-----------------------------------------------------------------
             // General API Client Issues
@@ -349,7 +372,7 @@ impl fmt::Display for Error {
             //-----------------------------------------------------------------
             // Repository Server Issues
             //-----------------------------------------------------------------
-            Error::RepositoryServerNotInitialized => write!(f, "Publication Server not initialized"),
+            Error::RepositoryServerNotInitialized => write!(f, "Publication Server not initialized, see 'krillc pubserver server init --help'"),
             Error::RepositoryServerHasPublishers => write!(f, "Publication Server cannot be removed, still has publishers"),
             Error::RepositoryServerAlreadyInitialized => write!(f, "Publication Server already initialized"),
 
@@ -371,7 +394,6 @@ impl fmt::Display for Error {
 
             // CA Repo Issues
             Error::CaRepoInUse(ca) => write!(f, "CA '{}' already uses this repository", ca),
-            Error::CaRepoAlreadyConfigured(ca) => write!(f, "CA '{}' already has a repository, see issue 481 for planned work to support migrations", ca),
             Error::CaRepoIssue(ca, e) => write!(f, "CA '{}' cannot get response from repository '{}'. Is the 'service_uri' in the XML reachable? Note that when upgrading Krill you should re-use existing configuration and data. For a fresh \
             re-install of Krill you will need to send XML to all other parties again: parent(s), children, and repository", ca,        e),
             Error::CaRepoResponseInvalidXml(ca, e) => write!(f, "CA '{}' got invalid repository response xml: {}", ca, e),
@@ -385,6 +407,17 @@ impl fmt::Display for Error {
             Error::CaParentResponseInvalidXml(ca, e) => write!(f, "CA '{}' got invalid parent response xml: {}", ca, e),
             Error::CaParentResponseWrongXml(ca) => write!(f, "CA '{}' got repository response when adding parent", ca),
             Error::CaParentAddNotResponsive(ca, parent) => write!(f, "CA '{}' cannot get response from parent '{}'. Is the 'service_uri' in the XML reachable? Note that when upgrading Krill you should re-use existing configuration and data. For a fresh re-install of Krill you will need to send XML to all other parties again: parent(s), children, and repository",        ca, parent),
+            Error::CaParentSyncError(ca, parent, rcn, error_msg) => {
+                write!(
+                    f,
+                    "CA '{}' could not sync with parent '{}', for resource class '{}', error: {}",
+                    ca,
+                    parent,
+                    rcn,
+                    error_msg
+                )
+            }
+
 
             //-----------------------------------------------------------------
             // RFC6492 (requesting resources)
@@ -410,7 +443,16 @@ impl fmt::Display for Error {
             Error::CaAuthorizationDuplicate(_ca, roa) => write!(f, "ROA '{}' already present", roa),
             Error::CaAuthorizationInvalidMaxLength(_ca, roa) => write!(f, "Invalid max length in ROA: '{}'", roa),
             Error::CaAuthorizationNotEntitled(_ca, roa) => write!(f, "Prefix in ROA '{}' not held by you", roa),
-            Error::RoaDeltaError(e) => write!(f, "ROA delta rejected:\n\n'{}' ", e),
+            Error::RoaDeltaError(_ca, e) => write!(f, "ROA delta rejected:\n\n'{}' ", e),
+
+            //-----------------------------------------------------------------
+            // Autonomous System Provider Authorization - ASPAs
+            //-----------------------------------------------------------------
+            Error::AspaCustomerAsNotEntitled(_ca, asn) => write!(f, "Customer AS '{}' is not held by you", asn),
+            Error::AspaCustomerAlreadyPresent(_ca, asn) => write!(f, "ASPA already exists for customer AS '{}'", asn),
+            Error::AspaCustomerUnknown(_ca, asn) => write!(f, "No current ASPA exists for customer AS '{}'", asn),
+            Error::AspaProvidersUpdateEmpty(_ca, asn) => write!(f, "Received empty update for ASPA for customer AS '{}'", asn),
+            Error::AspaProvidersUpdateConflict(_ca, e) => write!(f, "ASPA delta rejected:\n\n'{}'", e),
 
             //-----------------------------------------------------------------
             // Key Usage Issues
@@ -446,7 +488,12 @@ impl fmt::Display for Error {
             //-----------------------------------------------------------------
             // If we really don't know any more..
             //-----------------------------------------------------------------
-            Error::Custom(s) => s.fmt(f)
+            Error::Custom(s) => s.fmt(f),
+
+            Error::Multiple(errors) => {
+                let error_strings: Vec<String> = errors.iter().map(|e| e.to_string()).collect();
+                write!(f, "Multiple errors: {}", error_strings.join(", "))
+            }
         }
     }
 }
@@ -524,6 +571,12 @@ impl From<PublicationDeltaError> for Error {
     }
 }
 
+impl From<UpgradeError> for Error {
+    fn from(e: UpgradeError) -> Self {
+        Error::UpgradeError(e)
+    }
+}
+
 impl Error {
     pub fn signer(e: impl Display) -> Self {
         Error::SignerError(e.to_string())
@@ -544,6 +597,10 @@ impl Error {
     pub fn custom(msg: impl fmt::Display) -> Self {
         Error::Custom(msg.to_string())
     }
+
+    pub fn io_error_with_context(context: String, cause: io::Error) -> Self {
+        Error::IoError(KrillIoError::new(context, cause))
+    }
 }
 
 impl std::error::Error for Error {}
@@ -600,6 +657,9 @@ impl Error {
             // internal configuration error
             Error::ConfigError(e) => ErrorResponse::new("sys-config", &self).with_cause(e),
 
+            // upgrade error
+            Error::UpgradeError(e) => ErrorResponse::new("sys-upgrade", &self).with_cause(e),
+
             //-----------------------------------------------------------------
             // General API Client Issues (label: api-*)
             //-----------------------------------------------------------------
@@ -680,7 +740,6 @@ impl Error {
             Error::CaUnknown(ca) => ErrorResponse::new("ca-unknown", &self).with_ca(ca),
 
             Error::CaRepoInUse(ca) => ErrorResponse::new("ca-repo-same", &self).with_ca(ca),
-            Error::CaRepoAlreadyConfigured(ca) => ErrorResponse::new("ca-repo-already-configured", &self).with_ca(ca),
 
             Error::CaRepoIssue(ca, err) => ErrorResponse::new("ca-repo-issue", &self).with_ca(ca).with_cause(err),
 
@@ -719,6 +778,11 @@ impl Error {
                 .with_ca(ca)
                 .with_parent(parent),
 
+            Error::CaParentSyncError(ca, parent, rcn, _errors) => ErrorResponse::new("ca-parent-sync", &self)
+                .with_ca(ca)
+                .with_parent(parent)
+                .with_resource_class(rcn),
+
             //-----------------------------------------------------------------
             // RFC6492 (requesting resources, not on JSON api)
             //-----------------------------------------------------------------
@@ -748,7 +812,6 @@ impl Error {
             Error::CaAuthorizationUnknown(ca, auth) => {
                 ErrorResponse::new("ca-roa-unknown", &self).with_ca(ca).with_auth(auth)
             }
-
             Error::CaAuthorizationDuplicate(ca, auth) => ErrorResponse::new("ca-roa-duplicate", &self)
                 .with_ca(ca)
                 .with_auth(auth),
@@ -761,10 +824,28 @@ impl Error {
                 .with_ca(ca)
                 .with_auth(auth),
 
-            Error::RoaDeltaError(roa_delta_error) => {
-                ErrorResponse::new("ca-roa-delta-error", "Delta rejected, see included json")
-                    .with_roa_delta_error(roa_delta_error)
-            }
+            Error::RoaDeltaError(ca, roa_delta_error) => ErrorResponse::new("ca-roa-delta-error", &self)
+                .with_ca(ca)
+                .with_roa_delta_error(roa_delta_error),
+
+            //-----------------------------------------------------------------
+            // Autonomous System Provider Authorization - ASPA
+            //-----------------------------------------------------------------
+            Error::AspaCustomerAsNotEntitled(ca, asn) => ErrorResponse::new("ca-aspa-not-entitled", &self)
+                .with_ca(ca)
+                .with_asn(*asn),
+            Error::AspaCustomerAlreadyPresent(ca, asn) => ErrorResponse::new("ca-aspa-customer-as-duplicate", &self)
+                .with_ca(ca)
+                .with_asn(*asn),
+            Error::AspaCustomerUnknown(ca, asn) => ErrorResponse::new("ca-aspa-unknown-customer-as", &self)
+                .with_ca(ca)
+                .with_asn(*asn),
+            Error::AspaProvidersUpdateEmpty(ca, asn) => ErrorResponse::new("ca-aspa-delta-empty", &self)
+                .with_ca(ca)
+                .with_asn(*asn),
+            Error::AspaProvidersUpdateConflict(ca, conflict) => ErrorResponse::new("ca-aspa-delta-error", &self)
+                .with_ca(ca)
+                .with_aspa_providers_conflict(conflict),
 
             //-----------------------------------------------------------------
             // Key Usage Issues (key-*)
@@ -776,6 +857,7 @@ impl Error {
             Error::KeyUseNoIssuedCert => ErrorResponse::new("key-no-cert", &self),
             Error::KeyUseNoMatch(ki) => ErrorResponse::new("key-no-match", &self).with_key_identifier(ki),
             Error::KeyRollNotAllowed => ErrorResponse::new("key-roll-disallowed", &self),
+
             //-----------------------------------------------------------------
             // Resource Issues (label: rc-*)
             //-----------------------------------------------------------------
@@ -799,6 +881,7 @@ impl Error {
             // If we really don't know any more..
             //-----------------------------------------------------------------
             Error::Custom(_msg) => ErrorResponse::new("general-error", &self),
+            Error::Multiple(_errors) => ErrorResponse::new("multiple-errors", &self),
         }
     }
 
@@ -1143,14 +1226,13 @@ mod tests {
         error.add_invalid_length(invalid_length);
         error.add_unknown(unknown);
 
-        // println!(
-        //     "{}",
-        //     serde_json::to_string_pretty(&Error::RoaDeltaError(error).to_error_response()).unwrap()
-        // );
+        let ca = Handle::from_str("ca").unwrap();
+
+        let error = Error::RoaDeltaError(ca, error);
 
         verify(
             include_str!("../../test-resources/errors/ca-roa-delta-error.json"),
-            Error::RoaDeltaError(error),
+            error,
         );
     }
 }
