@@ -1,27 +1,26 @@
-use std::convert::TryFrom;
-use std::ops::Deref;
-use std::sync::Arc;
-use std::{collections::HashMap, vec};
+use std::{collections::HashMap, convert::TryFrom, ops::Deref, sync::Arc, vec};
 
 use bytes::Bytes;
 use chrono::Duration;
 
 use rpki::{
-    cert::{Cert, KeyUsage, Overclaim, TbsCert},
-    crypto::{KeyIdentifier, PublicKey},
-    rta::RtaBuilder,
+    repository::{
+        cert::{Cert, KeyUsage, Overclaim, TbsCert},
+        crypto::{KeyIdentifier, PublicKey},
+        rta::RtaBuilder,
+        x509::{Serial, Time, Validity},
+    },
     uri,
-    x509::{Serial, Time, Validity},
 };
 
 use crate::{
     commons::{
         api::{
-            self, CertAuthInfo, ChildHandle, EntitlementClass, Entitlements, Handle, IdCertPem, IssuanceRequest,
-            IssuedCert, ObjectName, ParentCaContact, ParentHandle, RcvdCert, RepoInfo, RepositoryContact,
-            RequestResourceLimit, ResourceClassName, ResourceSet, Revocation, RevocationRequest, RevocationResponse,
-            RoaDefinition, RtaList, RtaName, RtaPrepResponse, SigningCert, StorableCaCommand, TaCertDetails,
-            TrustAnchorLocator,
+            self, AspaCustomer, AspaDefinitionList, AspaDefinitionUpdates, AspaProvidersUpdate, CertAuthInfo,
+            ChildHandle, EntitlementClass, Entitlements, Handle, IdCertPem, IssuanceRequest, IssuedCert, ObjectName,
+            ParentCaContact, ParentHandle, RcvdCert, RepoInfo, RepositoryContact, RequestResourceLimit,
+            ResourceClassName, ResourceSet, Revocation, RevocationRequest, RevocationResponse, RoaDefinition, RtaList,
+            RtaName, RtaPrepResponse, SigningCert, StorableCaCommand, TaCertDetails, TrustAnchorLocator,
         },
         crypto::{CsrInfo, IdCert, IdCertBuilder, KrillSigner, ProtocolCms, ProtocolCmsBuilder},
         error::{Error, RoaDeltaError},
@@ -32,9 +31,9 @@ use crate::{
     constants::test_mode_enabled,
     daemon::{
         ca::{
-            events::ChildCertificateUpdates, ta_handle, CaEvt, CaEvtDet, ChildDetails, Cmd, CmdDet, Ini, PreparedRta,
-            ResourceClass, ResourceTaggedAttestation, RouteAuthorization, RouteAuthorizationUpdates, Routes,
-            RtaContentRequest, RtaPrepareRequest, Rtas, SignedRta,
+            events::ChildCertificateUpdates, ta_handle, AspaDefinitions, CaEvt, CaEvtDet, ChildDetails, Cmd, CmdDet,
+            DropReason, Ini, PreparedRta, ResourceClass, ResourceTaggedAttestation, RouteAuthorization,
+            RouteAuthorizationUpdates, Routes, RtaContentRequest, RtaPrepareRequest, Rtas, SignedRta,
         },
         config::{Config, IssuanceTimingConfig},
     },
@@ -106,6 +105,12 @@ pub struct CertAuth {
     #[serde(skip_serializing_if = "Rtas::is_empty", default = "Rtas::default")]
     rtas: Rtas,
 
+    #[serde(
+        skip_serializing_if = "AspaDefinitions::is_empty",
+        default = "AspaDefinitions::default"
+    )]
+    aspas: AspaDefinitions,
+
     #[serde(skip_serializing, default = "CertAuthStatus::default")]
     status: CertAuthStatus,
 }
@@ -127,6 +132,7 @@ impl Aggregate for CertAuth {
         let children = HashMap::new();
         let routes = Routes::default();
         let rtas = Rtas::default();
+        let aspas = AspaDefinitions::default();
         let repository = None;
 
         Ok(CertAuth {
@@ -145,6 +151,7 @@ impl Aggregate for CertAuth {
 
             routes,
             rtas,
+            aspas,
             status: CertAuthStatus::Active,
         })
     }
@@ -203,15 +210,20 @@ impl Aggregate for CertAuth {
                 updates,
             } => {
                 let rc = self.resources.get_mut(&resource_class_name).unwrap();
-                let (issued, removed) = updates.unpack();
-                for iss in issued {
-                    rc.certificate_issued(iss)
+                let (issued, removed, suspended_certs, unsuspended_certs) = updates.unpack();
+                for cert in issued {
+                    rc.certificate_issued(cert)
                 }
+
+                for cert in unsuspended_certs {
+                    rc.certificate_unsuspended(cert)
+                }
+
                 for rem in removed {
                     rc.key_revoked(&rem);
 
                     // This loop is inefficient, but certificate revocations are not that common, so it's
-                    // not a big deal. Tracking this better would require that track the child handle somehow.
+                    // not a big deal. Tracking this better would require that we track the child handle somehow.
                     // That is a bit hard when this revocation is the result from a republish where we lost
                     // all resources delegated to the child.
                     for child in self.children.values_mut() {
@@ -220,6 +232,9 @@ impl Aggregate for CertAuth {
                         }
                     }
                 }
+                for cert in suspended_certs {
+                    rc.certificate_suspended(cert);
+                }
             }
 
             CaEvtDet::ChildUpdatedIdCert { child, id_cert } => {
@@ -234,6 +249,10 @@ impl Aggregate for CertAuth {
                 self.children.remove(&child);
             }
 
+            CaEvtDet::ChildSuspended { child } => self.children.get_mut(&child).unwrap().suspend(),
+
+            CaEvtDet::ChildUnsuspended { child } => self.children.get_mut(&child).unwrap().unsuspend(),
+
             //-----------------------------------------------------------------------
             // Being a child
             //-----------------------------------------------------------------------
@@ -358,6 +377,21 @@ impl Aggregate for CertAuth {
                 .unwrap()
                 .roas_updated(updates),
 
+            //-----------------------------------------------------------------------
+            // Autonomous System Provider Authorization
+            //-----------------------------------------------------------------------
+            CaEvtDet::AspaConfigAdded { aspa_config } => self.aspas.add_or_replace(aspa_config),
+            CaEvtDet::AspaConfigUpdated { customer, update } => self.aspas.apply_update(customer, &update),
+            CaEvtDet::AspaConfigRemoved { customer } => self.aspas.remove(customer),
+            CaEvtDet::AspaObjectsUpdated {
+                resource_class_name,
+                updates,
+            } => self
+                .resources
+                .get_mut(&resource_class_name)
+                .unwrap()
+                .aspa_objects_updated(updates),
+
             //-----------------------------------------------------------------------
             // Publication
             //-----------------------------------------------------------------------
@@ -394,7 +428,7 @@ impl Aggregate for CertAuth {
 
         match command.into_details() {
             // trust anchor
-            CmdDet::MakeTrustAnchor(uris, signer) => self.trust_anchor_make(uris, signer),
+            CmdDet::MakeTrustAnchor(uris, rsync_uri, signer) => self.trust_anchor_make(uris, rsync_uri, signer),
 
             // being a parent
             CmdDet::ChildAdd(child, id_cert, resources) => self.child_add(child, id_cert, resources),
@@ -403,6 +437,8 @@ impl Aggregate for CertAuth {
             CmdDet::ChildCertify(child, request, config, signer) => self.child_certify(child, request, &config, signer),
             CmdDet::ChildRevokeKey(child, request) => self.child_revoke_key(child, request),
             CmdDet::ChildRemove(child) => self.child_remove(&child),
+            CmdDet::ChildSuspendInactive(child) => self.child_suspend_inactive(&child),
+            CmdDet::ChildUnsuspend(child) => self.child_unsuspend(&child),
 
             // being a child
             CmdDet::GenerateNewIdKey(signer) => self.generate_new_id_key(signer),
@@ -416,6 +452,7 @@ impl Aggregate for CertAuth {
             CmdDet::UpdateRcvdCert(class_name, rcvd_cert, config, signer) => {
                 self.update_received_cert(class_name, rcvd_cert, &config, signer)
             }
+            CmdDet::DropResourceClass(rcn, reason, signer) => self.drop_resource_class(rcn, reason, signer),
 
             // Key rolls
             CmdDet::KeyRollInitiate(duration, signer) => self.keyroll_initiate(duration, signer),
@@ -426,7 +463,19 @@ impl Aggregate for CertAuth {
             CmdDet::RouteAuthorizationsUpdate(updates, config, signer) => {
                 self.route_authorizations_update(updates, &config, signer)
             }
-            CmdDet::RouteAuthorizationsRenew(config, signer) => self.route_authorizations_renew(&config, &signer),
+            CmdDet::RouteAuthorizationsRenew(config, signer) => {
+                self.route_authorizations_renew(false, &config, &signer)
+            }
+            CmdDet::RouteAuthorizationsForceRenew(config, signer) => {
+                self.route_authorizations_renew(true, &config, &signer)
+            }
+
+            // ASPA
+            CmdDet::AspasUpdate(updates, config, signer) => self.aspas_definitions_update(updates, &config, &signer),
+            CmdDet::AspasUpdateExisting(customer, update, config, signer) => {
+                self.aspas_update(customer, update, &config, &signer)
+            }
+            CmdDet::AspasRenew(config, signer) => self.aspas_renew(&config, &signer),
 
             // Republish
             CmdDet::RepoUpdate(contact, signer) => self.update_repo(contact, &signer),
@@ -476,7 +525,22 @@ impl CertAuth {
 
         let id_cert_pem = IdCertPem::from(&self.id.cert);
 
-        CertAuthInfo::new(handle, id_cert_pem, repo_info, parents, resources, children)
+        let suspended_children = self
+            .children
+            .iter()
+            .filter(|(_ca, details)| details.is_suspended())
+            .map(|(ca, _)| ca.clone())
+            .collect();
+
+        CertAuthInfo::new(
+            handle,
+            id_cert_pem,
+            repo_info,
+            parents,
+            resources,
+            children,
+            suspended_children,
+        )
     }
 
     /// Returns the current RoaDefinitions for this, i.e. the intended authorized
@@ -537,7 +601,12 @@ impl CertAuth {
 /// # Being a Trust Anchor
 ///
 impl CertAuth {
-    fn trust_anchor_make(&self, uris: Vec<uri::Https>, signer: Arc<KrillSigner>) -> KrillResult<Vec<CaEvt>> {
+    fn trust_anchor_make(
+        &self,
+        uris: Vec<uri::Https>,
+        rsync_uri: Option<uri::Rsync>,
+        signer: Arc<KrillSigner>,
+    ) -> KrillResult<Vec<CaEvt>> {
         if !self.resources.is_empty() {
             return Err(Error::custom("Cannot turn CA with resources into TA"));
         }
@@ -579,7 +648,7 @@ impl CertAuth {
             signer.sign_cert(cert, &key)?
         };
 
-        let tal = TrustAnchorLocator::new(uris, &cert);
+        let tal = TrustAnchorLocator::new(uris, rsync_uri, &cert);
 
         let ta_cert_details = TaCertDetails::new(cert, resources, tal);
 
@@ -684,13 +753,30 @@ impl CertAuth {
         let child_keys = child.issued(rcn);
 
         let mut issued_certs = vec![];
-        let mut not_after = Time::now();
+
+        // Check current issued certificates, so we may lie a tiny bit here.. i.e. we want to avoid that
+        // child CAs feel the urge to request new certificates all the time - so we will only tell them
+        // about the normal - longer - not after time if their current certificate(s) will expire within
+        // the configured number of weeks. I.e. using defaults:
+        //  - they would be eligible to a not-after of 52 weeks
+        //  - we only tell them 4 weeks before their old cert would expire
+        //
+        // Note that a child may have multiple keys and issued certificates if they are doing a keyroll.
+        // Typically these certificates will have almost the same expiration time, but even if they don't
+        // and one of them is about to expire, while the other is still valid for a while.. then telling
+        // the child that they are eligible to the not after time of the other is still fine - it would
+        // still trigger them to request a replacement for the first which was about to expire.
+        let mut not_after = Time::now() + Duration::weeks(issuance_timing.timing_child_certificate_valid_weeks);
+        let threshold = Time::now() + Duration::weeks(issuance_timing.timing_child_certificate_reissue_weeks_before);
+
         for ki in child_keys {
             if let Some(issued) = my_rc.issued(&ki) {
                 issued_certs.push(issued.clone());
-                let eligible_not_after = Self::child_cert_eligible_not_after(issued, issuance_timing);
-                if eligible_not_after > not_after {
-                    not_after = eligible_not_after
+
+                let expires = issued.validity().not_after();
+
+                if expires > threshold {
+                    not_after = expires;
                 }
             }
         }
@@ -704,17 +790,6 @@ impl CertAuth {
         ))
     }
 
-    fn child_cert_eligible_not_after(issued: &IssuedCert, issuance_timing: &IssuanceTimingConfig) -> Time {
-        let expiration_time = issued.validity().not_after();
-        if expiration_time
-            > Time::now() + chrono::Duration::weeks(issuance_timing.timing_child_certificate_reissue_weeks_before)
-        {
-            expiration_time
-        } else {
-            Time::now() + chrono::Duration::weeks(issuance_timing.timing_child_certificate_valid_weeks)
-        }
-    }
-
     /// Returns a child, or an error if the child is unknown.
     pub fn get_child(&self, child: &Handle) -> KrillResult<&ChildDetails> {
         match self.children.get(child) {
@@ -812,7 +887,7 @@ impl CertAuth {
     ) -> KrillResult<IssuedCert> {
         let my_rc = self.resources.get(&rcn).ok_or(Error::ResourceClassUnknown(rcn))?;
 
-        let child = self.get_child(&child)?;
+        let child = self.get_child(child)?;
         child.resources().apply_limit(&limit)?;
 
         my_rc.issue_cert(csr_info, child.resources(), limit, issuance_timing, signer)
@@ -913,7 +988,7 @@ impl CertAuth {
     }
 
     fn child_remove(&self, child_handle: &ChildHandle) -> KrillResult<Vec<CaEvt>> {
-        let child = self.get_child(&child_handle)?;
+        let child = self.get_child(child_handle)?;
 
         let mut version = self.version;
         let handle = &self.handle;
@@ -959,6 +1034,128 @@ impl CertAuth {
         Ok(res)
     }
 
+    // Suspend a child. The intention is that this is called when it is discovered
+    // that the child has been inactive, i.e. not contacting this parent for a pro-longed
+    // period of time (hours).
+    //
+    // When a child is suspended we need to:
+    // - mark it as suspended
+    // - withdraw all certificates issued to it (suspend them)
+    fn child_suspend_inactive(&self, child_handle: &ChildHandle) -> KrillResult<Vec<CaEvt>> {
+        let mut res = vec![];
+
+        let child = self.get_child(child_handle)?;
+
+        if child.is_suspended() {
+            return Ok(res); // nothing to do, child is already suspended
+        }
+
+        let mut version = self.version;
+        let handle = &self.handle;
+
+        // Find all the certs in all RCs for this child and suspend them.
+        for (rcn, rc) in self.resources.iter() {
+            let certified_keys = child.issued(rcn);
+
+            if certified_keys.is_empty() {
+                continue;
+            }
+
+            let mut cert_updates = ChildCertificateUpdates::default();
+
+            for key in certified_keys {
+                if let Some(issued) = rc.issued(&key) {
+                    cert_updates.suspend(issued.clone());
+                }
+            }
+
+            res.push(CaEvtDet::child_certificates_updated(
+                handle,
+                version,
+                rcn.clone(),
+                cert_updates,
+            ));
+            version += 1;
+        }
+
+        // Only mark the child as suspended if there was at least one certificate
+        // to suspend above. If not this is a no-op - the child has not yet requested
+        // any certificates so there is nothing to suspend.
+        if !res.is_empty() {
+            info!("CA '{}' suspended inactive child '{}'", handle, child_handle);
+            res.push(CaEvtDet::child_suspended(handle, version, child_handle.clone()));
+        }
+
+        Ok(res)
+    }
+
+    // Unsuspend a child. The intention is that this is called automatically when
+    // a suspended (inactive) child CA is seen to contact this parent again.
+    //
+    // When a child is unsuspended we need to:
+    // - mark it as unsuspended
+    // - republish existing suspended certificates for it, provided that
+    //    - they will not expire for another day
+    //    - they do not exceed the current resource entitlements of the CA
+    // - other suspended certificates will just be removed.
+    //
+    // Then the child may or may not request new certificates as it sees fit.
+    // I.e. the unsuspend should be done before the child gets an answer to its
+    // RFC 6492 list request.
+    fn child_unsuspend(&self, child_handle: &ChildHandle) -> KrillResult<Vec<CaEvt>> {
+        let mut res = vec![];
+
+        let child = self.get_child(child_handle)?;
+
+        if !child.is_suspended() {
+            return Ok(res); // nothing to do, child is not suspended
+        }
+
+        let mut version = self.version;
+        let handle = &self.handle;
+
+        // Find all the certs in all RCs for this child and suspend them.
+        for (rcn, rc) in self.resources.iter() {
+            let certified_keys = child.issued(rcn);
+
+            if certified_keys.is_empty() {
+                continue;
+            }
+
+            let mut cert_updates = ChildCertificateUpdates::default();
+
+            for key in certified_keys {
+                if let Some(cert) = rc.suspended(&key) {
+                    // check that the cert is actually not expired or about to expire and not overclaiming
+                    if cert.validity().not_after() > Time::now() + Duration::days(1)
+                        && child.resources().contains(cert.resource_set())
+                    {
+                        // certificate is still fit for publication, so move it back to issued
+                        cert_updates.unsuspend(cert.clone());
+                    } else {
+                        // certificate should not be published as is. Remove it and the child will request
+                        // a new certificate because the resources and or validity entitlements will have
+                        // changed.
+                        cert_updates.remove(cert.subject_key_identifier());
+                    }
+                }
+            }
+
+            res.push(CaEvtDet::child_certificates_updated(
+                handle,
+                version,
+                rcn.clone(),
+                cert_updates,
+            ));
+            version += 1;
+        }
+
+        info!("CA '{}' unsuspended child '{}'", handle, child_handle);
+        res.push(CaEvtDet::child_unsuspended(handle, version, child_handle.clone()));
+
+        Ok(res)
+    }
+
     /// Returns `true` if the child is known, `false` otherwise. No errors.
     fn has_child(&self, child_handle: &Handle) -> bool {
         self.children.contains_key(child_handle)
@@ -985,6 +1182,10 @@ impl CertAuth {
         self.parents.keys()
     }
 
+    pub fn nr_parents(&self) -> usize {
+        self.parents.len()
+    }
+
     pub fn parent_known(&self, parent: &ParentHandle) -> bool {
         self.parents.contains_key(parent)
     }
@@ -1195,7 +1396,7 @@ impl CertAuth {
         for ent in entitlements.classes() {
             let parent_rc_name = ent.class_name();
 
-            match self.find_parent_rc(&parent_handle, &parent_rc_name) {
+            match self.find_parent_rc(&parent_handle, parent_rc_name) {
                 Some(rc) => {
                     // We have a matching RC, make requests (note this may be a no-op).
                     event_details.append(&mut self.make_request_events(ent, rc, signer.deref())?);
@@ -1246,7 +1447,7 @@ impl CertAuth {
     ///
     /// This will also generate appropriate events for changes affecting
     /// issued ROAs and delegated certificates - if because resources were
-    //  lost and ROAs/Certs would be become invalid.
+    /// lost and ROAs/Certs would be become invalid.
     fn update_received_cert(
         &self,
         rcn: ResourceClassName,
@@ -1258,7 +1459,14 @@ impl CertAuth {
 
         let rc = self.resources.get(&rcn).ok_or(Error::ResourceClassUnknown(rcn))?;
 
-        let evt_details = rc.update_received_cert(self.handle(), rcvd_cert, &self.routes, config, signer.deref())?;
+        let evt_details = rc.update_received_cert(
+            self.handle(),
+            rcvd_cert,
+            &self.routes,
+            &self.aspas,
+            config,
+            signer.deref(),
+        )?;
 
         let mut res = vec![];
         let mut version = self.version;
@@ -1270,6 +1478,31 @@ impl CertAuth {
 
         Ok(res)
     }
+
+    /// Drop a resource class because it no longer works under this parent for the specified
+    /// reason. Note that this will generate revocation requests for the current keys which
+    /// will be sent to the parent on a best effort basis - e.g. if the parent removed the resource
+    /// class it may well refuse to revoke the keys - it may not known them.
+    fn drop_resource_class(
+        &self,
+        rcn: ResourceClassName,
+        reason: DropReason,
+        signer: Arc<KrillSigner>,
+    ) -> KrillResult<Vec<CaEvt>> {
+        warn!("Dropping resource class '{}' because of reason: {}", rcn, reason);
+
+        let rc = self
+            .resources
+            .get(&rcn)
+            .ok_or_else(|| Error::ResourceClassUnknown(rcn.clone()))?;
+        let revoke_requests = rc.revoke(signer.deref())?;
+
+        Ok(self.events_from_details(vec![CaEvtDet::ResourceClassRemoved {
+            resource_class_name: rcn,
+            parent: rc.parent_handle().clone(),
+            revoke_requests,
+        }]))
+    }
 }
 
 /// # Key Rolls
@@ -1372,27 +1605,26 @@ impl CertAuth {
     ///    - Will return an error in case the repo is already set (issue 481)
     ///    - Will support migrations using key rollover in future (issue 480)
     ///    - Assumes that the repository can be reached (this is checked by CaManager before issuing the command to this CA)
-    pub fn update_repo(&self, contact: RepositoryContact, _signer: &KrillSigner) -> KrillResult<Vec<CaEvt>> {
-        if let Some(_existing_contact) = &self.repository {
-            // Disallow, see issue 481
-            return Err(Error::CaRepoAlreadyConfigured(self.handle.clone()));
-            // TODO: check that it is indeed different and then allow (issue 480)
-            // if existing_contact == &contact {
-            //     return Err(Error::CaRepoInUse(self.handle.clone()));
-            // }
-            // // Initiate rolls in all RCs so we can use the new repo in the new key.
-            // let info = contact.repo_info().clone();
-            // for rc in self.resources.values() {
-            //     // If we are in any keyroll, reject.. because we will need to
-            //     // introduce the change as a key roll (new key, new repo, etc),
-            //     // and we can only do one roll at a time.
-            //     if !rc.key_roll_possible() {
-            //         // If we can't roll... well then we have to bail out.
-            //         // Note: none of these events are committed in that case.
-            //         return Err(Error::KeyRollNotAllowed);
-            //     }
-            //     evt_dets.append(&mut rc.keyroll_initiate(&info, Duration::seconds(0), &signer)?);
-            // }
+    pub fn update_repo(&self, contact: RepositoryContact, signer: &KrillSigner) -> KrillResult<Vec<CaEvt>> {
+        let mut evt_dets = vec![];
+        if let Some(existing_contact) = &self.repository {
+            if existing_contact == &contact {
+                return Err(Error::CaRepoInUse(self.handle.clone()));
+            }
+            // Initiate rolls in all RCs so we can use the new repo in the new key.
+            let info = contact.repo_info().clone();
+            for rc in self.resources.values() {
+                // If we are in any keyroll, reject.. because we will need to
+                // introduce the change as a key roll (new key, new repo, etc),
+                // and we can only do one roll at a time.
+                if !rc.key_roll_possible() {
+                    // If we can't roll... well then we have to bail out.
+                    // Note: none of these events are committed in that case.
+                    return Err(Error::KeyRollNotAllowed);
+                }
+
+                evt_dets.append(&mut rc.keyroll_initiate(&info, Duration::seconds(0), signer)?);
+            }
         }
 
         // register updated repo
@@ -1402,7 +1634,7 @@ impl CertAuth {
             contact.service_uri()
         );
 
-        let evt_dets = vec![CaEvtDet::RepoUpdated { contact }];
+        evt_dets.push(CaEvtDet::RepoUpdated { contact });
         Ok(self.events_from_details(evt_dets))
     }
 }
@@ -1425,7 +1657,7 @@ impl CertAuth {
 
         // for rc in self.resources
         for (rcn, rc) in self.resources.iter() {
-            let updates = rc.update_roas(&routes, None, config, signer.deref())?;
+            let updates = rc.update_roas(&routes, config, signer.deref())?;
             if updates.contains_changes() {
                 info!("CA '{}' under RC '{}' updated ROAs: {}", self.handle, rcn, updates);
 
@@ -1440,16 +1672,25 @@ impl CertAuth {
     }
 
     /// Renew existing ROA objects if needed.
-    pub fn route_authorizations_renew(&self, config: &Config, signer: &KrillSigner) -> KrillResult<Vec<CaEvt>> {
+    pub fn route_authorizations_renew(
+        &self,
+        force: bool,
+        config: &Config,
+        signer: &KrillSigner,
+    ) -> KrillResult<Vec<CaEvt>> {
         let mut evt_dets = vec![];
 
         for (rcn, rc) in self.resources.iter() {
-            let updates = rc.renew_roas(&config.issuance_timing, signer)?;
+            let updates = rc.renew_roas(force, &config.issuance_timing, signer)?;
             if updates.contains_changes() {
-                info!(
-                    "CA '{}' reissued ROAs under RC '{}' before they would expire: {}",
-                    self.handle, rcn, updates
-                );
+                if force {
+                    info!("CA '{}' reissued all ROAs under RC '{}'", self.handle, rcn);
+                } else {
+                    info!(
+                        "CA '{}' reissued ROAs under RC '{}' before they would expire: {}",
+                        self.handle, rcn, updates
+                    );
+                }
 
                 evt_dets.push(CaEvtDet::RoasUpdated {
                     resource_class_name: rcn.clone(),
@@ -1512,13 +1753,171 @@ impl CertAuth {
         }
 
         if !delta_errors.is_empty() {
-            Err(Error::RoaDeltaError(delta_errors))
+            Err(Error::RoaDeltaError(self.handle().clone(), delta_errors))
         } else {
             Ok((desired_routes, res))
         }
     }
 }
 
+/// # Autonomous System Provider Authorizations
+///
+impl CertAuth {
+    /// Show current AspaDefinitions
+    pub fn aspas_definitions_show(&self) -> AspaDefinitionList {
+        AspaDefinitionList::new(self.aspas.all().cloned().collect())
+    }
+
+    /// Process AspaDefinitionUpdates:
+    /// - add new aspas
+    /// - replace existing
+    /// - remove aspas to be removed
+    pub fn aspas_definitions_update(
+        &self,
+        updates: AspaDefinitionUpdates,
+        config: &Config,
+        signer: &KrillSigner,
+    ) -> KrillResult<Vec<CaEvt>> {
+        let mut res = vec![];
+
+        let (add_or_replace, remove) = updates.unpack();
+
+        // Keep track of a copy of the AspaDefinitions so we can use to update ASPA objects
+        let mut all_aspas = self.aspas.clone();
+
+        for customer in remove {
+            if !all_aspas.has(customer) {
+                return Err(Error::AspaCustomerUnknown(self.handle().clone(), customer));
+            }
+            res.push(CaEvtDet::AspaConfigRemoved { customer });
+            all_aspas.remove(customer);
+        }
+
+        for aspa_config in add_or_replace {
+            let customer = aspa_config.customer();
+
+            if !self.all_resources().contains_asn(customer) {
+                return Err(Error::AspaCustomerAsNotEntitled(self.handle().clone(), customer));
+            }
+
+            // Update the aspas copy so we can update ASPA objects for the events
+            all_aspas.add_or_replace(aspa_config.clone());
+
+            match self.aspas.get(customer) {
+                None => res.push(CaEvtDet::AspaConfigAdded { aspa_config }),
+                Some(existing) => {
+                    // Determine the update from existing to (new) aspa_config
+                    let added = aspa_config
+                        .providers()
+                        .iter()
+                        .filter(|new_provider| !existing.providers().contains(new_provider))
+                        .copied()
+                        .collect();
+
+                    let removed = existing
+                        .providers()
+                        .iter()
+                        .filter(|existing| !aspa_config.providers().contains(existing))
+                        .copied()
+                        .collect();
+
+                    let update = AspaProvidersUpdate::new(added, removed);
+
+                    if update.contains_changes() {
+                        res.push(CaEvtDet::AspaConfigUpdated { customer, update })
+                    }
+                }
+            }
+        }
+
+        res.append(&mut self.create_updated_aspa_objects(&all_aspas, config, signer)?);
+
+        Ok(self.events_from_details(res))
+    }
+
+    pub fn aspas_update(
+        &self,
+        customer: AspaCustomer,
+        update: AspaProvidersUpdate,
+        config: &Config,
+        signer: &KrillSigner,
+    ) -> KrillResult<Vec<CaEvt>> {
+        self.verify_update(customer, &update)?;
+
+        let mut all_aspas = self.aspas.clone();
+        all_aspas.apply_update(customer, &update);
+
+        let mut res = self.create_updated_aspa_objects(&all_aspas, config, signer)?;
+        res.push(CaEvtDet::AspaConfigUpdated { customer, update });
+
+        Ok(self.events_from_details(res))
+    }
+
+    /// Renew existing ASPA objects if needed.
+    pub fn aspas_renew(&self, config: &Config, signer: &KrillSigner) -> KrillResult<Vec<CaEvt>> {
+        let mut evt_dets = vec![];
+
+        for (rcn, rc) in self.resources.iter() {
+            let updates = rc.renew_aspas(&config.issuance_timing, signer)?;
+            if updates.contains_changes() {
+                info!(
+                    "CA '{}' reissued ASPAs under RC '{}' before they would expire",
+                    self.handle, rcn
+                );
+
+                evt_dets.push(CaEvtDet::AspaObjectsUpdated {
+                    resource_class_name: rcn.clone(),
+                    updates,
+                });
+            }
+        }
+
+        Ok(self.events_from_details(evt_dets))
+    }
+
+    fn create_updated_aspa_objects(
+        &self,
+        all_aspas: &AspaDefinitions,
+        config: &Config,
+        signer: &KrillSigner,
+    ) -> KrillResult<Vec<CaEvtDet>> {
+        let mut update_events = vec![];
+
+        for (rcn, rc) in self.resources.iter() {
+            let updates = rc.update_aspas(all_aspas, config, signer)?;
+            if updates.contains_changes() {
+                update_events.push(CaEvtDet::AspaObjectsUpdated {
+                    resource_class_name: rcn.clone(),
+                    updates,
+                });
+            }
+        }
+        Ok(update_events)
+    }
+
+    /// Verifies whether the update can be applied.
+    fn verify_update(&self, customer: AspaCustomer, update: &AspaProvidersUpdate) -> KrillResult<()> {
+        if update.is_empty() {
+            return Err(Error::AspaProvidersUpdateEmpty(self.handle().clone(), customer));
+        }
+
+        if !self.all_resources().contains_asn(customer) {
+            return Err(Error::AspaCustomerAsNotEntitled(self.handle().clone(), customer));
+        }
+
+        let current = self
+            .aspas
+            .get(customer)
+            .ok_or_else(|| Error::AspaCustomerUnknown(self.handle().clone(), customer))?;
+
+        current
+            .verify_update(update)
+            .map_err(|conflict| Error::AspaProvidersUpdateConflict(self.handle().clone(), conflict))?;
+
+        Ok(())
+    }
+}
+
 /// # Resource Tagged Attestations
 ///
 impl CertAuth {
@@ -1637,14 +2036,14 @@ impl CertAuth {
                 .current_resources()
                 .ok_or_else(|| Error::custom("RC for RTA has no resources"))?;
 
-            let intersection = rc_resources.intersection(&resources);
+            let intersection = rc_resources.intersection(resources);
             if intersection.is_empty() {
                 return Err(Error::custom(
                     "RC for prepared RTA no longer contains relevant resources",
                 ));
             }
 
-            let ee = rc.create_rta_ee(&intersection, validity, *key, &signer)?;
+            let ee = rc.create_rta_ee(&intersection, validity, *key, signer)?;
             rc_ee.insert(rcn.clone(), ee);
         }
 
@@ -1661,7 +2060,7 @@ impl CertAuth {
         // If there are no other keys supplied, then we MUST have all resources.
         // Otherwise we will just assume that others sign over the resources that
         // we do not have.
-        if keys.is_empty() && !self.all_resources().contains(&resources) {
+        if keys.is_empty() && !self.all_resources().contains(resources) {
             return Err(Error::RtaResourcesNotHeld);
         }
 
@@ -1672,7 +2071,7 @@ impl CertAuth {
                 let intersection = resources.intersection(rc_resources);
                 if !intersection.is_empty() {
                     let key = signer.create_key()?;
-                    let ee = rc.create_rta_ee(&intersection, validity, key, &signer)?;
+                    let ee = rc.create_rta_ee(&intersection, validity, key, signer)?;
                     rc_ee.insert(rcn.clone(), ee);
                 }
             }
