@@ -1,23 +1,24 @@
-use std::fmt;
-use std::sync::Arc;
+use std::{fmt, sync::Arc};
 
 use chrono::Duration;
 
 use rpki::uri;
 
-use crate::commons::{
-    actor::Actor,
-    api::{
-        ChildHandle, Entitlements, Handle, IssuanceRequest, ParentCaContact, ParentHandle, RcvdCert, RepositoryContact,
-        ResourceClassName, ResourceSet, RevocationRequest, RevocationResponse, RtaName, StorableCaCommand,
-        StorableRcEntitlement,
+use crate::{
+    commons::{
+        actor::Actor,
+        api::{
+            AspaCustomer, AspaDefinitionUpdates, AspaProvidersUpdate, ChildHandle, Entitlements, Handle,
+            IssuanceRequest, ParentCaContact, ParentHandle, RcvdCert, RepositoryContact, ResourceClassName,
+            ResourceSet, RevocationRequest, RevocationResponse, RtaName, StorableCaCommand, StorableRcEntitlement,
+        },
+        crypto::{IdCert, KrillSigner},
+        eventsourcing::{self, StoredCommand},
+    },
+    daemon::{
+        ca::{CaEvt, ResourceTaggedAttestation, RouteAuthorizationUpdates, RtaContentRequest, RtaPrepareRequest},
+        config::Config,
     },
-    crypto::{IdCert, KrillSigner},
-    eventsourcing::{self, StoredCommand},
-};
-use crate::daemon::{
-    ca::{CaEvt, ResourceTaggedAttestation, RouteAuthorizationUpdates, RtaContentRequest, RtaPrepareRequest},
-    config::Config,
 };
 
 //------------ StoredCaCommand ---------------------------------------------
@@ -28,6 +29,8 @@ pub type StoredCaCommand = StoredCommand<StorableCaCommand>;
 
 pub type Cmd = eventsourcing::SentCommand<CmdDet>;
 
+pub type DropReason = String;
+
 //------------ CommandDetails ----------------------------------------------
 
 #[derive(Clone, Debug)]
@@ -36,7 +39,7 @@ pub enum CmdDet {
     // ------------------------------------------------------------
     // Being a TA
     // ------------------------------------------------------------
-    MakeTrustAnchor(Vec<uri::Https>, Arc<KrillSigner>),
+    MakeTrustAnchor(Vec<uri::Https>, Option<uri::Rsync>, Arc<KrillSigner>),
 
     // ------------------------------------------------------------
     // Being a parent
@@ -61,6 +64,17 @@ pub enum CmdDet {
     // Remove child (also revokes, and removes issued certs, and republishes)
     ChildRemove(ChildHandle),
 
+    // Suspend a child (done by a background process which checks for inactive children)
+    // When a child is inactive it is assumed that they no longer maintain their repository.
+    // The certificate(s) issued to the child will be removed (and revoked) until
+    // the child is seen again and unsuspended (see below).
+    ChildSuspendInactive(ChildHandle),
+
+    // Unsuspend a child (when it contacts the server again). I.e. mark it as active once
+    // again and republish existing certificates provided that they are not expired, or
+    // about to expire, and do not claim resources no longer associated with this child.
+    ChildUnsuspend(ChildHandle),
+
     // ------------------------------------------------------------
     // Being a child (only allowed if this CA is not self-signed)
     // ------------------------------------------------------------
@@ -88,6 +102,10 @@ pub enum CmdDet {
     // Process a new certificate received from a parent.
     UpdateRcvdCert(ResourceClassName, RcvdCert, Arc<Config>, Arc<KrillSigner>),
 
+    // Drop a resource class under a parent because of issues
+    // obtaining a certificate for it.
+    DropResourceClass(ResourceClassName, DropReason, Arc<KrillSigner>),
+
     // ------------------------------------------------------------
     // Key rolls
     // ------------------------------------------------------------
@@ -124,9 +142,29 @@ pub enum CmdDet {
     // Re-issue any and all ROA objects which would otherwise expire in
     // some time (default 4 weeks, configurable). Note that this command
     // is intended to be sent by the scheduler - once a day is fine - and
-    // will only be stored if there any updates to be done.
+    // will only be stored if there are any updates to be done.
     RouteAuthorizationsRenew(Arc<Config>, Arc<KrillSigner>),
 
+    // Re-issue all ROA objects regardless of their expiration time.
+    RouteAuthorizationsForceRenew(Arc<Config>, Arc<KrillSigner>),
+
+    // ------------------------------------------------------------
+    // ASPA Support
+    // ------------------------------------------------------------
+
+    // Update AspaDefinitions, adding new, replacing existing, or
+    // removing surplus.
+    AspasUpdate(AspaDefinitionUpdates, Arc<Config>, Arc<KrillSigner>),
+
+    // Updates an existing AspaProviders for the given AspaCustomer
+    AspasUpdateExisting(AspaCustomer, AspaProvidersUpdate, Arc<Config>, Arc<KrillSigner>),
+
+    // Re-issue any and all ASPA objects which would otherwise expire in
+    // some time (default 4 weeks, configurable). Note that this command
+    // is intended to be sent by the scheduler - once a day is fine - and
+    // will only be stored if there are any updates to be done.
+    AspasRenew(Arc<Config>, Arc<KrillSigner>),
+
     // ------------------------------------------------------------
     // Publishing
     // ------------------------------------------------------------
@@ -168,7 +206,7 @@ impl From<CmdDet> for StorableCaCommand {
             // ------------------------------------------------------------
             // Being a TA
             // ------------------------------------------------------------
-            CmdDet::MakeTrustAnchor(_, _) => StorableCaCommand::MakeTrustAnchor,
+            CmdDet::MakeTrustAnchor(_, _, _) => StorableCaCommand::MakeTrustAnchor,
 
             // ------------------------------------------------------------
             // Being a parent
@@ -197,6 +235,8 @@ impl From<CmdDet> for StorableCaCommand {
             }
             CmdDet::ChildRevokeKey(child, revoke_req) => StorableCaCommand::ChildRevokeKey { child, revoke_req },
             CmdDet::ChildRemove(child) => StorableCaCommand::ChildRemove { child },
+            CmdDet::ChildSuspendInactive(child) => StorableCaCommand::ChildSuspendInactive { child },
+            CmdDet::ChildUnsuspend(child) => StorableCaCommand::ChildUnsuspend { child },
 
             // ------------------------------------------------------------
             // Being a child
@@ -226,6 +266,10 @@ impl From<CmdDet> for StorableCaCommand {
                 resource_class_name,
                 resources: rcvd_cert.resources().clone(),
             },
+            CmdDet::DropResourceClass(resource_class_name, reason, _) => StorableCaCommand::DropResourceClass {
+                resource_class_name,
+                reason,
+            },
 
             // ------------------------------------------------------------
             // Key rolls
@@ -245,6 +289,16 @@ impl From<CmdDet> for StorableCaCommand {
                 updates: updates.into(),
             },
             CmdDet::RouteAuthorizationsRenew(_, _) => StorableCaCommand::ReissueBeforeExpiring,
+            CmdDet::RouteAuthorizationsForceRenew(_, _) => StorableCaCommand::ForceReissue,
+
+            // ------------------------------------------------------------
+            // ASPA Support
+            // ------------------------------------------------------------
+            CmdDet::AspasUpdate(updates, _, _) => StorableCaCommand::AspasUpdate { updates },
+            CmdDet::AspasUpdateExisting(customer, update, _, _) => {
+                StorableCaCommand::AspasUpdateExisting { customer, update }
+            }
+            CmdDet::AspasRenew(_, _) => StorableCaCommand::ReissueBeforeExpiring,
 
             // ------------------------------------------------------------
             // Publishing
@@ -265,8 +319,14 @@ impl From<CmdDet> for StorableCaCommand {
 
 impl CmdDet {
     /// Turns this CA into a TrustAnchor
-    pub fn make_trust_anchor(handle: &Handle, uris: Vec<uri::Https>, signer: Arc<KrillSigner>, actor: &Actor) -> Cmd {
-        eventsourcing::SentCommand::new(handle, None, CmdDet::MakeTrustAnchor(uris, signer), actor)
+    pub fn make_trust_anchor(
+        handle: &Handle,
+        uris: Vec<uri::Https>,
+        rsync_uri: Option<uri::Rsync>,
+        signer: Arc<KrillSigner>,
+        actor: &Actor,
+    ) -> Cmd {
+        eventsourcing::SentCommand::new(handle, None, CmdDet::MakeTrustAnchor(uris, rsync_uri, signer), actor)
     }
 
     /// Adds a child to this CA. Will return an error in case you try
@@ -336,6 +396,14 @@ impl CmdDet {
         eventsourcing::SentCommand::new(handle, None, CmdDet::ChildRemove(child_handle), actor)
     }
 
+    pub fn child_suspend_inactive(handle: &Handle, child_handle: ChildHandle, actor: &Actor) -> Cmd {
+        eventsourcing::SentCommand::new(handle, None, CmdDet::ChildSuspendInactive(child_handle), actor)
+    }
+
+    pub fn child_unsuspend(handle: &Handle, child_handle: ChildHandle, actor: &Actor) -> Cmd {
+        eventsourcing::SentCommand::new(handle, None, CmdDet::ChildUnsuspend(child_handle), actor)
+    }
+
     pub fn update_id(handle: &Handle, signer: Arc<KrillSigner>, actor: &Actor) -> Cmd {
         eventsourcing::SentCommand::new(handle, None, CmdDet::GenerateNewIdKey(signer), actor)
     }
@@ -383,6 +451,21 @@ impl CmdDet {
         )
     }
 
+    pub fn drop_resource_class(
+        handle: &Handle,
+        class_name: ResourceClassName,
+        reason: DropReason,
+        signer: Arc<KrillSigner>,
+        actor: &Actor,
+    ) -> Cmd {
+        eventsourcing::SentCommand::new(
+            handle,
+            None,
+            CmdDet::DropResourceClass(class_name, reason, signer),
+            actor,
+        )
+    }
+
     //-------------------------------------------------------------------------------
     // Key Rolls
     //-------------------------------------------------------------------------------
@@ -427,6 +510,35 @@ impl CmdDet {
         )
     }
 
+    //-------------------------------------------------------------------------------
+    // Autonomous System Provider Authorization
+    //-------------------------------------------------------------------------------
+    pub fn aspas_definitions_update(
+        ca: &Handle,
+        updates: AspaDefinitionUpdates,
+        config: Arc<Config>,
+        signer: Arc<KrillSigner>,
+        actor: &Actor,
+    ) -> Cmd {
+        eventsourcing::SentCommand::new(ca, None, CmdDet::AspasUpdate(updates, config, signer), actor)
+    }
+
+    pub fn aspas_update_aspa(
+        ca: &Handle,
+        customer: AspaCustomer,
+        update: AspaProvidersUpdate,
+        config: Arc<Config>,
+        signer: Arc<KrillSigner>,
+        actor: &Actor,
+    ) -> Cmd {
+        eventsourcing::SentCommand::new(
+            ca,
+            None,
+            CmdDet::AspasUpdateExisting(customer, update, config, signer),
+            actor,
+        )
+    }
+
     //-------------------------------------------------------------------------------
     // Resource Tagged Attestations
     //-------------------------------------------------------------------------------
