@@ -1,21 +1,22 @@
-use std::collections::HashMap;
-use std::fmt;
+use std::{collections::HashMap, fmt};
 
-use rpki::crypto::KeyIdentifier;
+use rpki::repository::crypto::KeyIdentifier;
 
 use crate::{
     commons::{
         api::{
-            ChildHandle, Handle, IssuanceRequest, IssuedCert, ObjectName, ParentCaContact, ParentHandle,
-            ParentResourceClassName, RcvdCert, RepositoryContact, ResourceClassName, ResourceSet, RevocationRequest,
-            RevokedObject, RoaAggregateKey, RtaName, TaCertDetails,
+            AspaCustomer, AspaDefinition, AspaProvidersUpdate, ChildHandle, Handle, IssuanceRequest, IssuedCert,
+            ObjectName, ParentCaContact, ParentHandle, ParentResourceClassName, RcvdCert, RepositoryContact,
+            ResourceClassName, ResourceSet, RevocationRequest, RevokedObject, RoaAggregateKey, RtaName, SuspendedCert,
+            TaCertDetails, UnsuspendedCert,
         },
         crypto::{IdCert, KrillSigner},
         eventsourcing::StoredEvent,
         KrillResult,
     },
     daemon::ca::{
-        AggregateRoaInfo, CertifiedKey, PreparedRta, PublishedRoa, Rfc8183Id, RoaInfo, RouteAuthorization, SignedRta,
+        AggregateRoaInfo, AspaInfo, CertifiedKey, PreparedRta, PublishedRoa, Rfc8183Id, RoaInfo, RouteAuthorization,
+        SignedRta,
     },
 };
 
@@ -342,7 +343,7 @@ impl RoaUpdates {
 impl fmt::Display for RoaUpdates {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         if !self.updated.is_empty() {
-            write!(f, "Added single VRP ROAs: ")?;
+            write!(f, "Updated single VRP ROAs: ")?;
             for roa in self.updated.keys() {
                 write!(f, "{} ", ObjectName::from(roa))?;
             }
@@ -354,7 +355,7 @@ impl fmt::Display for RoaUpdates {
             }
         }
         if !self.aggregate_updated.is_empty() {
-            write!(f, "Added ASN aggregated ROAs: ")?;
+            write!(f, "Updated ASN aggregated ROAs: ")?;
             for roa in self.aggregate_updated.keys() {
                 write!(f, "{} ", ObjectName::from(roa))?;
             }
@@ -369,6 +370,58 @@ impl fmt::Display for RoaUpdates {
     }
 }
 
+//------------ AspaObjectsUpdates ------------------------------------------
+
+#[derive(Clone, Debug, Default, Deserialize, Eq, PartialEq, Serialize)]
+pub struct AspaObjectsUpdates {
+    #[serde(skip_serializing_if = "Vec::is_empty", default)]
+    updated: Vec<AspaInfo>,
+
+    #[serde(skip_serializing_if = "Vec::is_empty", default)]
+    removed: Vec<AspaCustomer>,
+}
+
+impl AspaObjectsUpdates {
+    pub fn new(updated: Vec<AspaInfo>, removed: Vec<AspaCustomer>) -> Self {
+        AspaObjectsUpdates { updated, removed }
+    }
+
+    pub fn for_new_aspa_info(new_aspa: AspaInfo) -> Self {
+        AspaObjectsUpdates {
+            updated: vec![new_aspa],
+            removed: vec![],
+        }
+    }
+
+    pub fn add_updated(&mut self, update: AspaInfo) {
+        self.updated.push(update)
+    }
+
+    pub fn add_removed(&mut self, customer: AspaCustomer) {
+        self.removed.push(customer)
+    }
+
+    pub fn is_empty(&self) -> bool {
+        self.updated.is_empty() && self.removed.is_empty()
+    }
+
+    pub fn contains_changes(&self) -> bool {
+        !self.is_empty()
+    }
+
+    pub fn unpack(self) -> (Vec<AspaInfo>, Vec<AspaCustomer>) {
+        (self.updated, self.removed)
+    }
+
+    pub fn updated(&self) -> &Vec<AspaInfo> {
+        &self.updated
+    }
+
+    pub fn removed(&self) -> &Vec<AspaCustomer> {
+        &self.removed
+    }
+}
+
 //------------ ChildCertificateUpdates -------------------------------------
 
 /// Describes an update to the set of ROAs under a ResourceClass.
@@ -376,31 +429,82 @@ impl fmt::Display for RoaUpdates {
 pub struct ChildCertificateUpdates {
     issued: Vec<IssuedCert>,
     removed: Vec<KeyIdentifier>,
+    suspended: Vec<SuspendedCert>,
+    unsuspended: Vec<UnsuspendedCert>,
 }
 
 impl ChildCertificateUpdates {
-    pub fn new(issued: Vec<IssuedCert>, removed: Vec<KeyIdentifier>) -> Self {
-        ChildCertificateUpdates { issued, removed }
+    pub fn new(
+        issued: Vec<IssuedCert>,
+        removed: Vec<KeyIdentifier>,
+        suspended: Vec<SuspendedCert>,
+        unsuspended: Vec<UnsuspendedCert>,
+    ) -> Self {
+        ChildCertificateUpdates {
+            issued,
+            removed,
+            suspended,
+            unsuspended,
+        }
     }
 
     pub fn is_empty(&self) -> bool {
-        self.issued.is_empty() && self.removed.is_empty()
+        self.issued.is_empty() && self.removed.is_empty() && self.suspended.is_empty() && self.unsuspended.is_empty()
     }
+
+    /// Add an issued certificate to the current set of issued certificates.
+    /// Note that this is typically a newly issued certificate, but it can
+    /// also be a previously issued certificate which had been suspended and
+    /// is now unsuspended.
     pub fn issue(&mut self, new: IssuedCert) {
         self.issued.push(new);
     }
+
+    /// Remove certificates for a key identifier. This will ensure that they
+    /// are revoked.
     pub fn remove(&mut self, ki: KeyIdentifier) {
         self.removed.push(ki);
     }
 
+    /// List all currently issued (not suspended) certificates.
     pub fn issued(&self) -> &Vec<IssuedCert> {
         &self.issued
     }
+
+    /// List all removals (revocations).
     pub fn removed(&self) -> &Vec<KeyIdentifier> {
         &self.removed
     }
-    pub fn unpack(self) -> (Vec<IssuedCert>, Vec<KeyIdentifier>) {
-        (self.issued, self.removed)
+
+    /// Suspend a certificate
+    pub fn suspend(&mut self, suspended_cert: SuspendedCert) {
+        self.suspended.push(suspended_cert);
+    }
+
+    /// List all suspended certificates in this update.
+    pub fn suspended(&self) -> &Vec<SuspendedCert> {
+        &self.suspended
+    }
+
+    /// Unsuspend a certificate
+    pub fn unsuspend(&mut self, unsuspended_cert: UnsuspendedCert) {
+        self.unsuspended.push(unsuspended_cert);
+    }
+
+    /// List all unsuspended certificates in this update.
+    pub fn unsuspended(&self) -> &Vec<UnsuspendedCert> {
+        &self.unsuspended
+    }
+
+    pub fn unpack(
+        self,
+    ) -> (
+        Vec<IssuedCert>,
+        Vec<KeyIdentifier>,
+        Vec<SuspendedCert>,
+        Vec<UnsuspendedCert>,
+    ) {
+        (self.issued, self.removed, self.suspended, self.unsuspended)
     }
 }
 
@@ -421,21 +525,30 @@ pub enum CaEvtDet {
     },
 
     // Being a parent Events
+    /// A child was added to this (parent) CA
     ChildAdded {
         child: ChildHandle,
         id_cert: IdCert,
         resources: ResourceSet,
     },
+
+    /// A certificate was issued to the child of this (parent) CA
     ChildCertificateIssued {
         child: ChildHandle,
         resource_class_name: ResourceClassName,
         ki: KeyIdentifier,
     },
+
+    /// A child key was revoked.
     ChildKeyRevoked {
         child: ChildHandle,
         resource_class_name: ResourceClassName,
         ki: KeyIdentifier,
     },
+
+    /// Child certificates (for potentially multiple children) were updated
+    /// under a CA resource class. I.e. child certificates were issued,
+    /// removed, or suspended.
     ChildCertificatesUpdated {
         resource_class_name: ResourceClassName,
         updates: ChildCertificateUpdates,
@@ -452,6 +565,14 @@ pub enum CaEvtDet {
         child: ChildHandle,
     },
 
+    // (Un)Suspend a child events
+    ChildSuspended {
+        child: ChildHandle,
+    },
+    ChildUnsuspended {
+        child: ChildHandle,
+    },
+
     // Being a child Events
     IdUpdated {
         id: Rfc8183Id,
@@ -553,6 +674,23 @@ pub enum CaEvtDet {
         updates: RoaUpdates,
     },
 
+    // ASPA
+    AspaConfigAdded {
+        aspa_config: AspaDefinition,
+    },
+    AspaConfigUpdated {
+        customer: AspaCustomer,
+        update: AspaProvidersUpdate,
+    },
+    AspaConfigRemoved {
+        customer: AspaCustomer,
+    },
+    AspaObjectsUpdated {
+        // Tracks ASPA *object* which are (re-)issued in a resource class.
+        resource_class_name: ResourceClassName,
+        updates: AspaObjectsUpdates,
+    },
+
     // Publishing
     RepoUpdated {
         // Adds the repository contact for this CA so that publication can commence,
@@ -685,6 +823,14 @@ impl CaEvtDet {
     pub(super) fn child_removed(handle: &Handle, version: u64, child: ChildHandle) -> CaEvt {
         StoredEvent::new(handle, version, CaEvtDet::ChildRemoved { child })
     }
+
+    pub(super) fn child_suspended(handle: &Handle, version: u64, child: ChildHandle) -> CaEvt {
+        StoredEvent::new(handle, version, CaEvtDet::ChildSuspended { child })
+    }
+
+    pub(super) fn child_unsuspended(handle: &Handle, version: u64, child: ChildHandle) -> CaEvt {
+        StoredEvent::new(handle, version, CaEvtDet::ChildUnsuspended { child })
+    }
 }
 
 impl fmt::Display for CaEvtDet {
@@ -731,7 +877,7 @@ impl fmt::Display for CaEvtDet {
                 )?;
                 let issued = updates.issued();
                 if !issued.is_empty() {
-                    write!(f, " (re-)issued keys: ")?;
+                    write!(f, " issued keys: ")?;
                     for iss in issued {
                         write!(f, " {}", iss.subject_key_identifier())?;
                     }
@@ -743,6 +889,21 @@ impl fmt::Display for CaEvtDet {
                         write!(f, " {}", rev)?;
                     }
                 }
+                let suspended = updates.suspended();
+                if !suspended.is_empty() {
+                    write!(f, " suspended keys: ")?;
+                    for cert in suspended {
+                        write!(f, " {}", cert.subject_key_identifier())?;
+                    }
+                }
+                let unsuspended = updates.unsuspended();
+                if !unsuspended.is_empty() {
+                    write!(f, " unsuspended keys: ")?;
+                    for cert in unsuspended {
+                        write!(f, " {}", cert.subject_key_identifier())?;
+                    }
+                }
+
                 Ok(())
             }
             CaEvtDet::ChildKeyRevoked {
@@ -761,6 +922,8 @@ impl fmt::Display for CaEvtDet {
                 write!(f, "updated child '{}' resources to '{}'", child, resources)
             }
             CaEvtDet::ChildRemoved { child } => write!(f, "removed child '{}'", child),
+            CaEvtDet::ChildSuspended { child } => write!(f, "suspended child '{}'", child),
+            CaEvtDet::ChildUnsuspended { child } => write!(f, "unsuspended child '{}'", child),
 
             // Being a child Events
             CaEvtDet::IdUpdated { id } => write!(f, "updated RFC8183 id to key '{}'", id.key_hash()),
@@ -869,17 +1032,49 @@ impl fmt::Display for CaEvtDet {
                 resource_class_name,
                 updates,
             } => {
-                write!(f, "updated ROAs under resource class '{}'", resource_class_name)?;
-                if !updates.updated.is_empty() {
+                write!(f, "updated ROA objects under resource class '{}'", resource_class_name)?;
+                if !updates.updated.is_empty() || !updates.aggregate_updated.is_empty() {
                     write!(f, " added: ")?;
                     for auth in updates.updated.keys() {
-                        write!(f, "{} ", auth)?;
+                        write!(f, "{} ", ObjectName::from(auth))?;
+                    }
+                    for agg_key in updates.aggregate_updated.keys() {
+                        write!(f, "{} ", ObjectName::from(agg_key))?;
                     }
                 }
-                if !updates.removed.is_empty() {
+                if !updates.removed.is_empty() || !updates.aggregate_removed.is_empty() {
                     write!(f, " removed: ")?;
                     for auth in updates.removed.keys() {
-                        write!(f, "{} ", auth)?;
+                        write!(f, "{} ", ObjectName::from(auth))?;
+                    }
+                    for agg_key in updates.aggregate_removed.keys() {
+                        write!(f, "{} ", ObjectName::from(agg_key))?;
+                    }
+                }
+                Ok(())
+            }
+
+            // Autonomous System Provider Authorization
+            CaEvtDet::AspaConfigAdded { aspa_config: addition } => write!(f, "{}", addition),
+            CaEvtDet::AspaConfigUpdated { customer, update } => {
+                write!(f, "updated ASPA config for customer ASN: {} {}", customer, update)
+            }
+            CaEvtDet::AspaConfigRemoved { customer } => write!(f, "removed ASPA config for customer ASN: {}", customer),
+            CaEvtDet::AspaObjectsUpdated {
+                resource_class_name,
+                updates,
+            } => {
+                write!(f, "updated ASPA objects under resource class '{}'", resource_class_name)?;
+                if !updates.updated().is_empty() {
+                    write!(f, " updated:")?;
+                    for upd in updates.updated() {
+                        write!(f, " {}", ObjectName::aspa(upd.customer()))?;
+                    }
+                }
+                if !updates.removed().is_empty() {
+                    write!(f, " removed:")?;
+                    for rem in updates.removed() {
+                        write!(f, " {}", ObjectName::aspa(*rem))?;
                     }
                 }
                 Ok(())
