@@ -11,7 +11,8 @@ use std::{
 use bytes::Bytes;
 use chrono::Duration;
 
-use rpki::{
+use rpki::repository::{
+    aspa::Aspa,
     cert::Cert,
     crl::{Crl, TbsCertList},
     crypto::{DigestAlgorithm, KeyIdentifier, PublicKey},
@@ -25,7 +26,7 @@ use crate::{
     commons::{
         api::{
             rrdp::PublishElement, Base64, Handle, IssuedCert, ObjectName, RcvdCert, RepositoryContact,
-            ResourceClassName, Revocation, Revocations,
+            ResourceClassName, Revocation, Revocations, Timestamp,
         },
         crypto::KrillSigner,
         error::Error,
@@ -34,12 +35,12 @@ use crate::{
     },
     constants::CA_OBJECTS_DIR,
     daemon::{
-        ca::{CaEvt, CertAuth},
+        ca::{CaEvt, CertAuth, CertifiedKey, ChildCertificateUpdates, RoaUpdates},
         config::{Config, IssuanceTimingConfig},
     },
 };
 
-use super::{CertifiedKey, ChildCertificateUpdates, RoaUpdates};
+use super::AspaObjectsUpdates;
 
 //------------ CaObjectsStore ----------------------------------------------
 
@@ -89,6 +90,12 @@ impl PreSaveEventListener<CertAuth> for CaObjectsStore {
                     } => {
                         objects.update_roas(resource_class_name, updates, timing, signer)?;
                     }
+                    super::CaEvtDet::AspaObjectsUpdated {
+                        resource_class_name,
+                        updates,
+                    } => {
+                        objects.update_aspas(resource_class_name, updates, timing, signer)?;
+                    }
                     super::CaEvtDet::ChildCertificatesUpdated {
                         resource_class_name,
                         updates,
@@ -328,7 +335,7 @@ impl CaObjects {
         }
     }
 
-    #[allow(clippy::clippy::mutable_key_type)]
+    #[allow(clippy::mutable_key_type)]
     /// Returns all PublishedElements mapped to each RepositoryContact.
     /// There could be more than one repository - although usually there isn't.
     pub fn repo_elements_map(&self) -> HashMap<RepositoryContact, Vec<PublishElement>> {
@@ -337,10 +344,10 @@ impl CaObjects {
         if let Some(repo) = &self.repo {
             res.insert(repo.clone(), vec![]);
 
-            for rco in self.classes.values() {
+            for resource_class_objects in self.classes.values() {
                 // Note the map 'res' will get entries for other (old) repositories
                 // if there are any keys with such repositories.
-                rco.add_elements(&mut res, repo);
+                resource_class_objects.add_elements(&mut res, repo);
             }
         }
 
@@ -359,6 +366,24 @@ impl CaObjects {
         all_elements
     }
 
+    /// Returns the closest next update time from among manifests held by this CA
+    pub fn closest_next_update(&self) -> Option<Timestamp> {
+        let mut closest = None;
+
+        for resource_class_objects in self.classes.values() {
+            let rco_time = Timestamp::from(resource_class_objects.next_update_time());
+            if let Some(current_closest) = closest {
+                if current_closest > rco_time {
+                    closest = Some(rco_time);
+                }
+            } else {
+                closest = Some(rco_time);
+            }
+        }
+
+        closest
+    }
+
     pub fn deprecated_repos(&self) -> &Vec<DeprecatedRepository> {
         &self.deprecated_repos
     }
@@ -422,8 +447,7 @@ impl CaObjects {
         timing: &IssuanceTimingConfig,
         signer: &KrillSigner,
     ) -> KrillResult<()> {
-        let rco = self.get_class_mut(rcn)?;
-        rco.keyroll_stage(key, timing, signer)
+        self.get_class_mut(rcn)?.keyroll_stage(key, timing, signer)
     }
 
     // Activates the keyset by retiring the current set, and promoting
@@ -434,17 +458,16 @@ impl CaObjects {
         timing: &IssuanceTimingConfig,
         signer: &KrillSigner,
     ) -> KrillResult<()> {
-        let rco = self.get_class_mut(rcn)?;
-        rco.keyroll_activate(timing, signer)
+        self.get_class_mut(rcn)?.keyroll_activate(timing, signer)
     }
 
     // Finish a keyroll
     fn keyroll_finish(&mut self, rcn: &ResourceClassName) -> KrillResult<()> {
-        let rco = self.get_class_mut(rcn)?;
+        let resource_class_objects = self.get_class_mut(rcn)?;
 
-        // finish the key roll for this rco. This will remove the old key, and return
-        // an old_repo if there was one.
-        if let Some(old_repo) = rco.keyroll_finish()? {
+        // finish the key roll for this resource class objects. This will remove the old
+        // key, and return an old_repo if there was one.
+        if let Some(old_repo) = resource_class_objects.keyroll_finish()? {
             self.deprecate_repo_if_no_longer_used(old_repo);
         }
 
@@ -458,9 +481,20 @@ impl CaObjects {
         roa_updates: &RoaUpdates,
         timing: &IssuanceTimingConfig,
         signer: &KrillSigner,
+    ) -> KrillResult<()> {
+        self.get_class_mut(rcn)?.update_roas(roa_updates, timing, signer)
+    }
+
+    // Update the ASPAs in the current set
+    fn update_aspas(
+        &mut self,
+        rcn: &ResourceClassName,
+        updates: &AspaObjectsUpdates,
+        timing: &IssuanceTimingConfig,
+        signer: &KrillSigner,
     ) -> KrillResult<()> {
         let rco = self.get_class_mut(rcn)?;
-        rco.update_roas(roa_updates, timing, signer)
+        rco.update_aspas(updates, timing, signer)
     }
 
     // Update the delegated certificates in the current set
@@ -471,14 +505,12 @@ impl CaObjects {
         timing: &IssuanceTimingConfig,
         signer: &KrillSigner,
     ) -> KrillResult<()> {
-        let rco = self.get_class_mut(rcn)?;
-        rco.update_certs(cert_updates, timing, signer)
+        self.get_class_mut(rcn)?.update_certs(cert_updates, timing, signer)
     }
 
     // Update the received certificate.
     fn update_received_cert(&mut self, rcn: &ResourceClassName, cert: &RcvdCert) -> KrillResult<()> {
-        let rco = self.get_class_mut(rcn)?;
-        rco.update_received_cert(cert)
+        self.get_class_mut(rcn)?.update_received_cert(cert)
     }
 
     /// Reissue the MFT and CRL in this set if needed, i.e. if it's close to the next
@@ -488,10 +520,10 @@ impl CaObjects {
         let hours = timing.timing_publish_hours_before_next;
         let mut required = false;
 
-        for (_, rco) in self.classes.iter_mut() {
-            if rco.requires_re_issuance(hours) {
+        for (_, resource_class_objects) in self.classes.iter_mut() {
+            if resource_class_objects.requires_re_issuance(hours) {
                 required = true;
-                rco.reissue(timing, signer)?;
+                resource_class_objects.reissue(timing, signer)?;
             }
         }
 
@@ -505,8 +537,8 @@ impl CaObjects {
     // existing keys.
     fn update_repo(&mut self, repo: &RepositoryContact) {
         if let Some(old) = &self.repo {
-            for rco in self.classes.values_mut() {
-                rco.set_old_repo(old);
+            for resource_class_objects in self.classes.values_mut() {
+                resource_class_objects.set_old_repo(old);
             }
         }
         self.repo = Some(repo.clone());
@@ -534,7 +566,7 @@ impl ResourceClassObjects {
         ResourceClassObjects { keys }
     }
 
-    #[allow(clippy::clippy::mutable_key_type)]
+    #[allow(clippy::mutable_key_type)]
     /// Adds all the elements for this resource class to the map which is passed on. It will use
     /// the default repository, or an optional old repository if any of the keys had one as part
     /// of a repository migration.
@@ -622,6 +654,19 @@ impl ResourceClassObjects {
         }
     }
 
+    fn update_aspas(
+        &mut self,
+        updates: &AspaObjectsUpdates,
+        timing: &IssuanceTimingConfig,
+        signer: &KrillSigner,
+    ) -> KrillResult<()> {
+        match self.keys.borrow_mut() {
+            ResourceClassKeyState::Current(state) => state.current_set.update_aspas(updates, timing, signer),
+            ResourceClassKeyState::Staging(state) => state.current_set.update_aspas(updates, timing, signer),
+            ResourceClassKeyState::Old(state) => state.current_set.update_aspas(updates, timing, signer),
+        }
+    }
+
     fn update_certs(
         &mut self,
         cert_updates: &ChildCertificateUpdates,
@@ -647,6 +692,14 @@ impl ResourceClassObjects {
         }
     }
 
+    fn next_update_time(&self) -> Time {
+        match &self.keys {
+            ResourceClassKeyState::Current(state) => state.current_set.next_update_time(),
+            ResourceClassKeyState::Old(state) => state.current_set.next_update_time(),
+            ResourceClassKeyState::Staging(state) => state.current_set.next_update_time(),
+        }
+    }
+
     fn reissue(&mut self, timing: &IssuanceTimingConfig, signer: &KrillSigner) -> KrillResult<()> {
         match self.keys.borrow_mut() {
             ResourceClassKeyState::Current(state) => state.current_set.reissue(timing, signer),
@@ -769,29 +822,25 @@ pub struct CurrentKeyObjectSet {
     basic: BasicKeyObjectSet,
     #[serde(with = "objects_to_roas_serde")]
     roas: HashMap<ObjectName, PublishedRoa>,
+    #[serde(with = "objects_to_aspas_serde", skip_serializing_if = "HashMap::is_empty", default)]
+    aspas: HashMap<ObjectName, PublishedAspa>,
     #[serde(with = "objects_to_certs_serde")]
     certs: HashMap<ObjectName, PublishedCert>,
 }
 
 impl CurrentKeyObjectSet {
     pub fn new(
-        signing_cert: RcvdCert,
-        number: u64,
-        revocations: Revocations,
-        manifest: PublishedManifest,
-        crl: PublishedCrl,
+        basic: BasicKeyObjectSet,
         roas: HashMap<ObjectName, PublishedRoa>,
+        aspas: HashMap<ObjectName, PublishedAspa>,
         certs: HashMap<ObjectName, PublishedCert>,
     ) -> Self {
-        let basic = BasicKeyObjectSet {
-            signing_cert,
-            number,
-            revocations,
-            manifest,
-            crl,
-            old_repo: None,
-        };
-        CurrentKeyObjectSet { basic, roas, certs }
+        CurrentKeyObjectSet {
+            basic,
+            roas,
+            aspas,
+            certs,
+        }
     }
 
     /// Adds all the elements for this set to the map which is passed on. It will use
@@ -802,8 +851,8 @@ impl CurrentKeyObjectSet {
         let repo = self.old_repo.as_ref().unwrap_or(dflt_repo);
 
         let base_uri = self.signing_cert.ca_repository();
-        let mft_uri = base_uri.join(self.manifest.name().as_bytes());
-        let crl_uri = base_uri.join(self.crl.name().as_bytes());
+        let mft_uri = base_uri.join(self.manifest.name().as_bytes()).unwrap();
+        let crl_uri = base_uri.join(self.crl.name().as_bytes()).unwrap();
 
         let elements = map.entry(repo.clone()).or_insert_with(Vec::new);
         elements.push(PublishElement::new(Base64::from(&self.manifest.0), mft_uri));
@@ -812,14 +861,21 @@ impl CurrentKeyObjectSet {
         for (name, roa) in &self.roas {
             elements.push(PublishElement::new(
                 Base64::from(&roa.0),
-                base_uri.join(name.as_bytes()),
+                base_uri.join(name.as_bytes()).unwrap(),
+            ));
+        }
+
+        for (name, aspa) in &self.aspas {
+            elements.push(PublishElement::new(
+                Base64::from(&aspa.0),
+                base_uri.join(name.as_bytes()).unwrap(),
             ));
         }
 
         for (name, cert) in &self.certs {
             elements.push(PublishElement::new(
                 Base64::from(cert.as_ref()),
-                base_uri.join(name.as_bytes()),
+                base_uri.join(name.as_bytes()).unwrap(),
             ));
         }
     }
@@ -844,6 +900,29 @@ impl CurrentKeyObjectSet {
         self.reissue(timing, signer)
     }
 
+    fn update_aspas(
+        &mut self,
+        updates: &AspaObjectsUpdates,
+        timing: &IssuanceTimingConfig,
+        signer: &KrillSigner,
+    ) -> KrillResult<()> {
+        for aspa_info in updates.updated() {
+            let name = ObjectName::aspa(aspa_info.customer());
+            let aspa = PublishedAspa::new(aspa_info.aspa().clone());
+            if let Some(old) = self.aspas.insert(name, aspa) {
+                self.revocations.add(Revocation::from(&old));
+            }
+        }
+        for removed in updates.removed() {
+            let name = ObjectName::aspa(*removed);
+            if let Some(old) = self.aspas.remove(&name) {
+                self.revocations.add(Revocation::from(&old));
+            }
+        }
+
+        self.reissue(timing, signer)
+    }
+
     fn update_certs(
         &mut self,
         cert_updates: &ChildCertificateUpdates,
@@ -864,6 +943,21 @@ impl CurrentKeyObjectSet {
             }
         }
 
+        for cert in cert_updates.unsuspended() {
+            let name = ObjectName::from(cert.cert());
+            self.revocations.remove(&Revocation::from(cert.cert()));
+            if let Some(old) = self.certs.insert(name, cert.clone().into()) {
+                // this should not happen, but just to be safe.
+                self.revocations.add(Revocation::from(&old));
+            }
+        }
+
+        for suspended in cert_updates.suspended() {
+            let name = ObjectName::from(suspended.cert());
+            self.certs.remove(&name);
+            self.revocations.add(Revocation::from(suspended.cert()));
+        }
+
         self.reissue(timing, signer)
     }
 
@@ -871,7 +965,7 @@ impl CurrentKeyObjectSet {
         self.revocations.purge();
 
         self.crl = self.reissue_crl(&self.revocations, timing, signer)?;
-        self.manifest = self.reissue_mft(&self.crl, timing, signer)?;
+        self.manifest = self.reissue_mft(&self.crl, signer)?;
         self.number = self.next();
 
         Ok(())
@@ -890,7 +984,7 @@ impl CurrentKeyObjectSet {
         revocations.purge();
 
         let crl = self.basic.reissue_crl(&revocations, timing, signer)?;
-        let manifest = self.basic.reissue_mft(&crl, timing, signer)?;
+        let manifest = self.basic.reissue_mft(&crl, signer)?;
 
         Ok(BasicKeyObjectSet {
             signing_cert: self.signing_cert.clone(),
@@ -902,14 +996,9 @@ impl CurrentKeyObjectSet {
         })
     }
 
-    fn reissue_mft(
-        &self,
-        new_crl: &PublishedCrl,
-        timing: &IssuanceTimingConfig,
-        signer: &KrillSigner,
-    ) -> KrillResult<PublishedManifest> {
-        ManifestBuilder::with_objects(new_crl, &self.roas, &self.certs)
-            .build_new_mft(&self.signing_cert, self.next(), timing, signer)
+    fn reissue_mft(&self, new_crl: &PublishedCrl, signer: &KrillSigner) -> KrillResult<PublishedManifest> {
+        ManifestBuilder::with_objects(new_crl, &self.roas, &self.aspas, &self.certs)
+            .build_new_mft(&self.signing_cert, self.next(), signer)
             .map(|m| m.into())
     }
 }
@@ -919,6 +1008,7 @@ impl From<BasicKeyObjectSet> for CurrentKeyObjectSet {
         CurrentKeyObjectSet {
             basic,
             roas: HashMap::new(),
+            aspas: HashMap::new(),
             certs: HashMap::new(),
         }
     }
@@ -974,6 +1064,42 @@ mod objects_to_roas_serde {
     }
 }
 
+mod objects_to_aspas_serde {
+    use super::*;
+
+    use serde::de::{Deserialize, Deserializer};
+    use serde::ser::Serializer;
+    #[derive(Debug, Deserialize)]
+    struct NameRoaItem {
+        name: ObjectName,
+        aspa: PublishedAspa,
+    }
+
+    #[derive(Debug, Serialize)]
+    struct NameRoaItemRef<'a> {
+        name: &'a ObjectName,
+        aspa: &'a PublishedAspa,
+    }
+
+    pub fn serialize<S>(map: &HashMap<ObjectName, PublishedAspa>, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.collect_seq(map.iter().map(|(name, aspa)| NameRoaItemRef { name, aspa }))
+    }
+
+    pub fn deserialize<'de, D>(deserializer: D) -> Result<HashMap<ObjectName, PublishedAspa>, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let mut map = HashMap::new();
+        for item in Vec::<NameRoaItem>::deserialize(deserializer)? {
+            map.insert(item.name, item.aspa);
+        }
+        Ok(map)
+    }
+}
+
 mod objects_to_certs_serde {
     use super::*;
 
@@ -1047,8 +1173,8 @@ impl BasicKeyObjectSet {
         let repo = self.old_repo.as_ref().unwrap_or(dflt_repo);
 
         let base_uri = self.signing_cert.ca_repository();
-        let mft_uri = base_uri.join(self.manifest.name().as_bytes());
-        let crl_uri = base_uri.join(self.crl.name().as_bytes());
+        let mft_uri = base_uri.join(self.manifest.name().as_bytes()).unwrap();
+        let crl_uri = base_uri.join(self.crl.name().as_bytes()).unwrap();
 
         let elements = map.entry(repo.clone()).or_insert_with(Vec::new);
         elements.push(PublishElement::new(Base64::from(&self.manifest.0), mft_uri));
@@ -1062,12 +1188,12 @@ impl BasicKeyObjectSet {
         let issuer = signing_cert.subject().clone();
         let revocations = Revocations::default();
         let number = 1;
-        let next_hours = timing.timing_publish_next_hours;
+        let next_update = timing.publish_next();
 
-        let crl = CrlBuilder::build(signing_key, issuer, &revocations, number, next_hours, signer)?;
+        let crl = CrlBuilder::build(signing_key, issuer, &revocations, number, next_update, signer)?;
 
         let manifest = ManifestBuilder::with_crl_only(&crl)
-            .build_new_mft(&signing_cert, number, timing, signer)
+            .build_new_mft(&signing_cert, number, signer)
             .map(|m| m.into())?;
 
         Ok(BasicKeyObjectSet::new(signing_cert, number, revocations, manifest, crl))
@@ -1078,6 +1204,10 @@ impl BasicKeyObjectSet {
             || Some(self.signing_cert.uri()) != self.manifest.cert().ca_issuer()
     }
 
+    pub fn next_update_time(&self) -> Time {
+        self.manifest.next_update()
+    }
+
     fn next(&self) -> u64 {
         self.number + 1
     }
@@ -1100,7 +1230,7 @@ impl BasicKeyObjectSet {
         revocations.purge();
 
         let crl = self.reissue_crl(&revocations, timing, signer)?;
-        let manifest = self.reissue_mft(&crl, timing, signer)?;
+        let manifest = self.reissue_mft(&crl, signer)?;
 
         Ok(BasicKeyObjectSet {
             signing_cert: self.signing_cert.clone(),
@@ -1122,19 +1252,14 @@ impl BasicKeyObjectSet {
         let issuer = self.crl.issuer().clone();
         let number = self.next();
 
-        let next_hours = timing.timing_publish_next_hours;
+        let next_update = timing.publish_next();
 
-        CrlBuilder::build(signing_key, issuer, revocations, number, next_hours, signer)
+        CrlBuilder::build(signing_key, issuer, revocations, number, next_update, signer)
     }
 
-    fn reissue_mft(
-        &self,
-        new_crl: &PublishedCrl,
-        timing: &IssuanceTimingConfig,
-        signer: &KrillSigner,
-    ) -> KrillResult<PublishedManifest> {
+    fn reissue_mft(&self, new_crl: &PublishedCrl, signer: &KrillSigner) -> KrillResult<PublishedManifest> {
         ManifestBuilder::with_crl_only(new_crl)
-            .build_new_mft(&self.signing_cert, self.next(), timing, signer)
+            .build_new_mft(&self.signing_cert, self.next(), signer)
             .map(|m| m.into())
     }
 
@@ -1183,7 +1308,7 @@ impl Deref for PublishedCert {
 
 impl AsRef<Cert> for PublishedCert {
     fn as_ref(&self) -> &Cert {
-        &self.0.as_ref()
+        self.0.as_ref()
     }
 }
 
@@ -1228,6 +1353,47 @@ impl PartialEq for PublishedRoa {
 
 impl Eq for PublishedRoa {}
 
+//------------ PublishedAspa ----------------------------------------------
+
+#[derive(Clone, Debug, Deserialize, Serialize)]
+pub struct PublishedAspa(Aspa);
+
+impl PublishedAspa {
+    pub fn new(aspa: Aspa) -> Self {
+        PublishedAspa(aspa)
+    }
+
+    pub fn to_bytes(&self) -> Bytes {
+        self.0.to_captured().into_bytes()
+    }
+
+    pub fn mft_hash(&self) -> Bytes {
+        mft_hash(self.to_bytes().as_ref())
+    }
+}
+
+impl From<&PublishedAspa> for Revocation {
+    fn from(aspa: &PublishedAspa) -> Self {
+        Revocation::from(&aspa.0)
+    }
+}
+
+impl Deref for PublishedAspa {
+    type Target = Aspa;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl PartialEq for PublishedAspa {
+    fn eq(&self, other: &Self) -> bool {
+        self.to_bytes() == other.to_bytes()
+    }
+}
+
+impl Eq for PublishedAspa {}
+
 //------------ PublishedManifest ------------------------------------------
 
 #[derive(Clone, Debug, Deserialize, Serialize)]
@@ -1279,6 +1445,14 @@ impl PublishedCrl {
         self.0.to_captured().into_bytes()
     }
 
+    pub fn this_update(&self) -> Time {
+        self.0.this_update()
+    }
+
+    pub fn next_update(&self) -> Time {
+        self.0.next_update()
+    }
+
     pub fn name(&self) -> ObjectName {
         ObjectName::from(&self.0)
     }
@@ -1320,13 +1494,12 @@ impl CrlBuilder {
         issuer: Name,
         revocations: &Revocations,
         number: u64,
-        next_hours: i64,
+        next_update: Time,
         signer: &KrillSigner,
     ) -> KrillResult<PublishedCrl> {
         let aki = KeyIdentifier::from_public_key(signing_key);
 
         let this_update = Time::five_minutes_ago();
-        let next_update = Time::now() + Duration::hours(next_hours);
         let serial_number = Serial::from(number);
 
         let crl = TbsCertList::new(
@@ -1347,6 +1520,8 @@ impl CrlBuilder {
 
 #[allow(clippy::mutable_key_type)]
 pub struct ManifestBuilder {
+    this_update: Time,
+    next_update: Time,
     entries: HashMap<Bytes, Bytes>,
 }
 
@@ -1355,6 +1530,7 @@ impl ManifestBuilder {
     pub fn with_objects(
         crl: &PublishedCrl,
         roas: &HashMap<ObjectName, PublishedRoa>,
+        aspas: &HashMap<ObjectName, PublishedAspa>,
         certs: &HashMap<ObjectName, PublishedCert>,
     ) -> Self {
         let mut entries: HashMap<Bytes, Bytes> = HashMap::new();
@@ -1368,29 +1544,37 @@ impl ManifestBuilder {
             entries.insert(name.clone().into(), hash);
         }
 
+        // Add ASPAs
+        for (name, aspa) in aspas {
+            let hash = aspa.mft_hash();
+            entries.insert(name.clone().into(), hash);
+        }
+
         // Add all issued certs
         for (name, cert) in certs {
             let hash = cert.mft_hash();
             entries.insert(name.clone().into(), hash);
         }
 
-        ManifestBuilder { entries }
+        ManifestBuilder {
+            this_update: crl.this_update(),
+            next_update: crl.next_update(),
+            entries,
+        }
     }
 
     #[allow(clippy::mutable_key_type)]
     pub fn with_crl_only(crl: &PublishedCrl) -> Self {
         let mut entries: HashMap<Bytes, Bytes> = HashMap::new();
         entries.insert(crl.name().into(), crl.mft_hash());
-        ManifestBuilder { entries }
+        ManifestBuilder {
+            this_update: crl.this_update(),
+            next_update: crl.next_update(),
+            entries,
+        }
     }
 
-    fn build_new_mft(
-        self,
-        signing_cert: &RcvdCert,
-        number: u64,
-        issuance_timing: &IssuanceTimingConfig,
-        signer: &KrillSigner,
-    ) -> KrillResult<Manifest> {
+    fn build_new_mft(self, signing_cert: &RcvdCert, number: u64, signer: &KrillSigner) -> KrillResult<Manifest> {
         let signing_key = signing_cert.cert().subject_public_key_info();
 
         let mft_uri = signing_cert.mft_uri();
@@ -1400,30 +1584,25 @@ impl ManifestBuilder {
         let aki = KeyIdentifier::from_public_key(signing_key);
         let serial_number = Serial::from(number);
 
-        let this_update = Time::five_minutes_ago();
-        let now = Time::now();
-        let next_update = Time::now() + Duration::hours(issuance_timing.timing_publish_next_hours);
-        let valid_until = Time::now() + Duration::days(issuance_timing.timing_publish_valid_days);
-
         let entries = self.entries.iter().map(|(k, v)| FileAndHash::new(k, v));
 
         let manifest: Manifest = {
             let mft_content = ManifestContent::new(
                 serial_number,
-                this_update,
-                next_update,
+                self.this_update,
+                self.next_update,
                 DigestAlgorithm::default(),
                 entries,
             );
             let mut object_builder = SignedObjectBuilder::new(
                 signer.random_serial()?,
-                Validity::new(this_update, valid_until),
+                Validity::new(self.this_update, self.next_update),
                 crl_uri,
                 aia.clone(),
                 mft_uri,
             );
             object_builder.set_issuer(Some(signing_cert.cert().subject().clone()));
-            object_builder.set_signing_time(Some(now));
+            object_builder.set_signing_time(Some(Time::now()));
 
             signer.sign_manifest(mft_content, object_builder, &aki)?
         };
