@@ -1,31 +1,35 @@
 use chrono::Duration;
 use serde::{Deserialize, Serialize};
 
-use rpki::cert::Cert;
-use rpki::crypto::KeyIdentifier;
-use rpki::x509::{Time, Validity};
+use rpki::repository::{
+    cert::Cert,
+    crypto::KeyIdentifier,
+    x509::{Time, Validity},
+};
 
 use crate::{
     commons::{
         api::{
-            EntitlementClass, Handle, HexEncodedHash, IssuanceRequest, IssuedCert, ParentHandle, RcvdCert,
-            ReplacedObject, RepoInfo, RequestResourceLimit, ResourceClassInfo, ResourceClassName, ResourceSet,
-            Revocation, RevocationRequest,
+            EntitlementClass, Handle, IssuanceRequest, IssuedCert, ParentHandle, RcvdCert, ReplacedObject, RepoInfo,
+            RequestResourceLimit, ResourceClassInfo, ResourceClassName, ResourceSet, RevocationRequest, SuspendedCert,
+            UnsuspendedCert,
         },
         crypto::{CsrInfo, KrillSigner, SignSupport},
         error::Error,
         KrillResult,
     },
     daemon::{
-        ca::events::{ChildCertificateUpdates, RoaUpdates},
+        ca::events::RoaUpdates,
         ca::{
-            self, ta_handle, CaEvtDet, CertifiedKey, ChildCertificates, CurrentKey, KeyState, NewKey, OldKey,
-            PendingKey, Roas, Routes,
+            self, ta_handle, AspaObjects, AspaObjectsUpdates, CaEvtDet, CertifiedKey, ChildCertificates, CurrentKey,
+            KeyState, NewKey, OldKey, PendingKey, Roas, Routes,
         },
         config::{Config, IssuanceTimingConfig},
     },
 };
 
+use super::AspaDefinitions;
+
 //------------ ResourceClass -----------------------------------------------
 
 /// A CA may have multiple parents, e.g. two RIRs, and it may not get all its
@@ -48,6 +52,7 @@ pub struct ResourceClass {
     parent_rc_name: ResourceClassName,
 
     roas: Roas,
+    aspas: AspaObjects,
     certificates: ChildCertificates,
 
     last_key_change: Time,
@@ -71,6 +76,7 @@ impl ResourceClass {
             parent_handle,
             parent_rc_name,
             roas: Roas::default(),
+            aspas: AspaObjects::default(),
             certificates: ChildCertificates::default(),
             last_key_change: Time::now(),
             key_state: KeyState::create(pending_key),
@@ -84,6 +90,7 @@ impl ResourceClass {
             parent_handle: ta_handle(),
             parent_rc_name,
             roas: Roas::default(),
+            aspas: AspaObjects::default(),
             certificates: ChildCertificates::default(),
             last_key_change: Time::now(),
             key_state: KeyState::create(pending_key),
@@ -178,7 +185,8 @@ impl ResourceClass {
         &self,
         handle: &Handle,
         rcvd_cert: RcvdCert,
-        routes: &Routes,
+        all_routes: &Routes,
+        all_aspas: &AspaDefinitions,
         config: &Config,
         signer: &KrillSigner,
     ) -> KrillResult<Vec<CaEvtDet>> {
@@ -200,14 +208,34 @@ impl ResourceClass {
                     );
 
                     let current_key = CertifiedKey::create(rcvd_cert);
-                    Ok(vec![CaEvtDet::KeyPendingToActive {
+
+                    let roa_updates = self.roas.update(all_routes, &current_key, config, signer)?;
+                    let aspa_updates = self.aspas.update(all_aspas, &current_key, config, signer)?;
+
+                    let mut events = vec![CaEvtDet::KeyPendingToActive {
                         resource_class_name: self.name.clone(),
                         current_key,
-                    }])
+                    }];
+
+                    if roa_updates.contains_changes() {
+                        events.push(CaEvtDet::RoasUpdated {
+                            resource_class_name: self.name.clone(),
+                            updates: roa_updates,
+                        })
+                    }
+
+                    if aspa_updates.contains_changes() {
+                        events.push(CaEvtDet::AspaObjectsUpdated {
+                            resource_class_name: self.name.clone(),
+                            updates: aspa_updates,
+                        })
+                    }
+
+                    Ok(events)
                 }
             }
             KeyState::Active(current) => {
-                self.update_rcvd_cert_current(handle, current, rcvd_cert, routes, config, signer)
+                self.update_rcvd_cert_current(handle, current, rcvd_cert, all_routes, all_aspas, config, signer)
             }
             KeyState::RollPending(pending, current) => {
                 if rcvd_cert_ki == pending.key_id() {
@@ -217,7 +245,7 @@ impl ResourceClass {
                         new_key,
                     }])
                 } else {
-                    self.update_rcvd_cert_current(handle, current, rcvd_cert, routes, config, signer)
+                    self.update_rcvd_cert_current(handle, current, rcvd_cert, all_routes, all_aspas, config, signer)
                 }
             }
             KeyState::RollNew(new, current) => {
@@ -228,22 +256,24 @@ impl ResourceClass {
                         rcvd_cert,
                     }])
                 } else {
-                    self.update_rcvd_cert_current(handle, current, rcvd_cert, routes, config, signer)
+                    self.update_rcvd_cert_current(handle, current, rcvd_cert, all_routes, all_aspas, config, signer)
                 }
             }
             KeyState::RollOld(current, _old) => {
                 // We will never request a new certificate for an old key
-                self.update_rcvd_cert_current(handle, current, rcvd_cert, routes, config, signer)
+                self.update_rcvd_cert_current(handle, current, rcvd_cert, all_routes, all_aspas, config, signer)
             }
         }
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn update_rcvd_cert_current(
         &self,
         handle: &Handle,
         current_key: &CurrentKey,
         rcvd_cert: RcvdCert,
         routes: &Routes,
+        aspas: &AspaDefinitions,
         config: &Config,
         signer: &KrillSigner,
     ) -> KrillResult<Vec<CaEvtDet>> {
@@ -271,31 +301,13 @@ impl ResourceClass {
                 rcvd_cert.validity().not_after().to_rfc3339()
             );
 
-            // Check whether child certificates should be shrunk
-            //
-            // NOTE: We need to pro-actively shrink child certificates to avoid invalidating them.
-            //       But, if we gain additional resources it is up to child to request a new certificate
-            //       with those resources.
-            //
-            let mut updates = ChildCertificateUpdates::default();
-            for issued in self.certificates.overclaiming(rcvd_resources) {
-                let remaining_resources = issued.resource_set().intersection(rcvd_resources);
-                if remaining_resources.is_empty() {
-                    // revoke
-                    updates.remove(issued.subject_key_identifier());
-                } else {
-                    // re-issue
-                    let re_issued = self.re_issue(
-                        issued,
-                        Some(remaining_resources),
-                        current_key,
-                        None,
-                        &config.issuance_timing,
-                        signer,
-                    )?;
-                    updates.issue(re_issued);
-                }
-            }
+            // Prep certified key for updated received certificate
+            let updated_key = CertifiedKey::create(rcvd_cert);
+
+            // Shrink any overclaiming child certificates
+            let updates = self
+                .certificates
+                .shrink_overclaiming(&updated_key, &config.issuance_timing, signer)?;
             if !updates.is_empty() {
                 res.push(CaEvtDet::ChildCertificatesUpdated {
                     resource_class_name: self.name.clone(),
@@ -303,14 +315,28 @@ impl ResourceClass {
                 });
             }
 
-            // Check whether ROAs need to be re-issued.
-            let updated_key = CertifiedKey::create(rcvd_cert);
-            let updates = self.roas.update(routes, &updated_key, config, signer)?;
-            if !updates.is_empty() {
-                res.push(CaEvtDet::RoasUpdated {
-                    resource_class_name: self.name.clone(),
-                    updates,
-                });
+            // Re-issue ROAs based on updated resources.
+            // Note that route definitions will not have changed in this case, but the decision logic is all the same.
+            {
+                let updates = self.roas.update(routes, &updated_key, config, signer)?;
+                if !updates.is_empty() {
+                    res.push(CaEvtDet::RoasUpdated {
+                        resource_class_name: self.name.clone(),
+                        updates,
+                    });
+                }
+            }
+
+            // Re-issue ASPA objects based on updated resources.
+            // Note that aspa definitions will not have changed in this case, but the decision logic is all the same.
+            {
+                let updates = self.aspas.update(aspas, &updated_key, config, signer)?;
+                if !updates.is_empty() {
+                    res.push(CaEvtDet::AspaObjectsUpdated {
+                        resource_class_name: self.name.clone(),
+                        updates,
+                    })
+                }
             }
         } else {
             info!(
@@ -499,24 +525,36 @@ impl ResourceClass {
                     self.key_state
                         .keyroll_activate(self.name.clone(), self.parent_rc_name.clone(), signer)?;
 
-                let roa_updates = self.roas.activate_key(new_key, issuance_timing, signer)?;
-                let roas_updated = CaEvtDet::RoasUpdated {
-                    resource_class_name: self.name.clone(),
-                    updates: roa_updates,
-                };
+                let mut events = vec![key_activated];
 
-                let mut cert_updates = ChildCertificateUpdates::default();
-                for issued in self.certificates.iter() {
-                    // re-issue
-                    let re_issued = self.re_issue(issued, None, new_key, None, issuance_timing, signer)?;
-                    cert_updates.issue(re_issued);
+                let roa_updates = self.roas.renew(true, new_key, issuance_timing, signer)?;
+                if !roa_updates.is_empty() {
+                    let roas_updated = CaEvtDet::RoasUpdated {
+                        resource_class_name: self.name.clone(),
+                        updates: roa_updates,
+                    };
+                    events.push(roas_updated);
                 }
-                let certs_updated = CaEvtDet::ChildCertificatesUpdated {
-                    resource_class_name: self.name.clone(),
-                    updates: cert_updates,
-                };
 
-                Ok(vec![key_activated, roas_updated, certs_updated])
+                let aspa_updates = self.aspas.renew(new_key, None, issuance_timing, signer)?;
+                if !aspa_updates.is_empty() {
+                    let aspas_updated = CaEvtDet::AspaObjectsUpdated {
+                        resource_class_name: self.name.clone(),
+                        updates: aspa_updates,
+                    };
+                    events.push(aspas_updated);
+                }
+
+                let cert_updates = self.certificates.activate_key(new_key, issuance_timing, signer)?;
+                if !cert_updates.is_empty() {
+                    let certs_updated = CaEvtDet::ChildCertificatesUpdated {
+                        resource_class_name: self.name.clone(),
+                        updates: cert_updates,
+                    };
+                    events.push(certs_updated);
+                }
+
+                Ok(events)
             }
         } else {
             Ok(vec![])
@@ -557,7 +595,7 @@ impl ResourceClass {
         let signing_key = self.get_current_key()?;
         let parent_resources = signing_key.incoming_cert().resources();
         let resources = parent_resources.intersection(child_resources);
-        let replaces = self.certificates.get(&csr.key_id()).map(ReplacedObject::from);
+        let replaces = self.certificates.get_issued(&csr.key_id()).map(ReplacedObject::from);
 
         let issued = SignSupport::make_issued_cert(
             csr,
@@ -572,41 +610,27 @@ impl ResourceClass {
         Ok(issued)
     }
 
-    fn re_issue(
-        &self,
-        previous: &IssuedCert,
-        updated_resources: Option<ResourceSet>,
-        signing_key: &CertifiedKey,
-        csr_info_opt: Option<CsrInfo>,
-        issuance_timing: &IssuanceTimingConfig,
-        signer: &KrillSigner,
-    ) -> KrillResult<IssuedCert> {
-        let (_uri, limit, resource_set, cert) = previous.clone().unpack();
-        let csr = csr_info_opt.unwrap_or_else(|| CsrInfo::from(&cert));
-        let resource_set = updated_resources.unwrap_or(resource_set);
-        let replaced = ReplacedObject::new(Revocation::from(&cert), HexEncodedHash::from(&cert));
-
-        let re_issued = SignSupport::make_issued_cert(
-            csr,
-            &resource_set,
-            limit,
-            Some(replaced),
-            signing_key,
-            issuance_timing.timing_child_certificate_valid_weeks,
-            signer,
-        )?;
-
-        Ok(re_issued)
-    }
-
     /// Stores an [IssuedCert](krill_commons.api.ca.IssuedCert)
     pub fn certificate_issued(&mut self, issued: IssuedCert) {
         self.certificates.certificate_issued(issued);
     }
 
+    pub fn certificate_unsuspended(&mut self, unsuspended: UnsuspendedCert) {
+        self.certificates.certificate_unsuspended(unsuspended);
+    }
+
+    pub fn certificate_suspended(&mut self, suspended: SuspendedCert) {
+        self.certificates.certificate_suspended(suspended);
+    }
+
     /// Returns an issued certificate for a key, if it exists
     pub fn issued(&self, ki: &KeyIdentifier) -> Option<&IssuedCert> {
-        self.certificates.get(ki)
+        self.certificates.get_issued(ki)
+    }
+
+    /// Returns a suspended certificate for a key, if it exists
+    pub fn suspended(&self, ki: &KeyIdentifier) -> Option<&SuspendedCert> {
+        self.certificates.get_suspended(ki)
     }
 
     /// Removes a revoked key.
@@ -620,9 +644,14 @@ impl ResourceClass {
 impl ResourceClass {
     /// Renew all ROAs under the current for which the not-after time closer
     /// than the given number of weeks
-    pub fn renew_roas(&self, issuance_timing: &IssuanceTimingConfig, signer: &KrillSigner) -> KrillResult<RoaUpdates> {
+    pub fn renew_roas(
+        &self,
+        force: bool,
+        issuance_timing: &IssuanceTimingConfig,
+        signer: &KrillSigner,
+    ) -> KrillResult<RoaUpdates> {
         let key = self.get_current_key()?;
-        self.roas.renew(key, issuance_timing, signer)
+        self.roas.renew(force, key, issuance_timing, signer)
     }
 
     /// Publish all ROAs under the new key
@@ -632,20 +661,13 @@ impl ResourceClass {
         signer: &KrillSigner,
     ) -> KrillResult<RoaUpdates> {
         let key = self.get_new_key()?;
-        self.roas.activate_key(key, issuance_timing, signer)
+        self.roas.renew(true, key, issuance_timing, signer)
     }
 
-    /// Updates the ROAs in accordance with the current authorizations, and
-    /// the target resources and key determined by the PublishMode.
-    pub fn update_roas(
-        &self,
-        routes: &Routes,
-        new_resources: Option<&ResourceSet>,
-        config: &Config,
-        signer: &KrillSigner,
-    ) -> KrillResult<RoaUpdates> {
+    /// Updates the ROAs in accordance with the current authorizations
+    pub fn update_roas(&self, routes: &Routes, config: &Config, signer: &KrillSigner) -> KrillResult<RoaUpdates> {
         let key = self.get_current_key()?;
-        let resources = new_resources.unwrap_or_else(|| key.incoming_cert().resources());
+        let resources = key.incoming_cert().resources();
         let routes = routes.filter(resources);
         self.roas.update(&routes, key, config, signer)
     }
@@ -656,6 +678,38 @@ impl ResourceClass {
     }
 }
 
+/// # Autonomous System Provider Authorization
+///
+impl ResourceClass {
+    /// Renew all ASPA objects under the current for which the not-after time
+    /// is closer than the given number of weeks
+    pub fn renew_aspas(
+        &self,
+        issuance_timing: &IssuanceTimingConfig,
+        signer: &KrillSigner,
+    ) -> KrillResult<AspaObjectsUpdates> {
+        let key = self.get_current_key()?;
+        let renew_threshold = Some(Time::now() + Duration::weeks(issuance_timing.timing_aspa_reissue_weeks_before));
+        self.aspas.renew(key, renew_threshold, issuance_timing, signer)
+    }
+
+    /// Updates the ASPA objects in accordance with the supplied definitions
+    pub fn update_aspas(
+        &self,
+        all_aspas: &AspaDefinitions,
+        config: &Config,
+        signer: &KrillSigner,
+    ) -> KrillResult<AspaObjectsUpdates> {
+        let key = self.get_current_key()?;
+        self.aspas.update(all_aspas, key, config, signer)
+    }
+
+    /// Apply ASPA object changes from events
+    pub fn aspa_objects_updated(&mut self, updates: AspaObjectsUpdates) {
+        self.aspas.updated(updates)
+    }
+}
+
 /// # Resource Tagged Attestations (RTA)
 ///
 impl ResourceClass {
@@ -679,7 +733,7 @@ impl ResourceClass {
         }
 
         let pub_key = signer.get_key_info(&key).map_err(Error::signer)?;
-        let ee = SignSupport::make_rta_ee_cert(resources, &current, validity, pub_key, signer)?;
+        let ee = SignSupport::make_rta_ee_cert(resources, current, validity, pub_key, signer)?;
 
         Ok(ee)
     }
