@@ -1,16 +1,13 @@
-use std::cmp::Ordering;
-use std::collections::HashMap;
-use std::fmt;
-use std::ops::Deref;
-use std::str::FromStr;
+use std::{cmp::Ordering, collections::HashMap, fmt, ops::Deref, str::FromStr};
 
 use chrono::Duration;
 use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
 
-use rpki::roa::{Roa, RoaBuilder};
-use rpki::sigobj::SignedObjectBuilder;
-use rpki::uri;
-use rpki::x509::Time;
+use rpki::repository::{
+    roa::{Roa, RoaBuilder},
+    sigobj::SignedObjectBuilder,
+    x509::Time,
+};
 
 use crate::{
     commons::{
@@ -176,10 +173,10 @@ impl RouteAuthorizationUpdates {
     pub fn affected_prefixes(&self) -> ResourceSet {
         let mut resources = ResourceSet::default();
         for roa in &self.added {
-            resources = resources.union(&roa.prefix().clone().into());
+            resources = resources.union(&roa.prefix().into());
         }
         for roa in &self.removed {
-            resources = resources.union(&roa.prefix().clone().into());
+            resources = resources.union(&roa.prefix().into());
         }
         resources
     }
@@ -507,7 +504,6 @@ impl Roas {
                 let roa = Self::make_roa(
                     &[*auth],
                     &name,
-                    None,
                     certified_key,
                     issuance_timing.timing_roa_valid_weeks,
                     signer,
@@ -654,9 +650,12 @@ impl Roas {
         }
     }
 
-    /// Re-new ROAs before they would expire
+    /// Re-new ROAs before they would expire, or when forced e.g. in case
+    /// ROAs need to be reissued because of a keyroll, or because of a change
+    /// in encoding (like forcing shorter subject names, see issue #700)
     pub fn renew(
         &self,
+        force: bool,
         certified_key: &CertifiedKey,
         issuance_timing: &IssuanceTimingConfig,
         signer: &KrillSigner,
@@ -667,11 +666,10 @@ impl Roas {
 
         for (auth, roa_info) in self.simple.iter() {
             let name = ObjectName::from(auth);
-            if roa_info.expires() < renew_threshold {
+            if force || roa_info.expires() < renew_threshold {
                 let roa = Self::make_roa(
                     &[*auth],
                     &name,
-                    None,
                     certified_key,
                     issuance_timing.timing_roa_valid_weeks,
                     signer,
@@ -684,13 +682,12 @@ impl Roas {
         for (roa_key, aggregate) in self.aggregate.iter() {
             let roa_info = aggregate.roa_info();
 
-            if roa_info.expires() < renew_threshold {
+            if force || roa_info.expires() < renew_threshold {
                 let authorizations = aggregate.authorizations().clone();
                 let name = ObjectName::from(roa_key);
                 let new_roa = Self::make_roa(
                     authorizations.as_slice(),
                     &name,
-                    None,
                     certified_key,
                     issuance_timing.timing_roa_valid_weeks,
                     signer,
@@ -705,82 +702,20 @@ impl Roas {
         Ok(updates)
     }
 
-    /// Re-generate all ROAs when a new key is being activated
-    pub fn activate_key(
-        &self,
-        certified_key: &CertifiedKey,
-        issuance_timing: &IssuanceTimingConfig,
-        signer: &KrillSigner,
-    ) -> KrillResult<RoaUpdates> {
-        let mut updates = RoaUpdates::default();
-
-        for (auth, roa) in self.simple.iter() {
-            let name = ObjectName::from(auth);
-            let new_roa = Self::make_roa(
-                &[*auth],
-                &name,
-                None,
-                certified_key,
-                issuance_timing.timing_roa_valid_weeks,
-                signer,
-            )?;
-            let new_roa_info = RoaInfo::updated_roa(roa, new_roa);
-            updates.update(*auth, new_roa_info);
-        }
-
-        for (roa_key, aggregate) in self.aggregate.iter() {
-            let roa = aggregate.roa_info();
-
-            let authorizations = aggregate.authorizations().clone();
-            let name = ObjectName::from(roa_key);
-            let new_roa = Self::make_roa(
-                authorizations.as_slice(),
-                &name,
-                None,
-                certified_key,
-                issuance_timing.timing_roa_valid_weeks,
-                signer,
-            )?;
-            let new_roa_info = RoaInfo::updated_roa(roa, new_roa);
-            let aggregate = AggregateRoaInfo::new(authorizations, new_roa_info);
-
-            updates.update_aggregate(*roa_key, aggregate);
-        }
-
-        Ok(updates)
-    }
-
-    pub fn iter(&self) -> impl Iterator<Item = (&RouteAuthorization, &RoaInfo)> {
-        self.simple.iter()
-    }
-
-    pub fn authorizations(&self) -> impl Iterator<Item = &RouteAuthorization> {
-        self.simple.keys()
-    }
-
     pub fn make_roa(
         authorizations: &[RouteAuthorization],
         name: &ObjectName,
-        new_repo: Option<&uri::Rsync>,
         certified_key: &CertifiedKey,
         weeks: i64,
         signer: &KrillSigner,
     ) -> KrillResult<Roa> {
         let incoming_cert = certified_key.incoming_cert();
-        let crl_uri = match &new_repo {
-            None => incoming_cert.crl_uri(),
-            Some(base_uri) => base_uri.join(incoming_cert.crl_name().as_bytes()),
-        };
-
-        let roa_uri = match &new_repo {
-            None => incoming_cert.uri_for_name(name),
-            Some(base_uri) => base_uri.join(name.as_bytes()),
-        };
-
-        let aia = incoming_cert.uri();
-
         let signing_key = certified_key.key_id();
 
+        let crl_uri = incoming_cert.crl_uri();
+        let roa_uri = incoming_cert.uri_for_name(name);
+        let aia = incoming_cert.uri();
+
         let asn = authorizations
             .first()
             .ok_or_else(|| Error::custom("Attempt to create ROA without prefixes"))?
@@ -825,7 +760,6 @@ impl Roas {
         let roa = Self::make_roa(
             authorizations.as_slice(),
             &name,
-            None,
             certified_key,
             issuance_timing.timing_roa_valid_weeks,
             signer,
