@@ -1,32 +1,34 @@
-use std::io;
-use std::io::Read;
-use std::net::{IpAddr, Ipv4Addr, SocketAddr};
-use std::path::PathBuf;
-use std::str::FromStr;
-use std::{env, fmt};
-use std::{fs::File, path::Path};
+use std::{
+    env, fmt,
+    fs::File,
+    io::{self, Read},
+    net::{IpAddr, Ipv4Addr, SocketAddr},
+    path::{Path, PathBuf},
+    str::FromStr,
+};
 
+use chrono::Duration;
 use clap::{App, Arg};
 use log::{error, LevelFilter};
-use serde::de;
-use serde::{Deserialize, Deserializer};
+use serde::{de, Deserialize, Deserializer};
+
 #[cfg(unix)]
 use syslog::Facility;
 
-use rpki::uri;
+use rpki::{repository::x509::Time, uri};
 
-use crate::commons::util::ext_serde;
-use crate::commons::{
-    api::{PublicationServerUris, PublisherHandle, Token},
-    error::KrillIoError,
+use crate::{
+    commons::{
+        api::{PublicationServerUris, PublisherHandle, Token},
+        error::KrillIoError,
+        util::ext_serde,
+    },
+    constants::*,
+    daemon::http::tls_keys,
 };
-use crate::constants::*;
-use crate::daemon::http::tls_keys;
 
 #[cfg(feature = "multi-user")]
-use crate::daemon::auth::providers::config_file::config::ConfigAuthUsers;
-#[cfg(feature = "multi-user")]
-use crate::daemon::auth::providers::openid_connect::ConfigAuthOpenIDConnect;
+use crate::daemon::auth::providers::{config_file::config::ConfigAuthUsers, openid_connect::ConfigAuthOpenIDConnect};
 
 //------------ ConfigDefaults ------------------------------------------------
 
@@ -51,9 +53,6 @@ impl ConfigDefaults {
         env::var(KRILL_ENV_FORCE_RECOVER).is_ok()
     }
 
-    fn service_uri() -> String {
-        "https://localhost:3000/".to_string()
-    }
     fn log_level() -> LevelFilter {
         match env::var(KRILL_ENV_LOG_LEVEL) {
             Ok(level) => match LevelFilter::from_str(&level) {
@@ -66,18 +65,23 @@ impl ConfigDefaults {
             _ => LevelFilter::Info,
         }
     }
+
     fn log_type() -> LogType {
         LogType::File
     }
+
     fn log_file() -> PathBuf {
         PathBuf::from("./krill.log")
     }
+
     fn syslog_facility() -> String {
         "daemon".to_string()
     }
+
     fn auth_type() -> AuthType {
         AuthType::AdminToken
     }
+
     fn admin_token() -> Token {
         match env::var(KRILL_ENV_ADMIN_TOKEN) {
             Ok(token) => Token::from(token),
@@ -90,18 +94,25 @@ impl ConfigDefaults {
             },
         }
     }
+
     #[cfg(feature = "multi-user")]
     fn auth_policies() -> Vec<PathBuf> {
         vec![]
     }
+
     #[cfg(feature = "multi-user")]
     fn auth_private_attributes() -> Vec<String> {
         vec![]
     }
-    fn ca_refresh() -> u32 {
+
+    fn ca_refresh_seconds() -> u32 {
         600
     }
 
+    fn ca_refresh_parents_batch_size() -> usize {
+        25
+    }
+
     fn post_limit_api() -> u64 {
         256 * 1024 // 256kB
     }
@@ -152,14 +163,14 @@ impl ConfigDefaults {
         90
     }
 
-    fn timing_publish_valid_days() -> i64 {
-        7
-    }
-
     fn timing_publish_next_hours() -> i64 {
         24
     }
 
+    fn timing_publish_next_jitter_hours() -> i64 {
+        4
+    }
+
     fn timing_publish_hours_before_next() -> i64 {
         8
     }
@@ -179,6 +190,14 @@ impl ConfigDefaults {
     fn timing_roa_reissue_weeks_before() -> i64 {
         4
     }
+
+    fn timing_aspa_valid_weeks() -> i64 {
+        52
+    }
+
+    fn timing_aspa_reissue_weeks_before() -> i64 {
+        4
+    }
 }
 
 //------------ Config --------------------------------------------------------
@@ -207,8 +226,7 @@ pub struct Config {
 
     pub pid_file: Option<PathBuf>,
 
-    #[serde(default = "ConfigDefaults::service_uri")]
-    pub service_uri: String,
+    service_uri: Option<uri::Https>,
 
     #[serde(
         default = "ConfigDefaults::log_level",
@@ -245,8 +263,15 @@ pub struct Config {
     #[cfg(feature = "multi-user")]
     pub auth_openidconnect: Option<ConfigAuthOpenIDConnect>,
 
-    #[serde(default = "ConfigDefaults::ca_refresh")]
-    pub ca_refresh: u32,
+    #[serde(default = "ConfigDefaults::ca_refresh_seconds", alias = "ca_refresh")]
+    pub ca_refresh_seconds: u32,
+
+    #[serde(default = "ConfigDefaults::ca_refresh_parents_batch_size")]
+    pub ca_refresh_parents_batch_size: usize,
+
+    #[serde(skip)]
+    suspend_child_after_inactive_seconds: Option<i64>,
+    suspend_child_after_inactive_hours: Option<i64>,
 
     #[serde(default = "ConfigDefaults::post_limit_api")]
     pub post_limit_api: u64,
@@ -284,15 +309,18 @@ pub struct Config {
     #[serde(flatten)]
     pub repository_retention: RepositoryRetentionConfig,
 
+    #[serde(flatten)]
+    pub metrics: MetricsConfig,
+
     pub testbed: Option<TestBed>,
 }
 
 #[derive(Clone, Debug, Deserialize)]
 pub struct IssuanceTimingConfig {
-    #[serde(default = "ConfigDefaults::timing_publish_valid_days")]
-    pub timing_publish_valid_days: i64,
     #[serde(default = "ConfigDefaults::timing_publish_next_hours")]
-    pub timing_publish_next_hours: i64,
+    timing_publish_next_hours: i64,
+    #[serde(default = "ConfigDefaults::timing_publish_next_jitter_hours")]
+    timing_publish_next_jitter_hours: i64,
     #[serde(default = "ConfigDefaults::timing_publish_hours_before_next")]
     pub timing_publish_hours_before_next: i64,
     #[serde(default = "ConfigDefaults::timing_child_certificate_valid_weeks")]
@@ -303,6 +331,26 @@ pub struct IssuanceTimingConfig {
     pub timing_roa_valid_weeks: i64,
     #[serde(default = "ConfigDefaults::timing_roa_reissue_weeks_before")]
     pub timing_roa_reissue_weeks_before: i64,
+    #[serde(default = "ConfigDefaults::timing_aspa_valid_weeks")]
+    pub timing_aspa_valid_weeks: i64,
+    #[serde(default = "ConfigDefaults::timing_aspa_reissue_weeks_before")]
+    pub timing_aspa_reissue_weeks_before: i64,
+}
+
+impl IssuanceTimingConfig {
+    /// Returns the next update time based on configuration:
+    ///
+    /// now + timing_publish_next_hours + random(0..timing_publish_next_jitter_hours)
+    /// defaults: now + 24 hours + 0 to 4 hours
+    pub fn publish_next(&self) -> Time {
+        let regular_mins = self.timing_publish_next_hours * 60;
+        let random_mins = {
+            use rand::Rng;
+            let mut rng = rand::thread_rng();
+            rng.gen_range(0..(60 * self.timing_publish_next_jitter_hours))
+        };
+        Time::now() + Duration::minutes(regular_mins + random_mins)
+    }
 }
 
 #[derive(Clone, Debug, Deserialize)]
@@ -368,6 +416,18 @@ impl RepositoryRetentionConfig {
     }
 }
 
+#[derive(Clone, Debug, Deserialize)]
+pub struct MetricsConfig {
+    #[serde(default)] // false
+    pub metrics_hide_ca_details: bool,
+    #[serde(default)] // false
+    pub metrics_hide_child_details: bool,
+    #[serde(default)] // false
+    pub metrics_hide_publisher_details: bool,
+    #[serde(default)] // false
+    pub metrics_hide_roa_details: bool,
+}
+
 #[derive(Clone, Debug, Deserialize)]
 pub struct TestBed {
     ta_aia: uri::Rsync,
@@ -428,11 +488,20 @@ impl Config {
     }
 
     pub fn service_uri(&self) -> uri::Https {
-        uri::Https::from_str(&self.service_uri).unwrap()
+        match &self.service_uri {
+            None => {
+                if self.ip == ConfigDefaults::ip() {
+                    uri::Https::from_string(format!("https://localhost:{}/", self.port)).unwrap()
+                } else {
+                    uri::Https::from_string(format!("https://{}:{}/", self.ip, self.port)).unwrap()
+                }
+            }
+            Some(uri) => uri.clone(),
+        }
     }
 
     pub fn rfc8181_uri(&self, publisher: &PublisherHandle) -> uri::Https {
-        uri::Https::from_string(format!("{}rfc8181/{}/", self.service_uri, publisher)).unwrap()
+        uri::Https::from_string(format!("{}rfc8181/{}/", self.service_uri(), publisher)).unwrap()
     }
 
     pub fn pid_file(&self) -> PathBuf {
@@ -454,6 +523,13 @@ impl Config {
         }
     }
 
+    pub fn suspend_child_after_inactive_seconds(&self) -> Option<i64> {
+        match self.suspend_child_after_inactive_seconds {
+            Some(seconds) => Some(seconds),
+            None => self.suspend_child_after_inactive_hours.map(|hours| hours * 3600),
+        }
+    }
+
     pub fn testbed(&self) -> Option<&TestBed> {
         self.testbed.as_ref()
     }
@@ -461,7 +537,7 @@ impl Config {
 
 /// # Create
 impl Config {
-    fn test_config(data_dir: &Path, enable_testbed: bool) -> Self {
+    fn test_config(data_dir: &Path, enable_testbed: bool, enable_ca_refresh: bool, enable_suspend: bool) -> Self {
         use crate::test;
 
         let ip = ConfigDefaults::ip();
@@ -471,7 +547,6 @@ impl Config {
         let https_mode = HttpsMode::Generate;
         let data_dir = data_dir.to_path_buf();
         let always_recover_data = false;
-        let service_uri = ConfigDefaults::service_uri();
 
         let log_level = LevelFilter::Debug;
         let log_type = LogType::Stderr;
@@ -488,7 +563,8 @@ impl Config {
         let auth_users = None;
         #[cfg(feature = "multi-user")]
         let auth_openidconnect = None;
-        let ca_refresh = 1;
+        let ca_refresh_seconds = if enable_ca_refresh { 1 } else { 86400 };
+        let ca_refresh_parents_batch_size = 10;
         let post_limit_api = ConfigDefaults::post_limit_api();
         let post_limit_rfc8181 = ConfigDefaults::post_limit_rfc8181();
         let rfc8181_log_dir = {
@@ -510,23 +586,27 @@ impl Config {
         let roa_aggregate_threshold = 3;
         let roa_deaggregate_threshold = 2;
 
-        let timing_publish_valid_days = ConfigDefaults::timing_publish_valid_days();
         let timing_publish_next_hours = ConfigDefaults::timing_publish_next_hours();
+        let timing_publish_next_jitter_hours = ConfigDefaults::timing_publish_next_jitter_hours();
         let timing_publish_hours_before_next = ConfigDefaults::timing_publish_hours_before_next();
         let timing_child_certificate_valid_weeks = ConfigDefaults::timing_child_certificate_valid_weeks();
         let timing_child_certificate_reissue_weeks_before =
             ConfigDefaults::timing_child_certificate_reissue_weeks_before();
         let timing_roa_valid_weeks = ConfigDefaults::timing_roa_valid_weeks();
         let timing_roa_reissue_weeks_before = ConfigDefaults::timing_roa_reissue_weeks_before();
+        let timing_aspa_valid_weeks = ConfigDefaults::timing_aspa_valid_weeks();
+        let timing_aspa_reissue_weeks_before = ConfigDefaults::timing_aspa_reissue_weeks_before();
 
         let issuance_timing = IssuanceTimingConfig {
-            timing_publish_valid_days,
             timing_publish_next_hours,
+            timing_publish_next_jitter_hours,
             timing_publish_hours_before_next,
             timing_child_certificate_valid_weeks,
             timing_child_certificate_reissue_weeks_before,
             timing_roa_valid_weeks,
             timing_roa_reissue_weeks_before,
+            timing_aspa_valid_weeks,
+            timing_aspa_reissue_weeks_before,
         };
 
         let repository_retention = RepositoryRetentionConfig {
@@ -538,6 +618,13 @@ impl Config {
             retention_archive: false,
         };
 
+        let metrics = MetricsConfig {
+            metrics_hide_ca_details: false,
+            metrics_hide_child_details: false,
+            metrics_hide_publisher_details: false,
+            metrics_hide_roa_details: false,
+        };
+
         let testbed = if enable_testbed {
             Some(TestBed::new(
                 test::rsync("rsync://localhost/ta/ta.cer"),
@@ -549,6 +636,8 @@ impl Config {
             None
         };
 
+        let suspend_child_after_inactive_seconds = if enable_suspend { Some(3) } else { None };
+
         Config {
             ip,
             port,
@@ -556,7 +645,7 @@ impl Config {
             data_dir,
             always_recover_data,
             pid_file,
-            service_uri,
+            service_uri: None,
             log_level,
             log_type,
             log_file,
@@ -571,7 +660,10 @@ impl Config {
             auth_users,
             #[cfg(feature = "multi-user")]
             auth_openidconnect,
-            ca_refresh,
+            ca_refresh_seconds,
+            ca_refresh_parents_batch_size,
+            suspend_child_after_inactive_seconds,
+            suspend_child_after_inactive_hours: None,
             post_limit_api,
             post_limit_rfc8181,
             rfc8181_log_dir,
@@ -584,18 +676,18 @@ impl Config {
             roa_deaggregate_threshold,
             issuance_timing,
             repository_retention,
+            metrics,
             testbed,
         }
     }
 
-    pub fn test(data_dir: &Path, enable_testbed: bool) -> Self {
-        Self::test_config(data_dir, enable_testbed)
+    pub fn test(data_dir: &Path, enable_testbed: bool, enable_ca_refresh: bool, enable_suspend: bool) -> Self {
+        Self::test_config(data_dir, enable_testbed, enable_ca_refresh, enable_suspend)
     }
 
     pub fn pubd_test(data_dir: &Path) -> Self {
-        let mut config = Self::test_config(data_dir, false);
+        let mut config = Self::test_config(data_dir, false, false, false);
         config.port = 3001;
-        config.service_uri = "https://localhost:3001/".to_string();
         config
     }
 
@@ -621,7 +713,7 @@ impl Config {
     pub fn create() -> Result<Self, ConfigError> {
         let config_file = Self::get_config_filename();
 
-        let config = match Self::read_config(&config_file) {
+        let mut config = match Self::read_config(&config_file) {
             Err(e) => {
                 if config_file == KRILL_DEFAULT_CONFIG_FILE {
                     Err(ConfigError::other(
@@ -640,6 +732,23 @@ impl Config {
                 Ok(config)
             }
         }?;
+
+        if config.ca_refresh_seconds < CA_REFRESH_SECONDS_MIN {
+            warn!(
+                "The value for 'ca_refresh_seconds' was below the minimum value, changing it to {} seconds",
+                CA_REFRESH_SECONDS_MIN
+            );
+            config.ca_refresh_seconds = CA_REFRESH_SECONDS_MIN;
+        }
+
+        if config.ca_refresh_seconds > CA_REFRESH_SECONDS_MAX {
+            warn!(
+                "The value for 'ca_refresh_seconds' was above the maximum value, changing it to {} seconds",
+                CA_REFRESH_SECONDS_MAX
+            );
+            config.ca_refresh_seconds = CA_REFRESH_SECONDS_MAX;
+        }
+
         config
             .verify()
             .map_err(|e| ConfigError::Other(format!("Error parsing config file: {}, error: {}", config_file, e)))?;
@@ -655,13 +764,10 @@ impl Config {
             return Err(ConfigError::other("Port number must be >1024"));
         }
 
-        if !self.service_uri.ends_with('/') {
-            return Err(ConfigError::other("service URI must end with '/'"));
-        } else {
-            uri::Https::from_str(&self.service_uri)
-                .map_err(|_| ConfigError::Other(format!("Invalid service uri: {}", self.service_uri)))?;
-
-            if self.service_uri.as_str().matches('/').count() != 3 {
+        if let Some(service_uri) = &self.service_uri {
+            if !service_uri.as_str().ends_with('/') {
+                return Err(ConfigError::other("service URI must end with '/'"));
+            } else if service_uri.as_str().matches('/').count() != 3 {
                 return Err(ConfigError::other(
                     "Service URI MUST specify a host name only, e.g. https://rpki.example.com:3000/",
                 ));
@@ -672,6 +778,19 @@ impl Config {
             return Err(ConfigError::other("timing_publish_next_hours must be at least 2"));
         }
 
+        if self.issuance_timing.timing_publish_next_jitter_hours < 0 {
+            return Err(ConfigError::other(
+                "timing_publish_next_jitter_hours must be at least 0",
+            ));
+        }
+
+        if self.issuance_timing.timing_publish_next_jitter_hours > (self.issuance_timing.timing_publish_next_hours / 2)
+        {
+            return Err(ConfigError::other(
+                "timing_publish_next_jitter_hours must be at most timing_publish_next_hours divided by 2",
+            ));
+        }
+
         if self.issuance_timing.timing_publish_hours_before_next < 1 {
             return Err(ConfigError::other(
                 "timing_publish_hours_before_next must be at least 1",
@@ -684,12 +803,6 @@ impl Config {
             ));
         }
 
-        if self.issuance_timing.timing_publish_valid_days < 1
-            || self.issuance_timing.timing_publish_valid_days < (self.issuance_timing.timing_publish_next_hours / 24)
-        {
-            return Err(ConfigError::other("timing_publish_valid_days must be 1 or bigger, and must be at least as long as timing_publish_next_hours"));
-        }
-
         if self.issuance_timing.timing_child_certificate_valid_weeks < 2 {
             return Err(ConfigError::other(
                 "timing_child_certificate_valid_weeks must be at least 2",
@@ -722,6 +835,15 @@ impl Config {
             ));
         }
 
+        if let Some(threshold) = self.suspend_child_after_inactive_hours {
+            if threshold < CA_SUSPEND_MIN_HOURS {
+                return Err(ConfigError::Other(format!(
+                    "suspend_child_after_inactive_hours must be {} or higher (or not set at all)",
+                    CA_SUSPEND_MIN_HOURS
+                )));
+            }
+        }
+
         Ok(())
     }
 
@@ -1119,7 +1241,7 @@ mod tests {
                 for component in &krill_key_components {
                     // framework components shouldn't log at Trace level
                     let should_be_enabled = should_be_enabled
-                        && (*log_msg_level < LL::Trace || !krill_framework_components.contains(&component));
+                        && (*log_msg_level < LL::Trace || !krill_framework_components.contains(component));
 
                     // verify that logging is enabled or not as expected
                     assert_eq!(
