@@ -1,18 +1,20 @@
 //! Deal with asynchronous scheduled processes, either triggered by an
 //! event that occurred, or planned (e.g. re-publishing).
 
-use std::sync::{Arc, RwLock};
-use std::time::Duration;
+use std::{
+    sync::{Arc, RwLock},
+    time::Duration,
+};
 
 use clokwerk::{self, ScheduleHandle, TimeUnits};
 use tokio::runtime::Runtime;
 
-use rpki::x509::Time;
+use rpki::repository::x509::Time;
 
 use crate::{
     commons::{
         actor::Actor,
-        api::{Handle, ParentHandle},
+        api::{Handle, ParentHandle, Timestamp},
         bgp::BgpAnalyser,
     },
     constants::{
@@ -41,7 +43,7 @@ pub struct Scheduler {
 
     /// Responsible for periodically reissuing ROAs before they would expire.
     #[allow(dead_code)] // just need to keep this in scope
-    cas_roas_renew: ScheduleHandle,
+    cas_objects_renew: ScheduleHandle,
 
     /// Responsible for letting CA check with their parents whether their resource
     /// entitlements have changed *and* for the shrinking of issued certificates, if
@@ -71,8 +73,8 @@ impl Scheduler {
         let cas_event_triggers = make_cas_event_triggers(event_queue.clone(), ca_manager.clone(), actor.clone());
 
         let cas_republish = make_cas_republish(ca_manager.clone(), event_queue);
-        let cas_roas_renew = make_cas_roa_renew(ca_manager.clone(), actor.clone());
-        let cas_refresh = make_cas_refresh(ca_manager, config.ca_refresh, actor.clone());
+        let cas_objects_renew = make_cas_objects_renew(ca_manager.clone(), actor.clone());
+        let cas_refresh = make_cas_refresh(ca_manager, config.ca_refresh_seconds, actor.clone());
 
         let announcements_refresh = make_announcements_refresh(bgp_analyser);
 
@@ -82,7 +84,7 @@ impl Scheduler {
         Scheduler {
             cas_event_triggers,
             cas_republish,
-            cas_roas_renew,
+            cas_objects_renew,
             cas_refresh,
             announcements_refresh,
             #[cfg(feature = "multi-user")]
@@ -93,15 +95,17 @@ impl Scheduler {
 
 #[allow(clippy::cognitive_complexity)]
 fn make_cas_event_triggers(event_queue: Arc<MessageQueue>, ca_manager: Arc<CaManager>, actor: Actor) -> ScheduleHandle {
+    let started = Timestamp::now();
+
     SkippingScheduler::run(1, "scan for queued triggers", move || {
-        let mut rt = Runtime::new().unwrap();
+        let rt = Runtime::new().unwrap();
 
         rt.block_on(async {
             for evt in event_queue.pop_all() {
                 match evt {
                     QueueTask::ServerStarted => {
                         info!("Will re-sync all CAs with their parents and repository after startup");
-                        ca_manager.cas_refresh_all(&actor).await;
+                        ca_manager.cas_refresh_all(started, &actor).await;
                         ca_manager.cas_repo_sync_all(&actor).await;
                     }
 
@@ -180,7 +184,7 @@ fn requeue_time_test() -> Time {
 async fn try_sync_repo(event_queue: &Arc<MessageQueue>, ca_manager: Arc<CaManager>, ca: Handle) {
     debug!("Synchronize CA {} with repository", ca);
 
-    if let Err(e) = ca_manager.ca_repo_sync_all(&ca).await {
+    if let Err(e) = ca_manager.cas_repo_sync_single(&ca).await {
         let requeue_time = if test_mode_enabled() {
             requeue_time_test()
         } else {
@@ -221,7 +225,7 @@ fn make_cas_republish(ca_server: Arc<CaManager>, event_queue: Arc<MessageQueue>)
         SCHEDULER_INTERVAL_SECONDS_REPUBLISH,
         "CA certificate republish",
         move || {
-            let mut rt = Runtime::new().unwrap();
+            let rt = Runtime::new().unwrap();
             rt.block_on(async {
                 debug!("Triggering background republication for all CAs, note this may be a no-op");
                 match ca_server.republish_all().await {
@@ -238,33 +242,34 @@ fn make_cas_republish(ca_server: Arc<CaManager>, event_queue: Arc<MessageQueue>)
     )
 }
 
-fn make_cas_roa_renew(ca_server: Arc<CaManager>, actor: Actor) -> ScheduleHandle {
+fn make_cas_objects_renew(ca_server: Arc<CaManager>, actor: Actor) -> ScheduleHandle {
     SkippingScheduler::run(SCHEDULER_INTERVAL_SECONDS_ROA_RENEW, "CA ROA renewal", move || {
-        let mut rt = Runtime::new().unwrap();
+        let rt = Runtime::new().unwrap();
         rt.block_on(async {
             debug!(
-                "Triggering background renewal for about to expire ROAs issued by all CAs, note this may be a no-op"
+                "Triggering background renewal for about to expire objects issued by all CAs, note this may be a no-op"
             );
-            if let Err(e) = ca_server.renew_roas_all(&actor).await {
-                error!("Background re-issuing of about to expire ROAs failed: {}", e);
+            if let Err(e) = ca_server.renew_objects_all(&actor).await {
+                error!("Background re-issuing of about to expire objects failed: {}", e);
             }
         })
     })
 }
 
 fn make_cas_refresh(ca_server: Arc<CaManager>, refresh_rate: u32, actor: Actor) -> ScheduleHandle {
+    let server_started = Timestamp::now();
     SkippingScheduler::run(refresh_rate, "CA certificate refresh", move || {
-        let mut rt = Runtime::new().unwrap();
+        let rt = Runtime::new().unwrap();
         rt.block_on(async {
             debug!("Triggering background refresh for all CAs");
-            ca_server.cas_refresh_all(&actor).await;
+            ca_server.cas_refresh_all(server_started, &actor).await;
         });
     })
 }
 
 fn make_announcements_refresh(bgp_analyser: Arc<BgpAnalyser>) -> ScheduleHandle {
     SkippingScheduler::run(5, "update RIS BGP info", move || {
-        let mut rt = Runtime::new().unwrap();
+        let rt = Runtime::new().unwrap();
         rt.block_on(async {
             if let Err(e) = bgp_analyser.update().await {
                 error!("Failed to update BGP announcements: {}", e)
@@ -276,7 +281,7 @@ fn make_announcements_refresh(bgp_analyser: Arc<BgpAnalyser>) -> ScheduleHandle
 #[cfg(feature = "multi-user")]
 fn make_login_cache_sweeper_sh(cache: Arc<LoginSessionCache>) -> ScheduleHandle {
     SkippingScheduler::run(60, "sweep session decryption cache", move || {
-        let mut rt = Runtime::new().unwrap();
+        let rt = Runtime::new().unwrap();
         rt.block_on(async {
             if let Err(e) = cache.sweep() {
                 error!("Background sweep of session decryption cache failed: {}", e);
@@ -377,10 +382,10 @@ mod tests {
         let counter_sh = counter.clone();
 
         let _schedule_handle = SkippingScheduler::run(1, "CA certificate refresh", move || {
-            let mut rt = Runtime::new().unwrap();
+            let rt = Runtime::new().unwrap();
             rt.block_on(async {
                 counter_sh.write().unwrap().inc();
-                tokio::time::delay_for(std::time::Duration::from_secs(2)).await;
+                tokio::time::sleep(std::time::Duration::from_secs(2)).await;
             });
         });
 
