@@ -1,18 +1,17 @@
-use std::fmt;
-use std::fs;
-use std::mem;
-use std::path::PathBuf;
-use std::str::{from_utf8_unchecked, FromStr};
-use std::sync::{Arc, RwLock};
 use std::{
     collections::{HashMap, VecDeque},
-    path::Path,
+    fmt, fs, mem,
+    path::{Path, PathBuf},
+    str::FromStr,
+    sync::{Arc, RwLock},
 };
 
 use bytes::Bytes;
-use rpki::crypto::KeyIdentifier;
-use rpki::uri;
-use rpki::x509::Time;
+
+use rpki::{
+    repository::{crypto::KeyIdentifier, x509::Time},
+    uri,
+};
 
 use crate::{
     commons::{
@@ -36,12 +35,12 @@ use crate::{
         REPOSITORY_RRDP_DIR, REPOSITORY_RSYNC_DIR,
     },
     daemon::config::{Config, RepositoryRetentionConfig},
-    pubd::publishers::Publisher,
-    pubd::{RepoAccessCmd, RepoAccessCmdDet, RepositoryAccessEvent, RepositoryAccessEventDetails, RepositoryAccessIni},
+    pubd::{
+        publishers::Publisher, RepoAccessCmd, RepoAccessCmdDet, RepositoryAccessEvent, RepositoryAccessEventDetails,
+        RepositoryAccessIni, RepositoryAccessInitDetails,
+    },
 };
 
-use super::RepositoryAccessInitDetails;
-
 //------------ RepositoryContentProxy ----------------------------------------
 
 /// We can only have one (1) RepositoryContent, but it is stored
@@ -49,6 +48,7 @@ use super::RepositoryAccessInitDetails;
 /// so that callers don't need to worry about storage details.
 #[derive(Debug)]
 pub struct RepositoryContentProxy {
+    cache: RwLock<Option<RepositoryContent>>,
     store: RwLock<KeyValueStore>,
     key: KeyStoreKey,
 }
@@ -59,32 +59,56 @@ impl RepositoryContentProxy {
         let store = KeyValueStore::disk(work_dir, PUBSERVER_CONTENT_DIR)?;
         let store = RwLock::new(store);
         let key = KeyStoreKey::simple(format!("{}.json", PUBSERVER_DFLT));
+        let cache = RwLock::new(None);
 
-        Ok(RepositoryContentProxy { store, key })
+        let proxy = RepositoryContentProxy { cache, store, key };
+        proxy.warm_cache()?;
+
+        Ok(proxy)
     }
 
-    // Initialize
+    fn warm_cache(&self) -> KrillResult<()> {
+        let key_store_read = self.store.read().unwrap();
+
+        if key_store_read.has(&self.key)? {
+            info!("Warming the repository content cache, this can take a minute for large repositories.");
+            let content = key_store_read.get(&self.key)?.unwrap();
+            self.cache.write().unwrap().replace(content);
+        }
+
+        Ok(())
+    }
+
+    /// Initialize
     pub fn init(&self, work_dir: &Path, uris: PublicationServerUris) -> KrillResult<()> {
         if self.store.read().unwrap().has(&self.key)? {
             Err(Error::RepositoryServerAlreadyInitialized)
         } else {
-            let (rrdp_base_uri, rsync_jail) = uris.unpack();
+            // initialize new repo content
+            let repository_content = {
+                let (rrdp_base_uri, rsync_jail) = uris.unpack();
 
-            let publishers = HashMap::new();
+                let publishers = HashMap::new();
 
-            let session = RrdpSession::default();
-            let stats = RepoStats::new(session);
+                let session = RrdpSession::default();
+                let stats = RepoStats::new(session);
 
-            let mut repo_dir = work_dir.to_path_buf();
-            repo_dir.push(REPOSITORY_DIR);
+                let mut repo_dir = work_dir.to_path_buf();
+                repo_dir.push(REPOSITORY_DIR);
 
-            let rrdp = RrdpServer::create(rrdp_base_uri, &repo_dir, session);
-            let rsync = RsyncdStore::new(rsync_jail, &repo_dir);
+                let rrdp = RrdpServer::create(rrdp_base_uri, &repo_dir, session);
+                let rsync = RsyncdStore::new(rsync_jail, &repo_dir);
 
-            let repo = RepositoryContent::new(publishers, rrdp, rsync, stats);
+                RepositoryContent::new(publishers, rrdp, rsync, stats)
+            };
 
+            // Store newly initialized repo content on disk
             let store = self.store.write().unwrap();
-            store.store(&self.key, &repo)?;
+            store.store(&self.key, &repository_content)?;
+
+            // Store newly initialized repo content in cache
+            let mut cache = self.cache.write().unwrap();
+            cache.replace(repository_content);
 
             Ok(())
         }
@@ -100,25 +124,29 @@ impl RepositoryContentProxy {
             store.drop_key(&self.key)?;
         }
 
+        let mut cache = self.cache.write().unwrap();
+        cache.take();
+
         Ok(())
     }
 
+    /// Return the repository content stats
     pub fn stats(&self) -> KrillResult<RepoStats> {
-        self.read_content().map(|c| c.stats().clone())
+        self.read(|content| Ok(content.stats().clone()))
     }
 
-    // Adds a publisher with an empty set of published objects.
-    // Replaces an existing publisher if it existed.
-    // This is only supposed to be called if adding the publisher
-    // to the RepositoryAccess was successful (and *that* will fail if
-    // the publisher is a duplicate). This method can only fail if
-    // there is an issue with the underlying key value store.
+    /// Add a publisher with an empty set of published objects.
+    ///
+    /// Replaces an existing publisher if it existed.
+    /// This is only supposed to be called if adding the publisher
+    /// to the RepositoryAccess was successful (and *that* will fail if
+    /// the publisher is a duplicate). This method can only fail if
+    /// there is an issue with the underlying key value store.
     pub fn add_publisher(&self, name: PublisherHandle) -> KrillResult<()> {
         self.write(|content| content.add_publisher(name))
     }
 
-    // Removes a publisher and its content. Will also write the updated
-    // RRDP and rsync content.
+    /// Removes a publisher and its content.
     pub fn remove_publisher(
         &self,
         name: &PublisherHandle,
@@ -128,9 +156,10 @@ impl RepositoryContentProxy {
         self.write(|content| content.remove_publisher(name, jail, config))
     }
 
-    // Publish an update for a publisher. Assumes that the RFC 8181 CMS has
-    // been verified, but will check that all objects are within the publisher's
-    // uri space (jail).
+    /// Publish an update for a publisher.
+    ///
+    /// Assumes that the RFC 8181 CMS has been verified, but will check that all objects
+    /// are within the publisher's uri space (jail).
     pub fn publish(
         &self,
         name: &PublisherHandle,
@@ -141,40 +170,57 @@ impl RepositoryContentProxy {
         self.write(|content| content.publish(name, delta.into(), jail, config))
     }
 
-    // Write all current files to disk
+    /// Write all current files to disk
     pub fn write_repository(&self, config: &RepositoryRetentionConfig) -> KrillResult<()> {
-        self.read_content()?.write_repository(config)
+        self.read(|content| content.write_repository(config))
     }
 
-    // Reset the RRDP session
+    /// Reset the RRDP session if it is initialized. Otherwise do nothing.
     pub fn session_reset(&self, config: &RepositoryRetentionConfig) -> KrillResult<()> {
-        self.write(|content| content.session_reset(config))
+        if self.cache.read().unwrap().is_some() {
+            self.write(|content| content.session_reset(config))
+        } else {
+            // repository server was not initialized on this Krill instance. Nothing to reset.
+            Ok(())
+        }
     }
 
+    /// Create a list reply containing all current objects for a publisher
+    pub fn list_reply(&self, name: &PublisherHandle) -> KrillResult<ListReply> {
+        self.read(|content| content.list_reply(name))
+    }
+
+    // Get all current objects for a publisher
+    pub fn current_objects(&self, name: &PublisherHandle) -> KrillResult<CurrentObjects> {
+        self.read(|content| content.objects_for_publisher(name).map(|o| o.clone()))
+    }
+
+    // Execute a closure on a mutable repository content in a single write 'transaction'
     fn write<F: FnOnce(&mut RepositoryContent) -> KrillResult<()>>(&self, op: F) -> KrillResult<()> {
+        // If there is any existing content, then we can assume that the cache
+        // has it - because it's initialized when we read the content during
+        // initialization.
         let store = self.store.write().unwrap();
-        let mut content: RepositoryContent = store.get(&self.key)?.ok_or(Error::RepositoryServerNotInitialized)?;
+        let mut cache = self.cache.write().unwrap();
 
-        op(&mut content)?;
+        let content: &mut RepositoryContent = cache.as_mut().ok_or(Error::RepositoryServerNotInitialized)?;
 
-        store.store(&self.key, &content)?;
+        op(content)?;
+
+        store.store(&self.key, content)?;
         Ok(())
     }
 
-    fn read_content(&self) -> KrillResult<RepositoryContent> {
-        self.store
-            .read()
-            .unwrap()
-            .get(&self.key)?
-            .ok_or(Error::RepositoryServerNotInitialized)
-    }
-
-    pub fn list_reply(&self, name: &PublisherHandle) -> KrillResult<ListReply> {
-        self.read_content()?.list_reply(name)
-    }
-
-    pub fn current_objects(&self, name: &PublisherHandle) -> KrillResult<CurrentObjects> {
-        self.read_content()?.objects_for_publisher(name).map(|o| o.clone())
+    // Execute a closure on a mutable repository content in a single read 'transaction'
+    //
+    // This function fails if the repository content is not initialized.
+    fn read<A, F: FnOnce(&RepositoryContent) -> KrillResult<A>>(&self, op: F) -> KrillResult<A> {
+        // Note that because the content is initialized it is implied that the cache MUST always be
+        // set. I.e. it is set on initialization and updated whenever the repository content is updated.
+        // So, we can safely read from the cache only.
+        let cache = self.cache.read().unwrap();
+        let content = cache.as_ref().ok_or(Error::RepositoryServerNotInitialized)?;
+        op(content)
     }
 }
 
@@ -279,6 +325,10 @@ impl RepositoryContent {
     }
 
     pub fn session_reset(&mut self, config: &RepositoryRetentionConfig) -> KrillResult<()> {
+        info!(
+            "Performing RRDP session reset. This ensures a consistent view for RPs in case we restarted from a backup."
+        );
+
         self.rrdp.session_reset();
         self.stats.session_reset(self.rrdp.notification());
         self.write_repository(config)?;
@@ -388,8 +438,6 @@ impl RsyncdStore {
                 .relative_to(&self.base_uri)
                 .ok_or_else(|| Error::publishing_outside_jail(publish.uri(), &self.base_uri))?;
 
-            let rel = unsafe { from_utf8_unchecked(rel) };
-
             let mut path = new_dir.clone();
             path.push(rel);
 
@@ -469,7 +517,7 @@ pub struct RrdpServer {
 }
 
 impl RrdpServer {
-    #[allow(clippy::clippy::too_many_arguments)]
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         rrdp_base_uri: uri::Https,
         rrdp_base_dir: PathBuf,
@@ -504,8 +552,8 @@ impl RrdpServer {
         let snapshot = Snapshot::create(session);
 
         let serial = 0;
-        let snapshot_uri = Self::new_snapshot_uri(&rrdp_base_uri, &session, serial);
-        let snapshot_path = Self::new_snapshot_path(&rrdp_base_dir, &session, serial);
+        let snapshot_uri = snapshot.uri(&rrdp_base_uri);
+        let snapshot_path = snapshot.path(&rrdp_base_dir);
         let snapshot_hash = HexEncodedHash::from_content(snapshot.xml().as_slice());
 
         let snapshot_ref = SnapshotRef::new(snapshot_uri, snapshot_path, snapshot_hash);
@@ -543,12 +591,10 @@ impl RrdpServer {
     /// state.
     fn session_reset(&mut self) {
         let session = RrdpSession::random();
-        let serial = 0;
-
         let snapshot = self.snapshot.session_reset(session);
 
-        let snapshot_uri = Self::new_snapshot_uri(&self.rrdp_base_uri, &session, serial);
-        let snapshot_path = Self::new_snapshot_path(&self.rrdp_base_dir, &session, serial);
+        let snapshot_uri = snapshot.uri(&self.rrdp_base_uri);
+        let snapshot_path = snapshot.path(&self.rrdp_base_dir);
         let snapshot_hash = HexEncodedHash::from_content(snapshot.xml().as_slice());
 
         let snapshot_ref = SnapshotRef::new(snapshot_uri, snapshot_path, snapshot_hash);
@@ -633,8 +679,8 @@ impl RrdpServer {
     // relevant.
     fn update_notification(&mut self, config: &RepositoryRetentionConfig) {
         let snapshot_ref = {
-            let snapshot_uri = self.snapshot_uri();
-            let snapshot_path = self.snapshot_path(self.serial);
+            let snapshot_uri = self.snapshot.uri(&self.rrdp_base_uri);
+            let snapshot_path = self.snapshot.path(&self.rrdp_base_dir);
             let snapshot_xml = self.snapshot.xml();
             let snapshot_hash = HexEncodedHash::from_content(snapshot_xml.as_slice());
             SnapshotRef::new(snapshot_uri, snapshot_path, snapshot_hash)
@@ -648,8 +694,8 @@ impl RrdpServer {
                 let xml = delta.xml();
                 let hash = HexEncodedHash::from_content(xml.as_slice());
 
-                let delta_uri = self.delta_uri(serial);
-                let delta_path = self.delta_path(serial);
+                let delta_uri = delta.uri(&self.rrdp_base_uri);
+                let delta_path = delta.path(&self.rrdp_base_dir);
                 let file_ref = FileRef::new(delta_uri, delta_path, hash);
                 DeltaRef::new(serial, file_ref)
             })
@@ -669,14 +715,14 @@ impl RrdpServer {
     /// no longer referenced in the notification file.
     fn write(&self, config: &RepositoryRetentionConfig) -> Result<(), Error> {
         // write snapshot if it's not there
-        let snapshot_path = self.snapshot_path(self.serial);
+        let snapshot_path = self.snapshot.path(&self.rrdp_base_dir);
         if !snapshot_path.exists() {
             self.snapshot.write_xml(&snapshot_path)?;
         }
 
         // write deltas if they are not there
         for delta in &self.deltas {
-            let path = self.delta_path(delta.serial());
+            let path = delta.path(&self.rrdp_base_dir);
             if !path.exists() {
                 // assume that if the delta exists, it is correct
                 delta.write_xml(&path)?;
@@ -760,7 +806,7 @@ impl RrdpServer {
                 // Skip the current serial
                 if serial == self.serial {
                     continue;
-                // Clean up old serial dirs
+                // Clean up old serial dirs once deltas are out of scope
                 } else if !self.notification.includes_delta(serial)
                     && !self.old_notifications.iter().any(|n| n.includes_delta(serial))
                 {
@@ -779,16 +825,25 @@ impl RrdpServer {
                     } else {
                         let _best_effort_rm = fs::remove_file(path);
                     }
-
-                // clean snapshots no longer referenced in retained notification files
-                // unless archiving mode is enabled -- in that case leave the snapshots
-                // and they will be archived when the delta go out of scope (see above)
-                } else if !self.old_notifications.iter().any(|n| n.includes_snapshot(serial))
-                    && !config.retention_archive
+                // We still need this old serial dir for the delta, but may not need the snapshot
+                // in it unless archiving is enabled.. in that case leave them and move them when
+                // the complete serial dir goes out of scope above.
+                } else if !config.retention_archive
+                    && !self
+                        .old_notifications
+                        .iter()
+                        .any(|old_notification| old_notification.includes_snapshot(serial))
                 {
-                    let snapshot_path = Self::new_snapshot_path(&self.rrdp_base_dir, &self.session, serial);
-                    if snapshot_path.exists() {
-                        let _best_effort_rm = fs::remove_file(snapshot_path);
+                    // see if the there is a snapshot file in this serial dir and if so do a best
+                    // effort removal.
+                    if let Ok(Some(snapshot_file_to_remove)) = Self::session_dir_snapshot(&session_dir, serial) {
+                        // snapshot files are stored under their own unique random dir, e.g:
+                        // <session_dir>/<serial>/<random>/snapshot.xml
+                        //
+                        // So also remove the otherwise empty parent directory.
+                        if let Some(snapshot_parent_dir) = snapshot_file_to_remove.parent() {
+                            let _ = fs::remove_dir_all(snapshot_parent_dir);
+                        }
                     }
                 } else {
                     // we still need this
@@ -815,7 +870,7 @@ impl RrdpServer {
 ///
 impl RrdpServer {
     pub fn notification_uri(&self) -> uri::Https {
-        self.rrdp_base_uri.join(b"notification.xml")
+        self.rrdp_base_uri.join(b"notification.xml").unwrap()
     }
 
     fn notification_path_new(&self) -> PathBuf {
@@ -830,45 +885,44 @@ impl RrdpServer {
         path
     }
 
-    fn snapshot_rel(session: &RrdpSession, serial: u64) -> String {
-        format!("{}/{}/snapshot.xml", session, serial)
+    pub fn session_dir_snapshot(session_path: &Path, serial: u64) -> KrillResult<Option<PathBuf>> {
+        Self::find_in_serial_dir(session_path, serial, "snapshot.xml")
     }
 
-    fn new_snapshot_path(base: &Path, session: &RrdpSession, serial: u64) -> PathBuf {
-        let mut path = base.to_path_buf();
-        path.push(Self::snapshot_rel(session, serial));
-        path
-    }
-
-    fn snapshot_path(&self, serial: u64) -> PathBuf {
-        Self::new_snapshot_path(&self.rrdp_base_dir, &self.session, serial)
-    }
-
-    fn new_snapshot_uri(base: &uri::Https, session: &RrdpSession, serial: u64) -> uri::Https {
-        base.join(Self::snapshot_rel(session, serial).as_ref())
-    }
-
-    fn snapshot_uri(&self) -> uri::Https {
-        Self::new_snapshot_uri(&self.rrdp_base_uri, &self.session, self.serial)
-    }
-
-    fn delta_rel(session: &RrdpSession, serial: u64) -> String {
-        format!("{}/{}/delta.xml", session, serial)
-    }
-
-    fn delta_uri(&self, serial: u64) -> uri::Https {
-        uri::Https::from_string(format!(
-            "{}{}",
-            self.rrdp_base_uri.to_string(),
-            Self::delta_rel(&self.session, serial)
-        ))
-        .unwrap() // Cannot fail. Config checked at startup.
-    }
-
-    fn delta_path(&self, serial: u64) -> PathBuf {
-        let mut path = self.rrdp_base_dir.clone();
-        path.push(Self::delta_rel(&self.session, serial));
-        path
+    /// Expects files (like delta.xml or snapshot.xml) under dir structure like:
+    /// <session_path>/<serial>/<some random>/<filename>
+    pub fn find_in_serial_dir(session_path: &Path, serial: u64, filename: &str) -> KrillResult<Option<PathBuf>> {
+        let serial_dir = session_path.join(serial.to_string());
+        if let Ok(randoms) = fs::read_dir(&serial_dir) {
+            for entry in randoms {
+                let entry = entry.map_err(|e| {
+                    Error::io_error_with_context(
+                        format!(
+                            "Could not open directory entry under RRDP directory {}",
+                            serial_dir.to_string_lossy()
+                        ),
+                        e,
+                    )
+                })?;
+                if let Ok(files) = fs::read_dir(entry.path()) {
+                    for file in files {
+                        let file = file.map_err(|e| {
+                            Error::io_error_with_context(
+                                format!(
+                                    "Could not open directory entry under RRDP directory {}",
+                                    entry.path().to_string_lossy()
+                                ),
+                                e,
+                            )
+                        })?;
+                        if file.file_name().to_string_lossy() == filename {
+                            return Ok(Some(file.path()));
+                        }
+                    }
+                }
+            }
+        }
+        Ok(None)
     }
 }
 
@@ -931,9 +985,13 @@ impl RepositoryAccessProxy {
     }
 
     fn read(&self) -> KrillResult<Arc<RepositoryAccess>> {
-        self.store
-            .get_latest(&self.key)
-            .map_err(|e| Error::custom(format!("Publication Server data missing: {}", e)))
+        if !self.initialized()? {
+            Err(Error::RepositoryServerNotInitialized)
+        } else {
+            self.store
+                .get_latest(&self.key)
+                .map_err(|e| Error::custom(format!("Publication Server data issue: {}", e)))
+        }
     }
 
     pub fn publishers(&self) -> KrillResult<Vec<PublisherHandle>> {
@@ -945,16 +1003,20 @@ impl RepositoryAccessProxy {
     }
 
     pub fn add_publisher(&self, req: rfc8183::PublisherRequest, actor: &Actor) -> KrillResult<()> {
-        let base_uri = self.read()?.base_uri_for(req.publisher_handle())?;
+        let base_uri = self.read()?.base_uri_for(req.publisher_handle())?; // will verify that server was initialized
         let cmd = RepoAccessCmdDet::add_publisher(&self.key, req, base_uri, actor);
         self.store.command(cmd)?;
         Ok(())
     }
 
     pub fn remove_publisher(&self, name: PublisherHandle, actor: &Actor) -> KrillResult<()> {
-        let cmd = RepoAccessCmdDet::remove_publisher(&self.key, name, actor);
-        self.store.command(cmd)?;
-        Ok(())
+        if !self.initialized()? {
+            Err(Error::RepositoryServerNotInitialized)
+        } else {
+            let cmd = RepoAccessCmdDet::remove_publisher(&self.key, name, actor);
+            self.store.command(cmd)?;
+            Ok(())
+        }
     }
 
     /// Returns the repository URI information for a publisher.
@@ -973,7 +1035,7 @@ impl RepositoryAccessProxy {
 
     /// Parse submitted bytes by a Publisher as an RFC8181 ProtocolCms object, and validates it.
     pub fn validate(&self, publisher: &PublisherHandle, msg: Bytes) -> KrillResult<ProtocolCms> {
-        let publisher = self.get_publisher(&publisher)?;
+        let publisher = self.get_publisher(publisher)?;
         let msg = ProtocolCms::decode(msg, false).map_err(|e| Error::Rfc8181Decode(e.to_string()))?;
         msg.validate(publisher.id_cert()).map_err(Error::Rfc8181Validation)?;
         Ok(msg)
@@ -1101,7 +1163,7 @@ impl RepositoryAccess {
     }
 
     fn notification_uri(&self) -> uri::Https {
-        self.rrdp_base.join(b"notification.xml")
+        self.rrdp_base.join(b"notification.xml").unwrap()
     }
 
     fn base_uri_for(&self, name: &PublisherHandle) -> KrillResult<uri::Rsync> {
