@@ -6,23 +6,19 @@ use std::{fmt, path::Path, str::FromStr, sync::Arc};
 
 use serde::de::DeserializeOwned;
 
-use crate::commons::error::KrillIoError;
-use crate::constants::KRILL_VERSION;
-use crate::daemon::ca::CertAuth;
-use crate::pubd::RepositoryAccess;
-use crate::{commons::api::Handle, daemon::config::Config};
 use crate::{
     commons::{
+        api::Handle,
         crypto::KrillSigner,
-        eventsourcing::{
-            AggregateStore, AggregateStoreError, CommandKey, KeyStoreKey, KeyStoreVersion, KeyValueError, KeyValueStore,
-        },
+        error::KrillIoError,
+        eventsourcing::{AggregateStoreError, CommandKey, KeyStoreKey, KeyValueError, KeyValueStore},
+        util::{file, KrillVersion},
     },
+    daemon::{config::Config, krillserver::KrillServer},
     pubd::RepositoryManager,
+    upgrades::v0_9_0::{CaObjectsMigration, PubdObjectsMigration},
 };
 
-use self::v0_9_0::{CaObjectsMigration, PubdObjectsMigration};
-
 pub mod v0_9_0;
 
 pub type UpgradeResult<T> = Result<T, UpgradeError>;
@@ -32,13 +28,13 @@ pub const MIGRATION_SCOPE: &str = "migration";
 //------------ UpgradeError --------------------------------------------------
 
 #[derive(Debug)]
+#[allow(clippy::large_enum_variant)]
 pub enum UpgradeError {
     AggregateStoreError(AggregateStoreError),
     KeyStoreError(KeyValueError),
     IoError(KrillIoError),
     Unrecognised(String),
     CannotLoadAggregate(Handle),
-    KrillError(crate::commons::error::Error),
     Custom(String),
 }
 
@@ -50,7 +46,6 @@ impl fmt::Display for UpgradeError {
             UpgradeError::IoError(e) => e.fmt(f),
             UpgradeError::Unrecognised(s) => write!(f, "Unrecognised command summary: {}", s),
             UpgradeError::CannotLoadAggregate(handle) => write!(f, "Cannot load: {}", handle),
-            UpgradeError::KrillError(e) => e.fmt(f),
             UpgradeError::Custom(s) => s.fmt(f),
         }
     }
@@ -85,7 +80,7 @@ impl From<KrillIoError> for UpgradeError {
 
 impl From<crate::commons::error::Error> for UpgradeError {
     fn from(e: crate::commons::error::Error) -> Self {
-        UpgradeError::KrillError(e)
+        UpgradeError::Custom(e.to_string())
     }
 }
 
@@ -98,7 +93,7 @@ pub trait UpgradeStore {
     fn needs_migrate(&self) -> Result<bool, UpgradeError>;
     fn migrate(&self) -> Result<(), UpgradeError>;
 
-    fn version_before(kv: &KeyValueStore, later: KeyStoreVersion) -> Result<bool, UpgradeError> {
+    fn version_before(kv: &KeyValueStore, later: KrillVersion) -> Result<bool, UpgradeError> {
         kv.version_is_before(later).map_err(UpgradeError::KeyStoreError)
     }
 
@@ -152,7 +147,7 @@ pub trait UpgradeStore {
 
     fn archive_to_migration_scope(&self, key: &KeyStoreKey) -> Result<(), UpgradeError> {
         self.store()
-            .archive_to(&key, MIGRATION_SCOPE)
+            .archive_to(key, MIGRATION_SCOPE)
             .map_err(UpgradeError::KeyStoreError)
     }
 
@@ -164,51 +159,67 @@ pub trait UpgradeStore {
 
 /// Should be called when Krill starts, before the KrillServer is initiated
 pub fn pre_start_upgrade(config: Arc<Config>) -> Result<(), UpgradeError> {
-    upgrade_0_9_0(config)
+    upgrade_data_to_0_9_0(config)
+}
+
+/// Should be called when the KrillServer is initiated, before the webserver is started
+/// and operators can make changes.
+pub async fn post_start_upgrade(config: &Config, server: &KrillServer) -> Result<(), UpgradeError> {
+    if needs_upgrade(&config.data_dir, "cas", KrillVersion::candidate(0, 9, 3, 2)) {
+        info!("Reissue ROAs on upgrade to force short EE certificate subjects in the objects");
+        server.force_renew_roas().await.map_err(|e| e.into())
+    } else {
+        Ok(())
+    }
 }
 
 pub async fn update_storage_version(work_dir: &Path) -> Result<(), UpgradeError> {
-    let current = KeyStoreVersion::current();
+    let current = KrillVersion::current();
 
-    let mut ca_dir = work_dir.to_path_buf();
-    ca_dir.push("cas");
-    if ca_dir.exists() {
-        let ca_store: AggregateStore<CertAuth> = AggregateStore::disk(work_dir, "cas")?;
-        if ca_store.get_version()? != current {
-            ca_store.set_version(&current)?;
-        }
+    if needs_v0_9_0_upgrade(work_dir, "cas") {
+        debug!("Updating version file for cas");
+        file::save_json(&current, &work_dir.join("cas/version"))?;
     }
 
-    let mut pubd_dir = work_dir.to_path_buf();
-    pubd_dir.push("pubd");
-    if pubd_dir.exists() {
-        let pubd_store: AggregateStore<RepositoryAccess> = AggregateStore::disk(work_dir, "pubd")?;
-        if pubd_store.get_version()? != current {
-            pubd_store.set_version(&current)?;
-        }
+    if needs_v0_9_0_upgrade(work_dir, "pubd") {
+        debug!("Updating version file for pubd");
+        file::save_json(&current, &work_dir.join("pubd/version"))?;
     }
 
-    info!("Upgraded Krill to version: {}", KRILL_VERSION);
     Ok(())
 }
 
-fn upgrade_0_9_0(config: Arc<Config>) -> Result<(), UpgradeError> {
-    let mut pubd_dir = config.data_dir.clone();
-    pubd_dir.push("pubd");
-    if pubd_dir.exists() {
+fn upgrade_data_to_0_9_0(config: Arc<Config>) -> Result<(), UpgradeError> {
+    let work_dir = &config.data_dir;
+    if needs_v0_9_0_upgrade(work_dir, "pubd") {
         PubdObjectsMigration::migrate(config.clone())?;
     }
-    let signer = Arc::new(KrillSigner::build(&config.data_dir)?);
-    let repo_manager = RepositoryManager::build(config.clone(), signer)?;
 
-    let mut cas_dir = config.data_dir.clone();
-    cas_dir.push("cas");
-    if cas_dir.exists() {
+    if needs_v0_9_0_upgrade(work_dir, "cas") {
+        let signer = Arc::new(KrillSigner::build(work_dir)?);
+        let repo_manager = RepositoryManager::build(config.clone(), signer)?;
+
         CaObjectsMigration::migrate(config, repo_manager)?;
     }
+
     Ok(())
 }
 
+fn needs_v0_9_0_upgrade(work_dir: &Path, ns: &str) -> bool {
+    needs_upgrade(work_dir, ns, KrillVersion::release(0, 9, 0))
+}
+
+fn needs_upgrade(work_dir: &Path, ns: &str, before: KrillVersion) -> bool {
+    let keystore_path = work_dir.join(ns);
+    if keystore_path.exists() {
+        let version_path = keystore_path.join("version");
+        let version_found = file::load_json(&version_path).unwrap_or_else(|_| KrillVersion::v0_5_0_or_before());
+        version_found < before
+    } else {
+        false
+    }
+}
+
 //------------ Tests ---------------------------------------------------------
 
 #[cfg(test)]
@@ -227,10 +238,10 @@ mod tests {
         let source = PathBuf::from("test-resources/migrations/v0_8_1/");
         file::backup_dir(&source, &work_dir).unwrap();
 
-        let config = Arc::new(Config::test(&work_dir, false));
+        let config = Arc::new(Config::test(&work_dir, false, false, false));
         let _ = config.init_logging();
 
-        upgrade_0_9_0(config).unwrap();
+        upgrade_data_to_0_9_0(config).unwrap();
 
         let _ = fs::remove_dir_all(work_dir);
     }
@@ -241,10 +252,10 @@ mod tests {
         let source = PathBuf::from("test-resources/migrations/v0_6_0/");
         file::backup_dir(&source, &work_dir).unwrap();
 
-        let config = Arc::new(Config::test(&work_dir, false));
+        let config = Arc::new(Config::test(&work_dir, false, false, false));
         let _ = config.init_logging();
 
-        upgrade_0_9_0(config).unwrap();
+        upgrade_data_to_0_9_0(config).unwrap();
 
         let _ = fs::remove_dir_all(work_dir);
     }
