@@ -1,37 +1,38 @@
 use std::{collections::HashMap, str::FromStr, sync::Arc};
 
-use api::{RepositoryContact, StorableCaCommand, StoredEffect};
-use ca::{CaEvt, IniDet, StoredCaCommand};
-use rpki::{crl::Crl, crypto::KeyIdentifier, manifest::Manifest, uri, x509::Time};
+use chrono::Duration;
+
+use rpki::{
+    repository::{crl::Crl, crypto::KeyIdentifier, manifest::Manifest, x509::Time},
+    uri,
+};
 
 use crate::{
     commons::{
         api::{
-            self, ChildHandle, Handle, HexEncodedHash, IssuanceRequest, IssuedCert, ObjectName, ParentHandle, RcvdCert,
-            RepoInfo, ResourceClassName, ResourceSet, Revocation, RevocationRequest, Revocations, RoaAggregateKey,
-            TaCertDetails,
+            ChildHandle, Handle, HexEncodedHash, IssuanceRequest, IssuedCert, ObjectName, ParentHandle, RcvdCert,
+            RepoInfo, RepositoryContact, ResourceClassName, ResourceSet, Revocation, RevocationRequest, Revocations,
+            RoaAggregateKey, StorableCaCommand, StoredEffect, TaCertDetails,
         },
         crypto::{IdCert, KrillSigner},
-        eventsourcing::{
-            Aggregate, AggregateStore, CommandKey, KeyStoreKey, KeyStoreVersion, KeyValueStore, StoredValueInfo,
-        },
+        eventsourcing::{Aggregate, AggregateStore, CommandKey, KeyStoreKey, KeyValueStore, StoredValueInfo},
         remote::rfc8183,
+        util::KrillVersion,
     },
-    constants::CASERVER_DIR,
+    constants::{CASERVER_DIR, KRILL_VERSION},
     daemon::{
         ca::{
-            self, ta_handle, BasicKeyObjectSet, CaEvtDet, CaObjects, CaObjectsStore, CurrentKeyObjectSet,
-            PublishedCert, PublishedRoa, ResourceClassKeyState, ResourceClassObjects, RouteAuthorization,
+            self, ta_handle, BasicKeyObjectSet, CaEvt, CaEvtDet, CaObjects, CaObjectsStore, CurrentKeyObjectSet,
+            IniDet, PublishedCert, PublishedRoa, ResourceClassKeyState, ResourceClassObjects, RouteAuthorization,
+            StoredCaCommand,
         },
         config::Config,
     },
     pubd::RepositoryManager,
-    upgrades::{UpgradeError, UpgradeResult, UpgradeStore},
+    upgrades::v0_9_0::{old_commands::*, old_events::*},
+    upgrades::{UpgradeError, UpgradeResult, UpgradeStore, MIGRATION_SCOPE},
 };
 
-use super::super::MIGRATION_SCOPE;
-use super::{old_commands::*, old_events::*};
-
 /// Migrate the current objects for each CA into the CaObjectStore
 pub struct CaObjectsMigration;
 
@@ -43,9 +44,9 @@ impl CaObjectsMigration {
 
         let signer = Arc::new(KrillSigner::build(&config.data_dir)?);
 
-        if store.version_is_before(KeyStoreVersion::V0_6)? {
+        if store.version_is_before(KrillVersion::release(0, 6, 0))? {
             Err(UpgradeError::custom("Cannot upgrade Krill installations from before version 0.6.0. Please upgrade to any version ranging from 0.6.0 to 0.8.1 first, and then upgrade to this version."))
-        } else if store.version_is_before(KeyStoreVersion::V0_9_0_RC1)? {
+        } else if store.version_is_before(KrillVersion::candidate(0, 9, 0, 1))? {
             info!("Krill version is older than 0.9.0-RC1, will now upgrade data structures.");
 
             // Populate object store which will contain all objects produced by CAs, while we are
@@ -178,7 +179,7 @@ impl UpgradeStore for CasStoreMigration {
             // event, based on the first recorded time in command keys.
             let time_for_init_command = match cmd_keys.first() {
                 Some(first_command) => {
-                    let old_cmd: OldStoredCaCommand = self.get(&first_command)?;
+                    let old_cmd: OldStoredCaCommand = self.get(first_command)?;
                     old_cmd.time
                 }
                 None => Time::now(),
@@ -243,21 +244,37 @@ impl UpgradeStore for CasStoreMigration {
             info!("Will migrate {} commands for CA {}", cmd_keys.len(), scope);
 
             let mut total_migrated = 0;
+            let time_started = Time::now();
+            let total_commands = cmd_keys.len();
+
             for cmd_key in cmd_keys {
                 // Do the migration counter first, so that we can just call continue when we need to skip commands
                 total_migrated += 1;
                 if total_migrated % 100 == 0 {
-                    info!("  migrated {} commands", total_migrated);
+                    // ETA:
+                    //  - (total_migrated / (now - started)) * total
+                    let mut time_passed = (Time::now().timestamp() - time_started.timestamp()) as usize;
+                    if time_passed == 0 {
+                        time_passed = 1; // avoid divide by zero.. we are doing approximate estimates here
+                    }
+                    let migrated_per_second: f64 = total_migrated as f64 / time_passed as f64;
+                    let expected_seconds = (total_commands as f64 / migrated_per_second) as i64;
+                    let eta = time_started + Duration::seconds(expected_seconds);
+                    info!(
+                        "  migrated {} commands, expect to finish: {}",
+                        total_migrated,
+                        eta.to_rfc3339()
+                    );
                 }
 
-                debug!("  command: {}", cmd_key);
+                trace!("  command: {}", cmd_key);
                 let mut old_cmd: OldStoredCaCommand = self.get(&cmd_key)?;
 
                 if let Some(evt_versions) = old_cmd.effect.events() {
                     let mut events = vec![];
                     for v in evt_versions {
-                        let migration_event_key = Self::event_key(&&migration_scope, *v);
-                        debug!("  +- event: {}", migration_event_key);
+                        let migration_event_key = Self::event_key(&migration_scope, *v);
+                        trace!("  +- event: {}", migration_event_key);
                         let old_evt: OldCaEvt = self.store.get(&migration_event_key)?.ok_or_else(|| {
                             UpgradeError::Custom(format!("Cannot parse old event: {}", migration_event_key))
                         })?;
@@ -313,17 +330,20 @@ impl UpgradeStore for CasStoreMigration {
         //    --> clean up the archived commands and events
         //    --> restore to previous version of Krill is not supported
         for scope in self.store.scopes()? {
-            info!("Check state of '{}' before cleanup.", scope);
+            info!("Will rebuild CA '{}' from events and warm up the cache", scope);
+
             let ca =
                 Handle::from_str(&scope).map_err(|e| UpgradeError::Custom(format!("Found invalid ca name: {}", e)))?;
 
-            self.ca_store.get_latest(&ca).map_err(|e| {
-                UpgradeError::Custom(format!("Could not rebuild CA '{}' after migration: {}", scope, e))
-            })?;
+            self.ca_store
+                .warm_aggregate(&ca)
+                .map_err(|e| UpgradeError::Custom(format!("Could not rebuild CA '{}' after migration: {}", ca, e)))?;
 
             self.drop_migration_scope(&scope)?;
         }
 
+        info!("Done migration CAs to Krill version {}", KRILL_VERSION);
+
         Ok(())
     }
 
@@ -707,34 +727,27 @@ impl OldResourceClass {
         roas: HashMap<ObjectName, PublishedRoa>,
         certs: HashMap<ObjectName, PublishedCert>,
     ) -> CurrentKeyObjectSet {
-        let current_set = key.current_set.clone();
-
-        let mft = Manifest::decode(current_set.manifest_info.current.content().to_bytes(), true).unwrap();
-        let crl = Crl::decode(current_set.crl_info.current.content().to_bytes()).unwrap();
-
-        CurrentKeyObjectSet::new(
-            key.incoming_cert.clone(),
-            current_set.number,
-            current_set.revocations,
-            mft.into(),
-            crl.into(),
-            roas,
-            certs,
-        )
+        let basic = Self::object_set_for_certified_key(key);
+        CurrentKeyObjectSet::new(basic, roas, HashMap::new(), certs)
     }
 
     fn object_set_for_certified_key(key: &OldCertifiedKey) -> BasicKeyObjectSet {
         let current_set = key.current_set.clone();
 
-        let mft = Manifest::decode(current_set.manifest_info.current.content().to_bytes(), true).unwrap();
-        let crl = Crl::decode(current_set.crl_info.current.content().to_bytes()).unwrap();
+        let manifest = Manifest::decode(current_set.manifest_info.current.content().to_bytes(), true)
+            .unwrap()
+            .into();
+
+        let crl = Crl::decode(current_set.crl_info.current.content().to_bytes())
+            .unwrap()
+            .into();
 
         BasicKeyObjectSet::new(
             key.incoming_cert.clone(),
             current_set.number,
             current_set.revocations,
-            mft.into(),
-            crl.into(),
+            manifest,
+            crl,
         )
     }
 
